#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2023 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

# The init() function initialize the required variables.
init(){
    opcache_ini="/opt/cyops/configs/php/10-opcache.ini"
    opcache_values="opcache.memory_consumption=256 \
                    opcache.max_accelerated_files=20000 \
                    opcache.validate_timestamps=0 \
                    opcache.preload=/opt/cyops-api/config/preload.php"
    php_ini="/opt/cyops/configs/php/php.ini"
    cache_values="realpath_cache_size=4096K realpath_cache_ttl=600"
}

#
# The update_opcache_ini() function added the following parameter and it's value even if user change it.
# opcache.memory_consumption=256 opcache.max_accelerated_files=20000 opcache.validate_timestamps=0 opcache.preload=/opt/cyops-api/config/preload.php
#
update_ini(){
   #
   # The shift command shift the single argument to the left for first single parameter (variable) to accept value in ini_file variable
   # and allow next input array in param_array variable. We can not pass the single variable and array at a time to any function.
   #
   local ini_file="$1" && shift
   # The following variable accept the array values
   local param_array=($@)
   local parameter=""
   local param_value=""
   local check_param_flag=""

   if [ ! -f $ini_file ]; then
       echo "The $ini_file file does not exist"
       return
   fi

   for key_value in ${param_array[@]}
   do
       # The following command get parameter which is before = symbol
       parameter=$(echo "$key_value" | cut -d"=" -f1)

       # The following command get parameter value which is after = symbol
       param_value=$(echo "$key_value" | cut -d"=" -f2)

       # The following command search for word start from (which is ^) and ignore if same word commented (which is start with ;)
       check_param_flag=$(grep -E "^\s*$parameter\s*=" $ini_file)

       if [ $? -eq 0 ]; then
           if ! echo $check_param_flag | grep -qw "$param_value"; then
               check_param_flag=$(echo "$check_param_flag" | sed  -e 's:/:\\\/:g')
               key_value=$(echo "$key_value" | sed  -e 's:/:\\\/:g')
               sed -i "s/$check_param_flag/$key_value/g" $ini_file
           fi
       else
           #
           # If parameter and it's value not found then add parameter-value to EOF.
           # Sometime in configuration file does not have new line and it is truncated.
           # Handle the empty line before append the value in the file.
           #
           echo "" >> $ini_file
           echo -e "\n$key_value" >> $ini_file
       fi
   done
}

####Main####
init
update_ini $opcache_ini ${opcache_values[@]}
update_ini $php_ini ${cache_values[@]}