<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Backup\Core\FolderBackupProcessor;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Command\LockableTrait;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\Exception\IOException;

class BackupAllCommand extends Command
{
    use LockableTrait;

    protected $folderBackupProcessor;

    public function __construct(FolderBackupProcessor $folderBackupProcessor)
    {
        $this->folderBackupProcessor = $folderBackupProcessor;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:backup:all')
            ->setAliases(['cybersponse:backup:all'])
            ->setDescription('Performs a Entity directory backup.');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        try {
            $this->folderBackupProcessor->backup();
        } catch (IOException | \Exception $exception) {
            $io->error($exception->getMessage());
            return $exception->getCode();
        }

        $io->success('Finished with system backup!');

        return Command::SUCCESS;
    }
}
