<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use Exception;
use App\Entity\ModelMetadata\ClusterCommand;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class ClusterPublishWaitCommand extends Command
{
    protected $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:cluster:publish:wait')
            ->setAliases(['cybersponse:cluster:publish:wait'])
            ->setDescription('Waiting for Model File Generation on the other nodes.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $errorIo = $io;
        $sequenceId = mt_rand(1111111111, 9999999999);
        $dasUri = $this->container->getParameter('das.uri');
        $dasClient = $this->container->get("app.proxy.client.das");
        $objectData = [
            'commandId' => 'cluster_publish',
            'command' => 'sudo -u nginx php /opt/cyops-api/bin/console app:cluster:publish --no-interaction --nodeId=$nodeId$ --sequenceId=' . $sequenceId,
            'includeFiredrillNodes' => false
        ];
        $headers =["Content-type" => "application/json"];
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode != 200) {
                return -1;
            }
            $dasRequests = new GuzzleRequest('get', $dasUri . '/cluster/?self=false', $headers);
            $nodeIds = [];
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData);
            $nodes = $jsonDecodeResponse->{"nodes"};
            foreach ($nodes as $node) {
                if (strpos(strtolower($node->{"comment"}), "firedrill in progress") === FALSE) {
                    array_push($nodeIds, $node->{"nodeId"});
                }
            }
        } catch (BadRequestHttpException | \Exception $error) {
            $errorIo->error($error->getMessage());
            return -1;
        }

        $entityManager = $this->container->get('doctrine.orm.entity_manager');
        foreach ($nodeIds as &$nodeId) {
            $clusterCommand = new ClusterCommand();
            $clusterCommand->setNodeId($nodeId);
            $clusterCommand->setCommandId("cybersponse:cluster:publish");
            $clusterCommand->setSequenceId($sequenceId);
            $clusterCommand->setStatus('todo');
            $entityManager->persist($clusterCommand);
        }
        $entityManager->flush();

        $clusterCommandRepository = $entityManager->getRepository(ClusterCommand::class);
        # wait for 10 mins for other cluster nodes to complete
        $counter = 60;
        $waitForStatus = count($clusterCommandRepository->findAll([
            'sequenceId' => $sequenceId,
            'status' => 'todo'
        ]));
        while ($waitForStatus > 0 && $counter > 0) {
            sleep(10);
            $counter = $counter - 1;
            $waitForStatus = count($clusterCommandRepository->findAll([
                'sequenceId' => $sequenceId,
                'status' => 'todo'
            ]));
        }
        $io->success('Finished with publish across cluster!');
        return Command::SUCCESS;
    }
}
