<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use Exception;
use InvalidArgumentException;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class DropMetadataCopyCommand extends Command
{
    protected $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:drop:metadata:copy')
            ->setAliases(['cybersponse:drop:metadata:copy'])
            ->setDescription('Drop Model Meta Data and Attributes Metadata Copies');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');

        $commandsToExecute = [
            "DROP TABLE IF EXISTS attrib_model_metadata_copy",
            "DROP TABLE IF EXISTS model_meta_copy",
        ];

        foreach ($commandsToExecute as $commandToExecute) {
            $metadataEntityDatabase = $metadataEntityManager->getConnection()->prepare($commandToExecute);
            try {
                $metadataEntityDatabase->execute();
            } catch (InvalidArgumentException | \Exception $e) {
                throw new \Exception('Failure: ' . $e->getMessage());
            }
        }

        $io->success('Successfully Removed ModelMetaData Copy and Attribute Metadata Copy');
        return Command::SUCCESS;
    }
}
