<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Constants\CacheConstants;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Cache\Adapter\FilesystemTagAwareAdapter;

class LogoutTokenCacheCommand extends Command
{
    protected $container;

    private $cache;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        $cacehDir = sprintf('%s/var/security', $this->container->getParameter('kernel.project_dir'));
        $this->cache = new FilesystemTagAwareAdapter('', 172800, $cacehDir);
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:logout:token')
            ->setAliases(['cybersponse:logout:token'])
            ->setDescription('Add logout token to cache')
            ->addArgument('tokens', InputArgument::IS_ARRAY, 'Tokens to be added to logout cache pool');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $tokens = $input->getArgument('tokens');
        if (count($tokens) > 0) {
            foreach ($tokens as $token) {
                if (strpos($token, 'Bearer') == 0) {
                    $timeout = 7200;
                } else {
                    $timeout = 172800;
                }

                $cacheKey = $this->cache->getItem($token);
                if (!$cacheKey->isHit()) {
                    $cacheKey->set($token);
                    $cacheKey->expiresAfter($timeout);
                    $cacheKey->tag(CacheConstants::LOGOUT_TOKEN);
                    $this->cache->save($cacheKey);
                }
            }
        }
        $io->success('Logout token added');
        return Command::SUCCESS;
    }
}
