<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use InvalidArgumentException;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MetadataActivateCommand extends Command
{
    protected $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:metadata:activate')
            ->setAliases(['cybersponse:metadata:activate'])
            ->setDescription('Activates staging model metadata by copying it to model metadata.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');

        $commandsToExecute = [
            "CREATE TABLE IF NOT EXISTS attrib_model_metadata_copy (LIKE attrib_model_metadata INCLUDING ALL)",
            "CREATE TABLE IF NOT EXISTS model_meta_copy (LIKE model_metadatas INCLUDING ALL)",

            "DELETE from attrib_model_metadata_copy",
            "DELETE from model_meta_copy",

            "INSERT into model_meta_copy select * from model_metadatas",
            "INSERT into attrib_model_metadata_copy select * from attrib_model_metadata",

            "DELETE from attrib_model_metadata",
            "INSERT into model_metadatas select * from staging_model_metadatas ON CONFLICT (uuid) DO UPDATE set uniqueconstraint=EXCLUDED.uniqueconstraint, displayname=EXCLUDED.displayname, tablename=EXCLUDED.tablename, ownable=EXCLUDED.ownable, taggable=EXCLUDED.taggable, trackable=EXCLUDED.trackable, peerreplicable=EXCLUDED.peerreplicable, defaultsort=EXCLUDED.defaultsort, indexable=EXCLUDED.indexable, writable=EXCLUDED.writable, descriptions=EXCLUDED.descriptions, userownable=EXCLUDED.userownable, queueable=EXCLUDED.queueable, module=EXCLUDED.module, system=EXCLUDED.system, softdeleteable=EXCLUDED.softdeleteable, archivable=EXCLUDED.archivable, paused=EXCLUDED.paused, partitionby=EXCLUDED.partitionby, archivalcriteria=EXCLUDED.archivalcriteria, archivalfilters=EXCLUDED.archivalfilters, importedBy=EXCLUDED.importedBy, replicationFilters=EXCLUDED.replicationFilters, enableReplication=EXCLUDED.enableReplication",
            "INSERT into attrib_model_metadata select * from attribute_metadatas",
            "DELETE from model_metadatas where uuid not in (select uuid from staging_model_metadatas)",
            "DELETE from attrib_model_metadata where sattrib is null",
            "DELETE from attribute_metadatas where sattrib is null"
        ];

        foreach ($commandsToExecute as $commandToExecute) {
            $metadataEntityDatabase = $metadataEntityManager->getConnection()->prepare($commandToExecute);
            try {
                $metadataEntityDatabase->execute();
            } catch (InvalidArgumentException | \Exception $e) {
                throw new \Exception('Failure: ' . $e->getMessage());
            }
        }

        $io->success('Metadata has been successfully activated from StagingModelMetadata to ModelMetadata.');
        return Command::SUCCESS;
    }
}
