<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Entity\Core\Module;
use App\Entity\ModelMetadata\ModelMetadata;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use ApiPlatform\Core\Metadata\Resource\Factory\ResourceNameCollectionFactoryInterface;

class MetadataGenerateModulesCommand extends Command
{
    protected $container;

    protected $resourceNameCollectionFactory;

    protected $resourceMetadataFactory;

    public function __construct(ContainerInterface $container, ResourceNameCollectionFactoryInterface $resourceNameCollectionFactory)
    {
        $this->container = $container;
        $this->resourceNameCollectionFactory = $resourceNameCollectionFactory;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:metadata:generate:modules')
            ->setAliases(['cybersponse:metadata:generate:modules'])
            ->setDescription('Process current resources in api and generate modules.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $progress = $io->createProgressBar();
        $progress->setFormat(' %current%/%max% [%bar%] Working on %message%...');
        $progress->setBarWidth(50);

        /** @var EntityManager $entityManager */
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Module::class);
        $moduleRepository = $entityManager->getRepository(Module::class);
        $queryBuilder = $moduleRepository->createQueryBuilder(Module::class);

        $modelMetadataRepository = $entityManager->getRepository(ModelMetadata::class);
        $modelMetaData = $modelMetadataRepository->findAll();

        $progress->start(count($modelMetaData));

        $allModuleTypes = [];

        /** @var ModelAwareResource $resource */
        foreach ($modelMetaData as $modelClass) {
            $moduleType = $modelClass->getType();
            $progress->setMessage($moduleType);

            $module = $moduleRepository->findOneBy(['type' => $moduleType]);
            $allModuleTypes[] = $moduleType;

            $descriptions = $modelClass->getDescriptions();
            $name = $moduleType;
            if (array_key_exists('plural', $descriptions)) {
                $name = $descriptions['plural'];
            }

            if (is_null($module)) {
                $module = new Module();
                $module->setType($moduleType);
            }

            $module
                ->setName($name)
                ->setDescription(sprintf('Allows interactions with the %s Module', $name))
                ->setSystem(false);
            $entityManager->persist($module);
            $entityManager->flush();

            $progress->advance();
        }

        if (count($allModuleTypes)) {
            $query = $queryBuilder
                ->delete(Module::class, 'm')
                ->where($queryBuilder->expr()->notIn('m.type', $allModuleTypes))
                ->andWhere('m.system = :system')
                ->setParameter('system', false)
                ->getQuery();
            $query->execute();
            $entityManager->flush();
        }

        $progress->finish();
        $io->success(sprintf('Generated all modules successfully'));
        return Command::SUCCESS;
    }
}
