<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Constants\AppConstants;
use InvalidArgumentException;
use PhpAmqpLib\Message\AMQPMessage;
use Symfony\Component\Console\Command\Command;
use PhpAmqpLib\Connection\AMQPStreamConnection;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MetadataRevertCommand extends Command
{
    protected $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:metadata:revert')
            ->setAliases(['cybersponse:metadata:revert'])
            ->setDescription('Reverts staging model metadata by copying it from model metadata.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');

        $commandsToExecute = [
            "TRUNCATE TABLE attribute_metadatas, staging_model_metadatas RESTART IDENTITY CASCADE",
            "INSERT into staging_model_metadatas select * from model_metadatas",
            "INSERT into attribute_metadatas select * from attrib_model_metadata"
        ];

        foreach ($commandsToExecute as $commandToExecute) {
            $metadataEntityDatabase = $metadataEntityManager->getConnection()->prepare($commandToExecute);
            try {
                $metadataEntityDatabase->execute();
            } catch (InvalidArgumentException | \Exception $e) {
                throw new \Exception('Failure: ' . $e->getMessage());
            }
        }
        $this->publishRevertMessage();
        $io->success('Metadata has been successfully reverted from StagingModelMetadata to ModelMetadata.');
        return Command::SUCCESS;
    }

    protected function prepareChangeData()
    {
        return [
            AppConstants::DELTA_DATA => [],
            AppConstants::COMPONENT => 'crudhub',
            AppConstants::ENTITY_TYPE => 'StagingModelMetadata',
            AppConstants::METHOD => 'POST',
            AppConstants::TRANSACTION_DATE => (int) ($_SERVER['REQUEST_TIME_FLOAT'] * 1000),
            AppConstants::ACTION => 'REVERT',
        ];
    }

    protected function publishRevertMessage()
    {
        $body = $this->prepareChangeData();
        $mqCredObj = parse_ini_file('/opt/cyops/configs/rabbitmq/rabbitmq_users.conf', false, INI_SCANNER_NORMAL);
        $rabbitMqUsername = $mqCredObj['cyops.rabbitmq.user'] ? $mqCredObj['cyops.rabbitmq.user'] : 'guest';
        $rabbitMqPassword = $mqCredObj['cyops.rabbitmq.password'] ? $mqCredObj['cyops.rabbitmq.password'] : 'guest';
        $rabbitMqHost = $mqCredObj['cyops.rabbitmq.host'] ? $mqCredObj['cyops.rabbitmq.host'] : 'localhost';
        $rabbitMqPort = $mqCredObj['cyops.rabbitmq.port'] ? $mqCredObj['cyops.rabbitmq.port'] : '5672';
        $rabbitMqVHost = $mqCredObj['cyops.rabbitmq.vhost'] ? $mqCredObj['cyops.rabbitmq.vhost'] : '/';

        $connection = new AMQPStreamConnection($rabbitMqHost, $rabbitMqPort, $rabbitMqUsername, $rabbitMqPassword, $rabbitMqVHost);
        $channel = $connection->channel();
        $channel->exchange_declare('cyops.crudhub.datanotify', 'topic', false, true, false);

        $msg = new AMQPMessage(json_encode($body), array('content_type' => 'application/json'));
        $channel->basic_publish($msg, 'cyops.crudhub.datanotify', 'route.crudhub.instructions.remoterequest.stagingmmdupdate');
        $channel->close();
        $connection->close();
    }
}
