<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use InvalidArgumentException;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MsspEntryCommand extends Command
{
    public function __construct(protected ContainerInterface $container)
    {
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:mssp:entry')
            ->setAliases(['cybersponse:mssp:entry'])
            ->setDefinition([
                new InputOption(
                    'hardwareKey',
                    'hkey',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting hardware key.'
                ),
                new InputOption(
                    'tenancy_enabled',
                    't_enabled',
                    InputOption::VALUE_OPTIONAL,
                    'Argument to specify whether Multi Tenancy is enabled.'
                ),
                new InputOption(
                    'user_id',
                    'usr_id',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting the DAS user_id.'
                ),
                new InputOption(
                    'role',
                    'rle',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting mssp role.'
                )
            ]);
    }

    protected function updateSelfId($entityManager, $uniqId, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Updating Self TenantId and AgentId Of Tenant and Agent Module',
            '=========================================================================',
            '',
        ]);

        try {
            $sql = "update TENANTS set tenantId='" . $uniqId . "' where role='self' and tenantid='745d976a-6ae8-4837-a364-64c251ef9e27'";
            $entityManager->getConnection()->prepare($sql)->execute();

            $sql = "update AGENTS set allowremoteconnectoroperation=true, agentId='" . $uniqId . "' where role='self' and agentid='745d976a-6ae8-4837-a364-64c251ef9e27'";
            $entityManager->getConnection()->prepare($sql)->execute();
        } catch (InvalidArgumentException | \Exception $e) {
            $out->writeln([
                '=========================================================================',
                'Updte Self TenantId and AgentId Of Tenant and Agent Module Failed',
                '=========================================================================', '',
            ]);
            throw new \Exception($e->getMessage());
        }

        $out->writeln([
            '=========================================================================',
            'Updated Self TenantId and AgentId Of Tenant and Agent Module',
            '=========================================================================',
            '',
        ]);
    }

    protected function createDefaultRemoteUser($entityManager, $user_id, $username, $out)
    {
        $sql = "INSERT INTO ACTORS (uuid, createuser, modifyuser, user_id, record_type, firstname, lastname, title, email, phonework,accessType) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '441c0092-8db1-4299-ba1a-6d704465465a', '441c0092-8db1-4299-ba1a-6d704465465a', '" . $user_id . "', 'person', '" . $username . "', 'User', 'Admin', 'admin@example.com', '646-275-9691','Concurrent')";
        try {
            $out->writeln([
                '=========================================================================',
                'Creating Default Remote User',
                '=========================================================================',
                '',
            ]);
            $tenantActor = $entityManager->getConnection()->prepare($sql);
            $tenantActor->execute();
        } catch (InvalidArgumentException | \Exception $e) {
            $out->writeln([$e->getMessage()]);
            $out->writeln([
                '=========================================================================',
                'Default Remote User Creation Failed',
                '=========================================================================',
                '',
            ]);
            throw new \Exception($e->getMessage());
        }
    }

    protected function defaultUserRole($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Assigning Remote User Default Roles',
            '=========================================================================',
            '',
        ]);

        $rolesUuid = $this->container->get('database_connection')->fetchAllAssociative("select uuid from roles where uuid in ('17562d60-b841-4ed7-912b-1ebaf063c8c9', '21d7a7c6-9d68-438f-b732-e8950c5f745b')");
        if (!empty($rolesUuid) && count($rolesUuid) == 2) {
            $sql = "INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '17562d60-b841-4ed7-912b-1ebaf063c8c9'), ('441c0092-8db1-4299-ba1a-6d704465465a', '21d7a7c6-9d68-438f-b732-e8950c5f745b')";
        } else {
            $rolesUuid = $this->container->get('database_connection')->fetchAllAssociative("select uuid from roles limit 1");
            $roleUuid = $rolesUuid[0]['uuid'];
            $sql = "INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '$roleUuid')";
        }
        $actorRole = $entityManager->getConnection()->prepare($sql);
        try {
            $actorRole->execute();
            $out->writeln([
                '=========================================================================',
                'Assigned Remote User Default Roles',
                '=========================================================================',
                '',
            ]);
        } catch (InvalidArgumentException | \Exception $e) {
            $out->writeln([$e->getMessage()]);
            $out->writeln(['Assigning Remote User Default Roles Failed']);
            throw new \Exception($e->getMessage());
        }
    }

    protected function defaultUserTeam($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Assigning Default Remote User Teams',
            '=========================================================================',
            '',
        ]);
        
        $teamsUuid = $this->container->get('database_connection')->fetchAllAssociative("select uuid from teams where uuid='6e569c09-3bd4-40f1-98b0-cc994464c3c5'");
        if (!empty($teamsUuid)) {
            $sql = "INSERT INTO TEAM_ACTOR (actor_uuid, team_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '6e569c09-3bd4-40f1-98b0-cc994464c3c5')";
        } else {
            $teamsUuid = $this->container->get('database_connection')->fetchAllAssociative("select uuid from teams limit 1");
            $teamUuid = $teamsUuid[0]['uuid'];
            $sql = "INSERT INTO TEAM_ACTOR (actor_uuid, team_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '$teamUuid')";
        }
        $actorTeam = $entityManager->getConnection()->prepare($sql);

        try {
            $actorTeam->execute();
            $out->writeln([
                '=========================================================================',
                'Assigned Default Remote User Teams',
                '=========================================================================',
                '',
            ]);
        } catch (InvalidArgumentException | \Exception $e) {
            $out->writeln([$e->getMessage()]);
            $out->writeln(['Assigning Team to Remote user Failed']);
            throw new \Exception($e->getMessage());
        }
    }

    protected function enableModuleReplication($entityManager, $role, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Enabling replication for modules',
            '=========================================================================',
            '',
        ]);

        if ($role == "master") {
            $sql = "Update staging_model_metadatas set peerreplicable=true, enablereplication=true where system=false and type not in ('people', 'appliances', 'agents', 'tenants', 'routers', 'threat-intel-feeds')";
            try {
                $entityManager->getConnection()->prepare($sql)->execute();
            } catch (InvalidArgumentException | \Exception $e) {
                $out->writeln(['Updating staging_model_metadatas Failed']);
                throw new \Exception($e->getMessage());
            }
        } else {
            $sql = "Update staging_model_metadatas set peerreplicable=true, enablereplication=true where type in ('alerts', 'incidents', 'emails', 'indicators', 'comments')";
            try {
                $entityManager->getConnection()->prepare($sql)->execute();
            } catch (InvalidArgumentException | \Exception $e) {
                $out->writeln(['Updating staging_model_metadatas Failed']);
                throw new \Exception($e->getMessage());
            }
        }
        $out->writeln([
            '=========================================================================',
            'Enabled replication for modules',
            '=========================================================================',
            '',
        ]);
    }

    protected function AddRemoteWorkflowStep($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Setting Remote Execution Playbook Workflow Step Visible',
            '=========================================================================',
            '',
        ]);

        $sql = "UPDATE public.workflow_step_types SET visible = true WHERE uuid = 'ab3b2e02-5e77-4ed6-8ebd-580f390063a5'";

        $workflowStep = $entityManager->getConnection()->prepare($sql);

        try {
            $workflowStep->execute();
            $out->writeln([
                '=========================================================================',
                'Remote Reference Playbook Step Update Success',
                '=========================================================================',
                '',
            ]);
        } catch (InvalidArgumentException | \Exception $e) {
            $out->writeln([$e->getMessage()]);
            $out->writeln(['Remote Reference Playbook Step Update Failed']);
            throw new \Exception($e->getMessage());
        }
    }

    protected function execute(InputInterface $input, OutputInterface $out)
    {
        try {
            $entityManager = $this->container->get('doctrine.orm.entity_manager');

            $role = $input->getOption('role');
            $tenancy_enabled = $input->getOption('tenancy_enabled');
            if ($tenancy_enabled == null) {
                $tenancy_enabled = false;
            }
            $user_id = $input->getOption('user_id') ? $input->getOption('user_id') : 'c59e0943-12e1-491f-a041-1071c93311ef';

            if ($role == 'master') {
                $username = 'Tenant';
            } else {
                $username = 'Master';
            }

            $uniqId = $input->getOption('hardwareKey');
            $this->updateSelfId($entityManager, $uniqId, $out);

            if (!$tenancy_enabled) {
                return Command::SUCCESS;
            }

            $this->createDefaultRemoteUser($entityManager, $user_id, $username, $out);

            $this->defaultUserRole($entityManager, $out);

            $this->defaultUserTeam($entityManager, $out);

            $this->enableModuleReplication($entityManager, $role, $out);

            $this->AddRemoteWorkflowStep($entityManager, $out);

            $out->writeln([
                '=========================================================================',
                'Publishing for MSSP Changes',
                '=========================================================================',
                '',
            ]);
            $command = sprintf('sudo -u nginx php /opt/cyops-api/bin/console app:system:update -laf');
            exec($command, $output, $errorCode);

            if ($errorCode) {
                $out->writeln(['Publish For MSSP Changes Failed']);
            } else {
                $out->writeln([
                    '=========================================================================',
                    'Publish For MSSP Changes Successful',
                    '=========================================================================',
                    '',
                ]);
            }
            return Command::SUCCESS;
        } catch (InvalidArgumentException | \Exception $e) {
            return Command::FAILURE;
        }
    }
}
