<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Command\SystemUpdate;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class PublishLockCommand extends Command
{
    protected $filesystem;
    public function __construct(protected ContainerInterface $container)
    {
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:publish:lock')
            ->setAliases(['cybersponse:publish:lock'])
            ->setDescription('Log 503 to the current node.')
            ->setHelp("This command allows you to lock request by setting 503.json.");
        $this->filesystem = new Filesystem();
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {

        $output->writeln([
            '===========================================================',
            'Locking the API request',
            '===========================================================',
            '',
        ]);
        $rootDir = $this->container->getParameter('kernel.project_dir');
        $logDir = sprintf('%s/public', $rootDir);
        $systemUpdate = new SystemUpdate();
        $systemUpdate
            ->setMessage('Locking For System Update')
            ->setProgressPercent(100);

        $this->log($input, $systemUpdate);
        $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));

        $output->writeln([
            '===========================================================',
            'API Request Locked',
            '===========================================================',
            '',
        ]);
        return Command::SUCCESS;
    }

    protected function log(InputInterface $input, SystemUpdate $systemUpdate)
    {
        $logDir = $this->getLogDir();
        $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));
    }

    private function getLogDir()
    {
        $rootDir = $this->container->getParameter('kernel.project_dir');
        return sprintf('%s/public', $rootDir);
    }
}
