<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use Symfony\Component\Console\Command\Command;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputOption;
use App\Entity\ViewTemplate\SystemViewTemplate;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class SystemViewTemplateBackupCommand extends Command
{
    private $entityManager;

    public function __construct(
        EntityManagerInterface $entityManager
    ) {
        $this->entityManager = $entityManager;
        parent::__construct();
    }

    protected function configure(): void
    {
        $this
            ->setName('app:restore:viewtemplate')
            ->setAliases(['fortisoar:restore:viewtemplate'])
            ->setDescription('This command will restore all systemview templates with their backups if present.')
            ->setDefinition([
                new InputOption(
                    'restore',
                    'r',
                    InputOption::VALUE_NONE,
                    'Arguments for systemviewtemplate restore.'
                ),
                new InputOption(
                    'system-view-template',
                    's',
                    InputOption::VALUE_REQUIRED,
                    'Arguments for systemviewtemplate restore.'
                )
            ]);
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        try {
            $beforeTimeStamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
            $name = $input->getOption('system-view-template');
            $restore = $input->getOption('restore');
            $io = new SymfonyStyle($input, $output);
            if ($restore) {
                $systemViewTemplateRepository = $this->entityManager->getRepository(SystemViewTemplate::class);
                $backupSystemViewTemplate = $this->getTemplate($systemViewTemplateRepository, $name . '-backup');
                $systemViewTemplate = $this->getTemplate($systemViewTemplateRepository, $name);
                $systemViewTemplate->setImportedBy($backupSystemViewTemplate->getImportedBy());
                $systemViewTemplate->setConfig($backupSystemViewTemplate->getConfig());
                $this->entityManager->persist($systemViewTemplate);
                $this->entityManager->flush();

                $io->success('SystemViewTemplate restore complete');
                $afterTimeStamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
                $io->writeln("Command took " . ($afterTimeStamp - $beforeTimeStamp) . " Second to execute");
                return Command::SUCCESS;
            } else {
                $io->error("Please provide [-r|--restore] to restore system view template from backup.\napp:backup:viewtemplate [-r|--restore RESTORE] [-s|--system-view-template SYSTEM-VIEW-TEMPLATE]");
                return Command::FAILURE;
            }
        } catch (NotFoundHttpException $nfe) {
            $io->error($nfe->getMessage());
            return Command::FAILURE;
        } catch (\Exception $ex) {
            $io->error($ex->getMessage());
            return Command::FAILURE;
        }
    }

    private function getTemplate($systemViewTemplateRepository, $name)
    {
        $systemViewTemplate = $systemViewTemplateRepository->findOneBy(['uuid' => $name]);
        if ($systemViewTemplate) {
            return $systemViewTemplate;
        } else {
            $errorMessage = sprintf('%s not found.', $name);
            throw new NotFoundHttpException($errorMessage);
        }
    }
}
