<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Constants\CacheConstants;
use App\Entity\ModelMetadata\ModelMetadata;
use App\Service\MetadataHelper;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Cache\Adapter\FilesystemTagAwareAdapter;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

class TypeClassMapCommand extends Command
{
    protected static $defaultName = 'app:type:class:map';
    protected $cache;
    private $inflector;

    protected function configure()
    {
        $this->setDescription('Prepare a map of module type to className');
    }

    public function __construct(protected EntityManagerInterface $entityManager, protected ContainerInterface $container, private MetadataHelper $metadataHelper)
    {
        $cacehDir = sprintf('%s/var/metadata', $this->container->getParameter('kernel.project_dir'));
        $this->cache = new FilesystemTagAwareAdapter('', 0, $cacehDir);
        $this->inflector = InflectorFactory::create()->build();
        parent::__construct();
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        // Sample Structure
        // [alerts => ["htmlEscape" => [], "files" =>[]]]
        $entityFieldMap = [];
        $io = new SymfonyStyle($input, $output);
        $this->cache->clear();
        $modelRepo = $this->entityManager->getRepository(ModelMetadata::class);
        $modelMetaDatas = $modelRepo->findAll();
        $systemFixturesFile = file_get_contents('/opt/cyops-api/src/StaticFile/SystemFixtures.json');
        $systemFixture = json_decode($systemFixturesFile);
        $entityMapLocal = [];
        foreach ($modelMetaDatas as $modelMetaData) {
            $singular = $modelMetaData->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $escapedSingular = $this->inflector->classify($escapedSingular);
            $this->metadataHelper->iterateORMObjectItems($modelMetaData, $escapedSingular, $entityMapLocal, $entityFieldMap);
        }
        foreach ($systemFixture as $key => $modelMetaData) {
            $singular = $modelMetaData->descriptions->singular;
            if ($singular == "Playbook") {
                $singular = "Workflow";
            } elseif ($singular == "Playbook Collection") {
                $singular = "Workflow Collection";
            } elseif ($singular == "Playbook Version") {
                $singular = "Workflow Version";
            }
            $escapedSingular = $this->inflector->classify($singular);
            $this->metadataHelper->iterateStdClassItems($modelMetaData, $escapedSingular, $entityMapLocal, $entityFieldMap);
        }

        $item = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        $item->set($entityMapLocal);
        $item->tag(CacheConstants::TAG_MODEL_METADATA);
        $this->cache->save($item);

        $entityFieldCache = $this->cache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP);
        $entityFieldCache->set($entityFieldMap);
        $entityFieldCache->tag(CacheConstants::TAG_MODEL_METADATA);
        $this->cache->save($entityFieldCache);

        $io->success('Generated Cache for Type Module Mapping');
        return Command::SUCCESS;
    }
}
