<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Constants\AppConstants;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class UtilCommand extends Command
{
    public function __construct(private ContainerInterface $container)
    {
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:util')
            ->setAliases(['cybersponse:util'])
            ->setDescription('This command allows you to access List of tags from cache pool to be cleared')
            ->addArgument('tags', InputArgument::IS_ARRAY, 'List of tags from cache pool to be cleared');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $tags = $input->getArgument('tags');
        if ((is_countable($tags) ? count($tags) : 0) > 0) {
            $headers = ["Content-type" => "application/json"];
            $method = 'POST';
            $uri = sprintf('%s/api/3/cache_util', $this->container->getParameter(AppConstants::CRUDHUB_PARAMETER_KEY));
            $data = ['operation' => 'invalidate-tag', 'tags' => $tags];
            $client = $this->container->get("app.proxy.client.crudhub");
            $request = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
            try {
                $response = $client->send($request, ['verify' => false]);
            } catch (BadRequestHttpException | \Exception $e) {
                $io->error('Cache cleared for given tags failed.');
                return -1;
            }

            $statusCode = $response->getStatusCode();
            if ($statusCode != Response::HTTP_OK) {
                $io->success('Cache cleared for given tags failed');
                return -1;
            }
        }
        $io->success('Cache cleared for given tags');
        return Command::SUCCESS;
    }
}
