<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\CompilerPass;

use GuzzleHttp\Client;
use App\Encoder\ProxyClient;
use App\Encoder\Hmac\HmacClient;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\DependencyInjection\Definition;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;

class ProxyClientCompilerPass implements CompilerPassInterface
{

    /**
     * You can modify the container here before it is dumped to PHP code.
     * @api
     */
    public function process(ContainerBuilder $container)
    {
        // todo get services for supported handlers by tag
        $supportedHandlers = [
            'hmac' => HmacClient::class
        ];

        $handlersConfig = $container->getParameter('app_proxy');

        $handlerMap = [];

        foreach ($handlersConfig['handlers'] as $handlerName => $handlerConfig) {
            $handlerType = $handlerConfig['authentication']['type'];
            $handlerClass = Client::class;
            $handlerName = "app.proxy.client.{$handlerName}";
            $aliasName = "proxy.client.{$handlerName}";

            if (array_key_exists($handlerType, $supportedHandlers)) {
                $handlerClass = $supportedHandlers[$handlerType];
            }

            $clientDefinition = new Definition($handlerClass);

            if (is_subclass_of($handlerClass, ProxyClient::class)) {
                $clientDefinition->addMethodCall('setRoutePrefix', [$handlerConfig['routePrefix']]);
                $clientDefinition->addMethodCall('setTargetUri', [$handlerConfig['targetUri']]);
                $clientDefinition->addMethodCall('setMonolog', [new Reference('logger')]);
                $clientDefinition->addMethodCall('setDebug', [$container->getParameter('kernel.debug')]);
                $clientDefinition->addMethodCall('setArguments', [$handlerConfig['authentication']['arguments']]);
                $clientDefinition->setPublic(true);
                $auth = ['handlerName' => $handlerName];
                if (array_key_exists('authorization', $handlerConfig)) {
                    $auth['permissions'] = $handlerConfig['authorization'];
                }
                $handlerMap[$handlerConfig['routePrefix']] = $auth;
            }
            $container->setDefinition($handlerName, $clientDefinition)->setPublic(true);
            $container->setAlias($aliasName, $handlerName);
            // $this->containers->get("app.proxy.client.<component>")
        }
        $container->setParameter('app.proxy.handler.map', $handlerMap);
    }
}
