<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Conditional;

use App\Conditional\ConditionSet;
use App\Conditional\QueryInterface;
use App\Conditional\QueryMatcher;

/**
 * Class ConditionSetEvaluator
 *
 * @package App\Validation
 */
class ConditionSetEvaluator
{
    public function __construct(protected QueryMatcher $queryMatcher)
    {
    }
    /**
     * @param $object
     *
     * @return array
     */
    public function evaluate($object, ConditionSet $conditionSet)
    {
        $resultSet = [];

        foreach ($conditionSet->getFields() as $fieldName => $fieldValidations) {
            $attributeName = null;

            foreach ($fieldValidations as $attributeName => $attributeValues) {
                if (!is_array($attributeValues)) {
                    $resultSet[$fieldName][$attributeName] = $attributeValues;
                    continue;
                }

                foreach ($attributeValues as $attributeValue) {
                    if ($attributeValue instanceof QueryInterface) {
                        $matches = $this->queryMatcher->matches($object, $attributeValue);
                        if ($matches) {
                            $resultSet[$fieldName][$attributeName] = $attributeValue->getResult();
                            break;
                        }
                    }
                }
            }
        }

        return $resultSet;
    }
}
