<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Conditional;

use ApiPlatform\Core\Api\IriConverterInterface;
use App\Conditional\QueryInterface;
use App\Constants\AttributeLevelOperators;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;

class QueryMatcher
{
    public const LOGIC_AND = 'AND';
    public const LOGIC_OR = 'OR';

    public function __construct(protected PropertyAccessorInterface $propertyAccessor, protected IriConverterInterface $iriConverter)
    {
    }
    /**
     * @param object $object
     *
     * @return bool
     */
    public function matches($object, QueryInterface $query)
    {
        $filters = $query->getFilters();
        $logic = $query->getLogic();

        $matches = false;

        foreach ($filters as $filter) {
            if ($filter instanceof QueryInterface) {
                $matches = $this->matches($object, $filter);
            } else {
                $fieldName = $filter['field'];
                $operator = $filter['operator'];
                $expectedValue = $filter['value'];

                $actualValue = $this->propertyAccessor->getValue($object, $fieldName);

                switch (strtolower($operator)) {
                    case AttributeLevelOperators::EQUALS:
                        $matches = $actualValue === $expectedValue;
                        if (!$matches && is_object($actualValue) && is_string($expectedValue)) {
                            $actualIri = $this->iriConverter->getIriFromItem($actualValue);
                            $matches = $actualIri === $expectedValue;
                        }
                        break;
                    case AttributeLevelOperators::NOT_EQUALS:
                        $matches = $actualValue !== $expectedValue;
                        if ($matches && is_object($actualValue) && is_string($expectedValue)) {
                            $actualIri = $this->iriConverter->getIriFromItem($actualValue);
                            $matches = $actualIri !== $expectedValue;
                        }
                        break;
                    case AttributeLevelOperators::GREATER_THAN:
                        $matches = $actualValue > $expectedValue;
                        break;
                    case AttributeLevelOperators::GREATER_THAN_EQUALS:
                        $matches = $actualValue >= $expectedValue;
                        break;
                    case AttributeLevelOperators::LESS_THAN:
                        $matches = $actualValue < $expectedValue;
                        break;
                    case AttributeLevelOperators::LESS_THAN_EQUALS:
                        $matches = $actualValue <= $expectedValue;
                        break;
                    case AttributeLevelOperators::IN:
                        $matches = in_array($actualValue, $expectedValue);
                        break;
                    case AttributeLevelOperators::NIN:
                        $matches = !in_array($actualValue, $expectedValue);
                        break;
                    case AttributeLevelOperators::IS_NULL:
                        $matches = !(is_null($actualValue) ^ $expectedValue);
                        break;
                    default:
                        $matches = false;
                        break;
                }
            }

            if ($matches === false && strcasecmp($logic, self::LOGIC_AND) === 0) {
                return false;
            }

            if ($matches === true && strcasecmp($logic, self::LOGIC_OR) === 0) {
                return true;
            }
        }

        return $matches;
    }


}
