<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */


namespace App\Consumer;

use Psr\Log\LoggerInterface;
use PhpAmqpLib\Message\AMQPMessage;
use Symfony\Component\Process\Process;
use OldSound\RabbitMqBundle\RabbitMq\ConsumerInterface;
use Symfony\Component\Process\Exception\ProcessFailedException;

class ApiMessageConsumer implements ConsumerInterface
{

  public function __construct(
    protected LoggerInterface $logger
  ) {
  }
  public function execute(AMQPMessage $msg)
  {
    $this->logger->info("Initiated handler command to install the solution");
    $body = $msg->body;
    $message = sprintf("--message='%s'", base64_encode($body));
    $process = new Process(["/bin/php", "/opt/cyops-api/bin/console", "app:solutionpack:import", "$message"]);
    $process->setTimeout(7200);
    $process->run();
    if (!$process->isSuccessful()) {
      $this->logger->error($process->getExitCode());
      $this->logger->error("Handler command failed, running the content hub sync command manually");
      if ($process->getExitCode() == 127) {
        // 127 indicate command not found or unsuccessful
        // While investigating bug 0894435, we identified then when msg size is large, it fails
        // In this case we are running sync command to show the data which might be present in the existing table
        // but might not be showing in content hub page
        $this->runContentHubSync();
      }
      return true;
    }
    $process->getOutput();
  }

  private function runContentHubSync()
  {
    $process = new Process(['/bin/php', '/opt/cyops-api/bin/console', 'fortisoar:contenthub:sync']);
    $process->setTimeout(7200);
    $process->run();
    if (!$process->isSuccessful()) {
      $this->logger->error($process->getExitCode());
      $this->logger->error("Manual sync failed, do run the content hub sync manually for latest changes");
      return true;
    }
  }
}
