<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Controller;

use App\Service\ViewTemplateService;
use App\Entity\ViewTemplate\UserViewTemplate;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;

class ViewTemplateController extends AbstractController
{

    protected $viewTemplateService;

    public function __construct(
        ViewTemplateService $viewTemplateService,
        protected SerializerInterface $serializer
    ) {
        $this->viewTemplateService = $viewTemplateService;
        $this->serializer = $serializer;
    }

    /**
     * @param $name
     * @return JsonResponse
     */
    public function getAction($name)
    {
        $serializer = $this->serializer;

        $viewTemplate = $this->findOrThrowNotFound($name);

        $normalizedViewTemplate = $serializer->normalize($viewTemplate, 'json');

        return new JsonResponse($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return JsonResponse
     */
    public function setAction($name, Request $request)
    {
        $viewTemplateProvider = $this->viewTemplateService;
        $serializer = $this->serializer;

        /** @var UserViewTemplate $userViewTemplate */
        $userViewTemplate = $serializer->deserialize(
            $request->getContent(),
            UserViewTemplate::class,
            'jsonld',
            [
                'object_to_populate' => $this->findOrThrowNotFound($name),
                'ignore_authorization' => true,
                'ignore_field_authorization' => true
            ]
        );

        $viewTemplateProvider->saveViewTemplate($name, $userViewTemplate);

        $normalizedViewTemplate = $serializer->normalize($userViewTemplate, 'json');

        return new JsonResponse($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return mixed
     */
    private function findOrThrowNotFound($name)
    {
        $viewTemplateProvider = $this->viewTemplateService;
        return $viewTemplateProvider->getViewTemplate($name);
    }
}
