<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\DataFixtures;

use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Bundle\FixturesBundle\FixtureGroupInterface;
use Doctrine\Persistence\ObjectManager;
use Nelmio\Alice\Loader\NativeLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;

class DataFixtures extends Fixture implements FixtureGroupInterface
{

    protected $containers;

    public function __construct(ContainerInterface $containers)
    {
        $this->containers = $containers;
    }
  
    const YML_FILES = ['modulesRolesFixtures.yml', '/Workflow/WorkflowStepType.yml', '/Workflow/WorkflowCollections.yml'];

    const JS_FILES = [
        '/Workflow/ReportManagementWorkflow/workflows.json', '/Workflow/ApprovalWorkflows/workflows.json', '/Workflow/SchedulerSystemWorkflow/workflows.json',
        '/EmailTemplates/EmailTemplate.json'
    ];

    public static function getGroups(): array{
        return ['data'];
    }

    public function load(ObjectManager $manager)
    {
        $loader = new \Nelmio\Alice\Loader\NativeLoader();
        foreach (self::YML_FILES as $file) {
            $ymlSet = $loader->loadFile(__DIR__ . '/' . $file);
            foreach ($ymlSet->getObjects() as $key => $value) {
                $manager->persist($ymlSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        $this->loadJsonFixtures($manager);
    }

    public function loadJsonFixtures($manager)
    {
        $serialize = $this->containers->get('serializer');
        foreach (self::JS_FILES as $file) {
            $file_content = json_decode(file_get_contents(__DIR__ . $file), true);
            foreach ($file_content as $key => $values) {
                foreach ($values as $value) {
                    $object = $serialize->deserialize(
                        json_encode($value),
                        $key,
                        'jsonld',
                        ['fetch_data' => false, "ignore_authorization" => true, "ignore_field_authorization" => true, "ignore_relationship_authorization" => true]
                    );
                    $manager->persist($object);
                }
                $manager->flush();
            }
        }
    }
}