<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\DataFixtures;

use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Bundle\FixturesBundle\FixtureGroupInterface;
use Doctrine\Persistence\ObjectManager;
use Nelmio\Alice\Loader\NativeLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ModelFixtures extends Fixture implements FixtureGroupInterface
{

    protected $containers;

    public function __construct(ContainerInterface $containers)
    {
        $this->containers = $containers;
    }

    const JSON_FIXTURES = [
        'picklists.json',
        'systemSettings.json',
        '/SystemViewTemplate/cs_queue_management.json', '/SystemViewTemplate/app.json',
        '/SystemViewTemplate/grid_columns.json'
    ];

    const MODELMETADATA_FIXTURE_DIR = ['/ModelMetadata'];

    const JS_FILES = [
        '/SystemViewTemplate/modules/agents/detail.json', '/SystemViewTemplate/modules/agents/form.json', '/SystemViewTemplate/modules/agents/list.json',
        '/SystemViewTemplate/modules/attachments/detail.json', '/SystemViewTemplate/modules/attachments/list.json', '/SystemViewTemplate/modules/attachments/form.json',
        '/SystemViewTemplate/modules/comments/detail.json', '/SystemViewTemplate/modules/comments/list.json', '/SystemViewTemplate/modules/comments/form.json',
        '/SystemViewTemplate/modules/tenants/detail.json', '/SystemViewTemplate/modules/tenants/list.json', '/SystemViewTemplate/modules/tenants/form.json',
        '/SystemViewTemplate/modules/tasks/detail.json', '/SystemViewTemplate/modules/tasks/list.json', '/SystemViewTemplate/modules/tasks/form.json',
        '/SystemViewTemplate/modules/tasks/settings.json',
        '/SystemViewTemplate/modules/approvals/detail.json', '/SystemViewTemplate/modules/approvals/list.json', '/SystemViewTemplate/modules/approvals/form.json',
        '/SystemViewTemplate/modules/companies/detail.json', '/SystemViewTemplate/modules/companies/list.json', '/SystemViewTemplate/modules/companies/form.json',
        '/SystemViewTemplate/modules/widgets/detail.json', '/SystemViewTemplate/modules/widgets/list.json', '/SystemViewTemplate/modules/widgets/form.json',
        '/SystemViewTemplate/modules/leave_schedules/detail.json','/SystemViewTemplate/modules/leave_schedules/list.json'
    ];

    public static function getGroups(): array{
        return ['minimal'];
    }

    public function load(ObjectManager $manager)
    {

        $loader = new \Nelmio\Alice\Loader\NativeLoader();
        $listOfMmds = array_diff(scandir(__DIR__ . '/ModelMetadata'), array('.', '..'));
        foreach (self::JSON_FIXTURES as $file) {
            $jsonSet = $loader->loadFile(__DIR__ . '/' . $file);
            foreach ($jsonSet->getObjects() as $key => $value) {
                $manager->persist($jsonSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        foreach ($listOfMmds as $file) {
            $jsonSet = $loader->loadFile(__DIR__ . '/ModelMetadata/' . $file);
            foreach ($jsonSet->getObjects() as $key => $value) {
                $manager->persist($jsonSet->getObjects()[$key]);
            }
            $manager->flush();
        }
        $this->loadJsonFixtures($manager);
    }

    public function loadJsonFixtures($manager)
    {
        $serialize = $this->containers->get('serializer');
        foreach (self::JS_FILES as $file) {
            $file_content = json_decode(file_get_contents(__DIR__ . $file), true);
            foreach ($file_content as $key => $values) {
                foreach ($values as $value) {
                    $object = $serialize->deserialize(
                        json_encode($value),
                        $key,
                        'jsonld',
                        ['fetch_data' => false, "ignore_authorization" => true, "ignore_field_authorization" => true, "ignore_relationship_authorization" => true]
                    );
                    $manager->persist($object);
                }
                $manager->flush();
            }
        }
    }
}
