<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Doctrine\DBAL\Types;

use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Types\Type;
use Doctrine\DBAL\Types\ConversionException;

class VarcharArrayType extends Type
{
    public function getSQLDeclaration(array $column, AbstractPlatform $platform)
    {
        return 'varchar[]';
    }

    public function convertToDatabaseValue($value, AbstractPlatform $platform)
    {
        if ($value === null) {
            return null;
        }
        if (is_array($value)){
            $encoded = '{' . implode(",", $value) . '}';
            if (json_last_error() !== JSON_ERROR_NONE) {
                throw ConversionException::conversionFailedSerialization($value, 'json', json_last_error_msg());
            }
            return $encoded;
        }
        return $value;
    }

    /**
     * {@inheritdoc}
     */
    public function convertToPHPValue($value, AbstractPlatform $platform)
    {
        // for perf, not converting to array here. can be converted on demand 
        return $value;
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'varchar_array';
    }

    /**
     * {@inheritdoc}
     */
    public function requiresSQLCommentHint(AbstractPlatform $platform)
    {
        return true;
    }
}
