<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Doctrine\ORM;


use App\Entity\Base\BaseEntity;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Ramsey\Uuid\Uuid;

class BaseEntityListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isInstanceOfBaseEntity($object) && is_null($object->getUuid())) {
            /** @var BaseEntity $object */
            $manager = $this->managerRegistry->getManagerForClass(get_class($object));
            if ($manager instanceof EntityManager) {
                $object->setUuid(Uuid::uuid4());
            }
        }
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isInstanceOfBaseEntity($object)
    {
        return $object instanceof BaseEntity;
    }
}
