<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
// api/src/Doctrine/CurrentUserExtension.php

namespace App\Doctrine;

use App\Entity\Query\Query;
use App\Traits\SoftDeleteable;
use Doctrine\ORM\QueryBuilder;
use App\Providers\QueryProvider;
use App\Entity\Workflow\Workflow;
use App\Doctrine\ORM\FilterQueryBuilder;
use App\Doctrine\ORM\SearchQueryBuilder;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Util\QueryNameGeneratorInterface;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Extension\QueryItemExtensionInterface;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Extension\QueryCollectionExtensionInterface;

final class QueryExtension implements QueryCollectionExtensionInterface, QueryItemExtensionInterface
{
    private $requeststack;
    private $request;
    private $filterQueryBuilder;
    protected $queryProvider;
    private $searchQueryBuilder;
    protected $entityManager;

    public function __construct(
        RequestStack $requeststack,
        FilterQueryBuilder $filterQueryBuilder,
        QueryProvider $queryProvider,
        SearchQueryBuilder $searchQueryBuilder,
        EntityManagerInterface $entityManager
    ) {
        $this->requeststack = $requeststack;
        $this->filterQueryBuilder = $filterQueryBuilder;
        $this->queryProvider = $queryProvider;
        $this->searchQueryBuilder = $searchQueryBuilder;
        $this->entityManager = $entityManager;
    }

    private function setCurrentRequest()
    {
        $this->request = $this->requeststack->getCurrentRequest();
    }

    public function applyToCollection(QueryBuilder $queryBuilder, QueryNameGeneratorInterface $queryNameGenerator, string $resourceClass, string $operationName = null): void
    {
        $this->setCurrentRequest();
        $this->addWhere($queryBuilder, $resourceClass);
    }

    public function applyToItem(QueryBuilder $queryBuilder, QueryNameGeneratorInterface $queryNameGenerator, string $resourceClass, array $identifiers, string $operationName = null, array $context = []): void
    {
        $this->setCurrentRequest();
        $this->addWhere($queryBuilder, $resourceClass);
    }

    public function isSoftDeleteable($entity_class)
    {
        return in_array(SoftDeleteable::class, class_uses($entity_class));
    }

    private function addWhere(QueryBuilder $queryBuilder, string $resourceClass): void
    {
        $query = $this->queryProvider->getQueryObject();
        if (!(isset($query['logic']) && isset($query['filters']))) {
            $query['logic'] = "AND";
            $query['filters'] = [];
        }
        $this->filterQueryBuilder->fromQueryObject($query, $queryBuilder, $resourceClass);
        $this->searchQueryBuilder->fromQueryObject($query, $queryBuilder, $resourceClass);
        if ($this->isSoftDeleteable($resourceClass) && array_key_exists(Query::SHOW_DELETED, $_GET) && array_key_exists('softdeleteable', $this->entityManager->getFilters()->getEnabledFilters())) {
            $this->entityManager->getFilters()->disable('softdeleteable');
        }
    }
}
