<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder;

use App\Encoder\CorrectSimpleJWS;
use App\Providers\JwtLogoutKeyProvider;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Lexik\Bundle\JWTAuthenticationBundle\Encoder\LcobucciJWTEncoder;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Lexik\Bundle\JWTAuthenticationBundle\Services\KeyLoader\KeyLoaderInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class CorrectJwtEncoder extends LcobucciJWTEncoder
{
    protected $container;
    protected $jwsProvider;
    protected $jwtLogoutKeyProvider;
    protected $requestStack;

    const ALGORITHM = 'RS256';

    public function __construct(
        ContainerInterface $container,
        protected KeyLoaderInterface $keyLoader,
        JwtLogoutKeyProvider $jwtLogoutKeyProvider,
        RequestStack $requestStack
    ) {
        $this->container = $container;
        $this->keyLoader = $keyLoader;
        $this->jwtLogoutKeyProvider =  $jwtLogoutKeyProvider;
        $this->requestStack = $requestStack;
    }

    public function decode($token)
    {
        try {
            $jws = CorrectSimpleJWS::load($token);
        } catch (\InvalidArgumentException $e) {
            throw new AuthenticationException('Unauthenticated');
        } catch (\Exception $err) {
            throw new AuthenticationException('Unauthenticated');
        }
        if ($jws instanceof CorrectSimpleJWS && !$jws->isValid($this->getPublicKey(), self::ALGORITHM, null, $this->requestStack->getCurrentRequest())) {
            throw new AuthenticationException('Unauthenticated');
        }

        if ($this->isLogout($token)) {
            throw new AuthenticationException('Unauthenticated');
        }

        return $jws->getPayload();
    }

    /**
     *
     */
    protected function getPublicKey()
    {
        while (!empty($errorStr = openssl_error_string())) {
        }
        $pkey = $this->keyLoader->loadKey("public");
        while (!empty($errorStr = openssl_error_string())) {
        }
        return $pkey;
    }

    /**
     * @param string $jwtKey
     * @return bool
     */
    private function isLogout($jwtKey)
    {
        if ($this->container == null) {
            return false;
        }
        return $this->jwtLogoutKeyProvider->isJwtLogoutKey("Bearer " . $jwtKey);
    }
}
