<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encoder;

use Namshi\JOSE\JWS;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Exception\AuthenticationException;

class CorrectSimpleJWS extends JWS
{

    public function isValid($key, $algo = null, $type = null, Request $request = null)
    {
        $payload = $this->getPayload();
        return $this->verify($key, $algo) && !$this->isExpired($payload) && $this->isValidType($payload, $type, $request);
    }

    public function isValidType($payload, $type, $request)
    {
        if (!isset($payload['type'])){
            throw new AuthenticationException('No Type specified');
        }
        $tokenType = $payload['type'];
        if (is_null($type)){
            # block at least restricted and noauth
            if ($tokenType === 'restricted' || $tokenType === 'noauth'){
                throw new AuthenticationException('Unsupported token type specified');
            }

            if ($tokenType === '2fa' && parse_url($request->getUri())['path'] !== '/api/auth/nfa-request'){
                throw new AuthenticationException('Unsupported token type specified');
            }

            if ($tokenType === 'saml' && parse_url($request->getUri())['path'] !== '/api/auth/token'){
                throw new AuthenticationException('Unsupported token type specified');
            }

            if ($tokenType === 'reset' && parse_url($request->getUri())['path'] !== '/api/auth/users') {
                throw new AuthenticationException('Unsupported token type specified');
            }
            return true;
        }
        else if ($tokenType === $type) {
            return true;
        }
        throw new AuthenticationException('Unauthenticated');
    }
 
    /**
     * @param $payload
     * @return bool
     */
    public function isExpired($payload)
    {
        if (isset($payload['exp']) && is_numeric($payload['exp'])) {
            $now = new \DateTime('now');
            return ($now->format('U') - $payload['exp']) > 0;
        }
        throw new AuthenticationException('Token Expired');
    }

}
