<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Encryptor;

use Defuse\Crypto\KeyProtectedByPassword;
use Symfony\Component\Filesystem\Filesystem;
use Ambta\DoctrineEncryptBundle\Encryptors\EncryptorInterface;

/**
 * Class for encrypting and decrypting with the defuse library
 *
 */

class FSRDefuseEncryptor implements EncryptorInterface
{
    private $fs;
    private $encryptionKey;
    private $keyFile;
    const FSRDEFUSE = 'TZhmKajFjyyRyfrQsuYnKhSPjCBrmG';
    const CLOUDKEY = '/opt/cyops/configs/cyops-api/.Defuse_cloud.key';
    private $legecyEncryptor = false;

    /**
     * {@inheritdoc}
     */
    public function __construct(string $keyFile)
    {
        $this->keyFile       = '/opt/cyops/configs/cyops-api/.Defuse.key';
        $this->fs            = new Filesystem();
        $this->legecyEncryptor = $this->isLegecyEncryptor();
    }

    public function isLegecyEncryptor()
    {
        if ($this->fs->exists(self::CLOUDKEY)) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * {@inheritdoc}
     */
    public function encrypt($data)
    {
        if ($this->legecyEncryptor) {
            return \Defuse\Crypto\Crypto::encryptWithPassword($data, $this->getKey());
        }
        return \Defuse\Crypto\Crypto::encrypt($data, $this->getKey());
    }

    /**
     * {@inheritdoc}
     */
    public function decrypt($data)
    {
        if ($this->legecyEncryptor) {
            return \Defuse\Crypto\Crypto::decryptWithPassword($data, $this->getKey());
        }
        return \Defuse\Crypto\Crypto::decrypt($data, $this->getKey());
    }

    private function getKey()
    {
        if ($this->encryptionKey === null) {
            if ($this->isLegecyEncryptor()) {
                $this->encryptionKey = file_get_contents(self::CLOUDKEY);
            } elseif ($this->fs->exists($this->keyFile)) {
                $protected_key = KeyProtectedByPassword::loadFromAsciiSafeString(trim(file_get_contents($this->keyFile)));
                $this->encryptionKey = $protected_key->unlockKey(self::FSRDEFUSE);
            } else {
                $this->encryptionKey = KeyProtectedByPassword::createRandomPasswordProtectedKey(self::FSRDEFUSE);
                $this->fs->dumpFile($this->keyFile, $this->encryptionKey);
                $protected_key = KeyProtectedByPassword::loadFromAsciiSafeString(file_get_contents($this->keyFile));
                $this->encryptionKey  = $protected_key->unlockKey(self::FSRDEFUSE);
            }
        }
        return $this->encryptionKey;
    }
}
