<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ActionSuggestion;

use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiProperty;
use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="Statistics", "fsr_module"="security", "fsr_singular"="Statistic"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     normalizationContext={"ignore_depth"=true},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" }
 *     }
 * )
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @ORM\Table(name="statistics", indexes={@Doctrine\ORM\Mapping\Index(name="statistics_action_uuid_idx", columns={"actionUuid"}), @Doctrine\ORM\Mapping\Index(name="statistics_record_uuid_idx", columns={"recordUuid"})})
 */
class Statistic extends BaseEntity
{

    /**
     * @Groups({"fsr_primary", "fsr_all", "recordUuid"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $recordUuid;

    /**
     * @Groups({"fsr_primary", "fsr_all", "recordType"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $recordType;

    /**
     * @var \App\Entity\Core\Picklist
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="App\Entity\Core\Picklist", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="statsType", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Groups({"fsr_primary", "fsr_all", "statsType"})
     */
    private $statsType;

    /**
     * @var \App\Entity\Core\Picklist
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="App\Entity\Core\Picklist", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="collectionFrequency", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Groups({"fsr_primary", "fsr_all", "collectionFrequency"})
     */
    private $collectionFrequency;

    /**
     * @Groups({"fsr_primary", "fsr_all", "action"})
     * @ORM\Column(type="string", length=500, nullable=false)
     */
    private $action;

    /**
     * @Groups({"fsr_primary", "fsr_all", "actionUuid"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $actionUuid;

    /**
     * @Groups({"fsr_primary", "fsr_all", "metadata"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $metadata;

    /**
     * @Doctrine\ORM\Mapping\Column(name="createdate", type="datetime", nullable=false, insertable=false, options={"default": "CURRENT_TIMESTAMP"})
     * @Groups({"fsr_primary", "fsr_all", "createDate"})
     * @ApiProperty(writable=false)
     */
    protected $createDate;

    /**
     * @Groups({"fsr_primary", "fsr_all", "taskId"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $taskId;

    /**
     * Get the value of recordUuid
     */
    public function getRecordUuid()
    {
        return $this->recordUuid;
    }

    /**
     * Set the value of recordUuid
     *
     * @return  self
     */
    public function setRecordUuid($recordUuid)
    {
        $this->recordUuid = $recordUuid;

        return $this;
    }

    /**
     * Get the value of recordType
     */
    public function getRecordType()
    {
        return $this->recordType;
    }

    /**
     * Set the value of recordType
     *
     * @return  self
     */
    public function setRecordType($recordType)
    {
        $this->recordType = $recordType;

        return $this;
    }

    /**
     * Get the value of metadata
     */
    public function getMetadata()
    {
        return $this->metadata;
    }

    /**
     * Set the value of metadata
     *
     * @return  self
     */
    public function setMetadata($metadata)
    {
        $this->metadata = $metadata;

        return $this;
    }

    /**
     * Get the value of statsType
     */
    public function getStatsType()
    {
        return $this->statsType;
    }

    /**
     * Set the value of statsType
     *
     * @return  self
     */
    public function setStatsType($statsType)
    {
        $this->statsType = $statsType;

        return $this;
    }

    /**
     * Get the value of createDate
     */
    public function getCreateDate()
    {
        return $this->createDate;
    }

    /**
     * Set the value of createDate
     *
     * @return  self
     */
    public function setCreateDate($createDate)
    {
        $this->createDate = $createDate;

        return $this;
    }

    /**
     * Get the value of actionUuid
     */
    public function getActionUuid()
    {
        return $this->actionUuid;
    }

    /**
     * Set the value of actionUuid
     *
     * @return  self
     */
    public function setActionUuid($actionUuid)
    {
        $this->actionUuid = $actionUuid;

        return $this;
    }

    /**
     * Get the value of action
     */
    public function getAction()
    {
        return $this->action;
    }

    /**
     * Set the value of action
     *
     * @return  self
     */
    public function setAction($action)
    {
        $this->action = $action;

        return $this;
    }

    /**
     * Get the value of collectionFrequency
     *
     * @return  \App\Entity\Core\Picklist
     */
    public function getCollectionFrequency()
    {
        return $this->collectionFrequency;
    }

    /**
     * Set the value of collectionFrequency
     *
     * @param  \App\Entity\Core\Picklist  $collectionFrequency
     *
     * @return  self
     */
    public function setCollectionFrequency(\App\Entity\Core\Picklist $collectionFrequency)
    {
        $this->collectionFrequency = $collectionFrequency;

        return $this;
    }

    /**
     * Get the value of taskId
     */
    public function getTaskId()
    {
        return $this->taskId;
    }

    /**
     * Set the value of taskId
     *
     * @return  self
     */
    public function setTaskId($taskId)
    {
        $this->taskId = $taskId;

        return $this;
    }
}
