<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Actors;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Core\AbstractFile;
use App\EventSubscriber\AvatarListener;
use App\Repository\AvatarRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\NotNull;

/**
 * @ApiResource(
 * attributes={"validation_groups"={"fsr_primary", "fsr_all"},"fsr_type"="avatar", "fsr_module"="application", "fsr_singular"="avatar"},
 * denormalizationContext={"groups"={"fsr_all"}, "fetch_data"=false},
 * normalizationContext={"include_file_data"=true},
 *     collectionOperations={
 *        "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *        "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *        "get" = { "security" = "is_granted('read.application', object)" },
 *        "put" = { "security" = "is_granted('update.application', object)" },
 *        "delete" = { "security" = "is_granted('delete.application', object)" }
 *    }
 * )
 * @ORM\Entity(repositoryClass=AvatarRepository::class)
 * @ORM\EntityListeners({"App\EventSubscriber\AvatarListener"})
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorColumn(name="record_type", type="string", length=20)
 * @ORM\Table(name="avatar", uniqueConstraints={@UniqueConstraint(name="uploadUser_unique", columns={"uploaduser"})})
 */

class Avatar extends AbstractFile
{

    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $name;


    /**
     * @Groups({"fsr_primary", "fsr_all", "height"})
     * @ORM\Column(type="integer", nullable=true)
     */
    private $height;

    /**
     * @Groups({"fsr_primary", "fsr_all", "width"})
     * @ORM\Column(type="integer", nullable=true)
     */
    private $width;

    /**
     * @Groups({"fsr_primary", "fsr_all", "uploadUser"})
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     */
    private $uploadUser;


    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }


    public function getHeight(): ?int
    {
        return $this->height;
    }

    public function setHeight(?int $height): self
    {
        $this->height = $height;

        return $this;
    }

    public function getWidth(): ?int
    {
        return $this->width;
    }

    public function setWidth(?int $width): self
    {
        $this->width = $width;

        return $this;
    }

    public function getUploadUser(): ?string
    {
        return $this->uploadUser;
    }

    public function setUploadUser(?string $uploadUser): self
    {
        $this->uploadUser = $uploadUser;

        return $this;
    }
}
