<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Authorization;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\FieldPermissionRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="field_permissions", "fsr_module"="security", "fsr_singular"="field_permission"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=FieldPermissionRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="field_permissions")
 */
class FieldPermission extends BaseEntity
{

    /**
     * @Groups({"fsr_primary", "fsr_all", "field"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $field;

    /**
     * @Groups({"fsr_primary", "fsr_all", "canRead"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $canRead;

    /**
     * @Groups({"fsr_primary", "fsr_all", "canWrite"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $canWrite;


    public function getField(): ?string
    {
        return $this->field;
    }

    public function setField(?string $field): self
    {
        $this->field = $field;

        return $this;
    }

    public function getCanRead(): ?bool
    {
        return $this->canRead;
    }

    public function setCanRead(?bool $canRead): self
    {
        $this->canRead = $canRead;

        return $this;
    }

    public function getCanWrite(): ?bool
    {
        return $this->canWrite;
    }

    public function setCanWrite(?bool $canWrite): self
    {
        $this->canWrite = $canWrite;

        return $this;
    }
}
