<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Authorization;

use App\Entity\Core\Module;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\Collection;
use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\ModulePermissionRepository;
use Doctrine\Common\Collections\ArrayCollection;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="module_permissions", "fsr_module"="security", "fsr_singular"="module_permission"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=ModulePermissionRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="module_permissions")
 */
class ModulePermission extends BaseEntity
{

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "canCreate"})
     */
    private $canCreate = false;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "canExecute"})
     */
    private $canExecute = false;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "canRead"})
     */
    private $canRead = false;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "canUpdate"})
     */
    private $canUpdate = false;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "canDelete"})
     */
    private $canDelete = false;

    /**
     * @ORM\ManyToMany(targetEntity=FieldPermission::class, cascade={"persist", "remove"})
     * @Doctrine\ORM\Mapping\JoinTable(name="modulepermission_fieldpermission",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="modulepermission_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="fieldpermission_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Groups({"fsr_primary", "fsr_all", "fieldPermissions"})
     */
    private $fieldPermissions;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\ManyToOne(targetEntity=Module::class, cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="module_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     * })
     * @Groups({"fsr_primary", "module"})
     */
    private $module;

    /**
     * @ORM\ManyToMany(targetEntity=Role::class, mappedBy="modulePermissions")
     * @ORM\OrderBy({"name" = "ASC"})
     * @Groups({"fsr_all", "roles"})
     */
    private $roles;

    public function __construct()
    {
        $this->fieldPermissions = new ArrayCollection();
        $this->roles = new ArrayCollection();
    }

    public function getModule()
    {
        return $this->module;
    }

    public function setModule($module)
    {
        $this->module = $module;

        return $this;
    }

    public function getCanCreate(): ?bool
    {
        return $this->canCreate;
    }

    public function setCanCreate(?bool $canCreate): self
    {
        $this->canCreate = $canCreate;

        return $this;
    }

    public function getCanExecute(): ?bool
    {
        return $this->canExecute;
    }

    public function setCanExecute(?bool $canExecute): self
    {
        $this->canExecute = $canExecute;

        return $this;
    }

    public function getCanRead(): ?bool
    {
        return $this->canRead;
    }

    public function setCanRead(?bool $canRead): self
    {
        $this->canRead = $canRead;

        return $this;
    }

    public function getCanUpdate(): ?bool
    {
        return $this->canUpdate;
    }

    public function setCanUpdate(?bool $canUpdate): self
    {
        $this->canUpdate = $canUpdate;

        return $this;
    }

    public function getCanDelete(): ?bool
    {
        return $this->canDelete;
    }

    public function setCanDelete(?bool $canDelete): self
    {
        $this->canDelete = $canDelete;

        return $this;
    }

    /**
     * @return Collection|FieldPermission[]
     */
    public function getFieldPermissions(): Collection
    {
        return $this->fieldPermissions;
    }

    public function addFieldPermission(FieldPermission $fieldPermission): self
    {
        if (!$this->fieldPermissions->contains($fieldPermission)) {
            $this->fieldPermissions[] = $fieldPermission;
        }

        return $this;
    }

    public function removeFieldPermission(FieldPermission $fieldPermission): self
    {
        if ($this->fieldPermissions->contains($fieldPermission)) {
            $this->fieldPermissions->removeElement($fieldPermission);
        }

        return $this;
    }

    /**
     * @return Collection|Role[]
     */
    public function getRoles(): Collection
    {
        return $this->roles;
    }

    public function addRole(Role $role): self
    {
        if (!$this->roles->contains($role)) {
            $this->roles[] = $role;
            $role->addModulePermission($this);
        }

        return $this;
    }

    public function removeRole(Role $role): self
    {
        if ($this->roles->contains($role)) {
            $this->roles->removeElement($role);
            $role->removeModulePermission($this);
        }

        return $this;
    }
}
