<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Authorization;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\RoleRepository;
use App\Traits\Importable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="roles", "fsr_module"="security", "fsr_singular"="Role"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     normalizationContext={"ignore_depth"=true},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" },
 *          "put" = { "security" = "is_granted('update.security', object)" },
 *          "delete" = { "security" = "is_granted('delete.security', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=RoleRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="roles")
 */
class Role extends BaseEntity
{
    use Importable;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="text", nullable=true)
     * @Groups({"fsr_primary", "fsr_all", "description"})
     */
    private $description;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=false, unique=true)
     * @Groups({"fsr_primary", "fsr_all", "name"})
     */
    private $name;

    /**
     * @Groups({"fsr_all"})
     * @ORM\ManyToMany(targetEntity=ModulePermission::class, orphanRemoval=true, inversedBy="roles", cascade={"persist"})
     * @Groups({"fsr_all", "modulePermissions"})
     */
    private $modulePermissions;

    public function __construct()
    {
        $this->modulePermissions = new ArrayCollection();
    }


    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    /**
     * @return Collection|ModulePermission[]
     */
    public function getModulePermissions(): Collection
    {
        return $this->modulePermissions;
    }

    public function addModulePermission(ModulePermission $modulePermission): self
    {
        if (!$this->modulePermissions->contains($modulePermission)) {
            $this->modulePermissions[] = $modulePermission;
        }

        return $this;
    }

    public function removeModulePermission(ModulePermission $modulePermission): self
    {
        if ($this->modulePermissions->contains($modulePermission)) {
            $this->modulePermissions->removeElement($modulePermission);
        }

        return $this;
    }
}
