<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use App\Entity\Base\BaseEntity;
use App\Validator\IsJsonType;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * @Doctrine\ORM\Mapping\MappedSuperclass
 * @Doctrine\ORM\Mapping\HasLifecycleCallbacks
 */
class AbstractFile extends BaseEntity
{
    /**
     * @Groups({"fsr_primary", "fsr_all", "id"})
     * @Doctrine\ORM\Mapping\Column(type="string", length=150, nullable=true, unique=true)
     */
    private $id;

    /**
     * @Groups({"fsr_primary", "fsr_all", "file"})
     * @Doctrine\ORM\Mapping\Column(type="string", length=255, nullable=true)
     */
    private $file;

    /**
     * @Groups({"fsr_primary", "fsr_all", "filename"})
     * @Doctrine\ORM\Mapping\Column(type="string", length=1024, nullable=true)
     */
    private $filename;

    /**
     * @Groups({"fsr_primary", "fsr_all", "mimeType"})
     * @Doctrine\ORM\Mapping\Column(name="`mimeType`", type="string", length=255, nullable=true, unique=false)
     */
    private $mimeType;

    /**
     * @Groups({"fsr_primary", "fsr_all", "uploadDate"})
     * @Doctrine\ORM\Mapping\Column(type="datetime", nullable=true)
     */
    private $uploadDate;

    /**
     * @Groups({"fsr_primary", "fsr_all", "size"})
     * @Doctrine\ORM\Mapping\Column(type="integer", nullable=true)
     */
    private $size;

    /**
     * @Groups({"fsr_primary", "fsr_all", "thumbnail"})
     * @Doctrine\ORM\Mapping\Column(type="text", nullable=true)
     */
    private $thumbnail;

    /**
     * @IsJsonType("object")
     * @Groups({"fsr_primary", "fsr_all", "metadata"})
     * @Doctrine\ORM\Mapping\Column(type="jsonb", nullable=true)
     */
    private $metadata = [];

    public function getId(): ?string
    {
        return $this->id;
    }

    public function setId(?string $id): self
    {
        $this->id = $id;

        return $this;
    }

    public function getFile(): ?string
    {
        return $this->file;
    }

    public function setFile(?string $file): self
    {
        $this->file = $file;

        return $this;
    }

    public function getFilename(): ?string
    {
        return $this->filename;
    }

    public function setFilename(?string $filename): self
    {
        $this->filename = $filename;

        return $this;
    }

    public function getMimeType(): ?string
    {
        return $this->mimeType;
    }

    public function setMimeType(?string $mimeType): self
    {
        $this->mimeType = $mimeType;

        return $this;
    }

    public function getUploadDate()
    {
        return $this->uploadDate;
    }

    public function setUploadDate( $uploadDate): self
    {
        $this->uploadDate = $uploadDate;

        return $this;
    }

    public function getSize(): ?int
    {
        return $this->size;
    }

    public function setSize(?int $size): self
    {
        $this->size = $size;

        return $this;
    }

    public function getThumbnail(): ?string
    {
        return $this->thumbnail;
    }

    public function setThumbnail(?string $thumbnail): self
    {
        $this->thumbnail = $thumbnail;

        return $this;
    }

    public function getMetadata()
    {
        return $this->metadata;
    }

    public function setMetadata(?array $metadata): self
    {
        $this->metadata = $metadata;

        return $this;
    }
}
