<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;
use Ambta\DoctrineEncryptBundle\Configuration\Encrypted;

/**
 * ConfigProp
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="config_props", "fsr_module"="config_props", "fsr_singular"="Config Prop"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     normalizationContext={"circular_reference_limit"=2},
 *     collectionOperations={
 *          "get" = { "path"="config_props", "security_post_denormalize" = "is_granted('read.config_props', object)" },
 *     },
 *     itemOperations={
 *          "get" = { "path"="config_props/{uuid}", "security" = "is_granted('read.config_props', object)" },
 *     }
 * )
 *
 * @Doctrine\ORM\Mapping\Table(name="config_prop", indexes={@Doctrine\ORM\Mapping\Index(name="config_props_id_idx", columns={"id"})})
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 */
class ConfigProp extends \App\Entity\Base\UuidBasedBaseEntity
{
    use \App\Traits\Indexable;

    /**
     * @var string|null
     *
     * @Doctrine\ORM\Mapping\Column(name="`value`", type="text", length=10485761, nullable=false, unique=false)
     * @Groups({"fsr_primary", "fsr_all", "value"})
     */
    protected $value;

    /**
     * @var \DateTime|null
     *
     * @Doctrine\ORM\Mapping\Column(name="`modifiedat`", type="datetime", nullable=true, unique=false)
     * @Groups({"fsr_primary", "fsr_all", "modifiedAt"})
     */
    protected $modifiedAt;

    /**
     * @var string|null
     *
     * @Doctrine\ORM\Mapping\Column(name="`key`", type="text", length=10485761, nullable=false, unique=true)
     * @Groups({"fsr_primary", "fsr_all", "key"})
     */
    protected $key;

    /**
     * @var string|null
     *
     * @Doctrine\ORM\Mapping\Column(name="`checksum`", type="text", length=10485761, nullable=true, unique=false)
     * @Groups({"fsr_primary", "fsr_all", "checksum", "fsr_encrypted"})
     * @Encrypted
     */
    protected $checksum;

    /**
     * @var string|null
     *
     * @Doctrine\ORM\Mapping\Column(name="`salt`", type="text", length=10485761, nullable=true, unique=false)
     * @Groups({"fsr_primary", "fsr_all", "salt", "fsr_encrypted"})
     * @Encrypted
     */
    protected $salt;


    /**
     * Set value
     *
     * @param string  $value
     *
     * @return Test
     */
    public function setValue($value = null)
    {
        $this->value = (string) $value;

        return $this;
    }

    /**
     * Get value
     *
     * @return string 
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * Set modifiedAt
     *
     * @param \DateTime  $modifiedAt
     *
     * @return Test
     */
    public function setModifiedAt($modifiedAt = null)
    {
        $this->modifiedAt = $modifiedAt;

        return $this;
    }

    /**
     * Get modifiedAt
     *
     * @return \DateTime 
     */
    public function getModifiedAt()
    {
        return $this->modifiedAt;
    }

    /**
     * Set key
     *
     * @param string  $key
     *
     * @return Test
     */
    public function setKey($key = null)
    {
        $this->key = $key;

        return $this;
    }

    /**
     * Get key
     *
     * @return string 
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * Set checksum
     *
     * @param string  $checksum
     *
     * @return Test
     */
    public function setChecksum($checksum = null)
    {
        $this->checksum = (string) $checksum;

        return $this;
    }

    /**
     * Get checksum
     *
     * @return string 
     */
    public function getChecksum()
    {
        return $this->checksum;
    }

    /**
     * Get the value of salt
     *
     * @return  string|null
     */
    public function getSalt()
    {
        return $this->salt;
    }

    /**
     * Set the value of salt
     *
     * @param  string|null  $salt
     *
     * @return  self
     */
    public function setSalt($salt)
    {
        $this->salt = $salt;

        return $this;
    }
}
