<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\EmailTemplateRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="email_templates", "fsr_module"="email_templates", "fsr_singular"="Email Template"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.email_templates', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.email_templates', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.email_templates', object)" },
 *          "put" = { "security" = "is_granted('update.email_templates', object)" },
 *          "delete" = { "security" = "is_granted('delete.email_templates', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=EmailTemplateRepository::class)
 * @ORM\Table(name="cyops.emailtemplate", schema="cyops")
 */
class EmailTemplate extends BaseEntity
{
    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "subject"})
     * @ORM\Column(type="string", length=1024, nullable=true)
     */
    private $subject;

    /**
     * @Groups({"fsr_primary", "fsr_all", "content"})
     * @ORM\Column(type="text", nullable=true)
     */
    private $content;

    /**
     * @Groups({"fsr_primary", "fsr_all", "visible"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $visible;


    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getSubject(): ?string
    {
        return $this->subject;
    }

    public function setSubject(?string $subject): self
    {
        $this->subject = $subject;

        return $this;
    }

    public function getContent(): ?string
    {
        return $this->content;
    }

    public function setContent(?string $content): self
    {
        $this->content = $content;

        return $this;
    }

    public function getVisible(): ?bool
    {
        return $this->visible;
    }

    public function setVisible(?bool $visible): self
    {
        $this->visible = $visible;

        return $this;
    }
}
