<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use App\Entity\Core\SolutionPack;
use Doctrine\Common\Collections\Collection;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\ExportTemplateRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="export_templates", "fsr_module"="security", "fsr_singular"="Export Template"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.security', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.security', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.security', object)" },
 *          "put" = { "security" = "is_granted('update.security', object)" },
 *          "delete" = { "security" = "is_granted('delete.security', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=ExportTemplateRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="export_templates", indexes={@Doctrine\ORM\Mapping\Index(name="export_templates_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="export_templates_id_search_idx", columns={"id"})})
 */
class ExportTemplate extends BaseEntity
{

    use \App\Traits\Indexable;
    use \App\Traits\Trackable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=1024, nullable=true)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "lastExportDate"})
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $lastExportDate;

    /**
     * @Groups({"fsr_all", "exportJobs"})
     * @ORM\OneToMany(targetEntity="ExportJob", mappedBy="template", cascade={"persist", "remove"})
     */
    private $exportJobs;

    /**
     * @Groups({"fsr_primary", "fsr_all", "options"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $options = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "type"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $type = 'Export Wizard';

    /**
     * @Groups({"fsr_primary","fsr_all", "solutionPack"})
     * @ORM\OneToOne(targetEntity="\App\Entity\Core\SolutionPack", mappedBy="template")
     */
    private $solutionPack;


    public function __construct()
    {
        $this->exportJobs = new ArrayCollection();
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getLastExportDate(): ?\DateTimeInterface
    {
        return $this->lastExportDate;
    }

    public function setLastExportDate(?\DateTimeInterface $lastExportDate): self
    {
        $this->lastExportDate = $lastExportDate;

        return $this;
    }

    /**
     * @return Collection|ExportJob[]
     */
    public function getExportJobs(): Collection
    {
        return $this->exportJobs;
    }

    public function addExportJob(ExportJob $exportJob): self
    {
        if (!$this->exportJobs->contains($exportJob)) {
            $this->exportJobs[] = $exportJob;
        }

        return $this;
    }

    public function removeExportJob(ExportJob $exportJob): self
    {
        if ($this->exportJobs->contains($exportJob)) {
            $this->exportJobs->removeElement($exportJob);
        }

        return $this;
    }

    public function getOptions(): ?array
    {
        return $this->options;
    }

    public function setOptions(?array $options): self
    {
        $this->options = $options;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getSolutionPack()
    {
        return $this->solutionPack;
    }

    public function setSolutionPack(SolutionPack $solutionPack = null): self
    {
        $this->solutionPack = $solutionPack;
        return $this;
    }
}
