<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\PgFileRepository;
use Ramsey\Uuid\Uuid;

/**
 * @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="files", "fsr_module"="files", "fsr_singular"="file"},
 *     collectionOperations={
 *          "get" = { "path"="pgfile", "security_post_denormalize" = "is_granted('read.files', object)" },
 *          "post" = { "path"="pgfile", "security_post_denormalize" = "is_granted('create.files', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "path"="pgfile/{uuid}", "security" = "is_granted('read.files', object)" },
 *          "put" = { "path"="pgfile/{uuid}", "security" = "is_granted('update.files', object)" },
 *          "delete" = { "path"="pgfile/{uuid}", "security" = "is_granted('delete.files', object)" }
 *     }
 * )
 * @Doctrine\ORM\Mapping\Entity(repositoryClass=PgFileRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="pgfile")
 */
class PgFile
{
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->uuid = Uuid::uuid4()->toString();
    }

    /**
     * @Doctrine\ORM\Mapping\Id
     * @Doctrine\ORM\Mapping\Column(name="uuid", type="string", length=150, nullable=false)
     */
    protected $uuid;

    /**
     * @Doctrine\ORM\Mapping\Column(type="blob", nullable=true)
     */
    private $file;

    /**
     * Set uuid
     *
     * @param integer $uuid
     * @return BaseEntity
     */
    public function setUuid()
    {
        $this->uuid = Uuid::uuid4()->toString();

        return $this;
    }

    /**
     * Get uuid
     *
     * @return integer
     */
    public function getUuid()
    {
        return $this->uuid;
    }


    public function getFile()
    {
        return $this->file;
    }

    public function setFile($file): self
    {
        $this->file = $file;

        return $this;
    }
}
