<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use App\Traits\Indexable;
use App\Traits\Importable;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use App\Repository\PicklistRepository;
use ApiPlatform\Core\Annotation\ApiProperty;
use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="picklists", "fsr_module"="application", "fsr_singular"="Picklist"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=PicklistRepository::class)
 * @ORM\EntityListeners({"App\EventSubscriber\PicklistListener"})
 * @ORM\Table(name="picklists")
 */
class Picklist extends BaseEntity
{
    use Indexable;
    use Importable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "itemValue"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $itemValue;

    /**
     * @Groups({"fsr_primary", "fsr_all", "orderIndex"})
     * @ORM\Column(type="integer", nullable=true)
     */
    private $orderIndex;

    /**
     * @Groups({"fsr_primary", "fsr_all", "color"})
     * @ORM\Column(type="string", length=10, nullable=true)
     */
    private $color;

    /**
     * @Groups({"fsr_primary", "fsr_all", "icon"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $icon;

    /**
     * @Groups({"fsr_primary", "fsr_all", "listName"})
     * @ORM\ManyToOne(targetEntity=PicklistName::class, inversedBy="picklists")
     * @ORM\JoinColumn(name="listName", referencedColumnName="uuid", nullable=false)
     * @ApiProperty(readableLink=false)
     */
    private $listName;


    public function getItemValue(): ?string
    {
        return $this->itemValue;
    }

    public function setItemValue(?string $itemValue): self
    {
        $this->itemValue = $itemValue;

        return $this;
    }

    public function getOrderIndex(): ?int
    {
        return $this->orderIndex;
    }

    public function setOrderIndex(?int $orderIndex): self
    {
        $this->orderIndex = $orderIndex;

        return $this;
    }

    public function getColor(): ?string
    {
        return $this->color;
    }

    public function setColor(?string $color): self
    {
        $this->color = $color;

        return $this;
    }

    public function getIcon(): ?string
    {
        return $this->icon;
    }

    public function setIcon(?string $icon): self
    {
        $this->icon = $icon;

        return $this;
    }

    public function getListName(): ?PicklistName
    {
        return $this->listName;
    }

    public function setListName(?PicklistName $listName): self
    {
        $this->listName = $listName;

        return $this;
    }
}
