<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use App\Traits\Indexable;
use App\Traits\Importable;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use App\Repository\PicklistNameRepository;
use Doctrine\Common\Collections\Collection;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="picklist_names", "fsr_module"="application", "fsr_singular"="Picklist Names"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=PicklistNameRepository::class)
 * @ORM\EntityListeners({"App\EventSubscriber\PicklistNameListener"})
 * @Doctrine\ORM\Mapping\Table(name="picklist_names")
 */
class PicklistName extends BaseEntity
{

    use Indexable;
    use Importable;
    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=false, unique=true)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "system"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $system = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "picklists"})
     * @ORM\OneToMany(targetEntity=Picklist::class, mappedBy="listName", orphanRemoval=true, cascade={"persist", "remove"})
     */
    private $picklists;

    public function __construct()
    {
        $this->picklists = new ArrayCollection();
    }


    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getSystem(): ?bool
    {
        return $this->system;
    }

    public function setSystem(?bool $system): self
    {
        $this->system = $system;

        return $this;
    }

    /**
     * @return Collection|Picklist[]
     */
    public function getPicklists(): Collection
    {
        return $this->picklists;
    }

    public function addPicklist(Picklist $picklist): self
    {
        if (!$this->picklists->contains($picklist)) {
            $this->picklists[] = $picklist;
            $picklist->setListName($this);
        }

        return $this;
    }

    public function removePicklist(Picklist $picklist): self
    {
        if ($this->picklists->contains($picklist)) {
            $this->picklists->removeElement($picklist);
            // set the owning side to null (unless already changed)
            if ($picklist->getListName() === $this) {
                $picklist->setListName(null);
            }
        }

        return $this;
    }
}
