<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\ProxyRepository;
use Ambta\DoctrineEncryptBundle\Configuration\Encrypted;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * @ORM\Entity(repositoryClass=ProxyRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="cyops.proxy", schema="cyops")
 * @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="proxy", "fsr_module"="security", "fsr_singular"="Proxy"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     normalizationContext={"circular_reference_limit"=2},
 *     collectionOperations={
 *          "get" = { "path"="proxy", "security_post_denormalize" = "is_granted('read.security', object)" },
 *     }
 * )
 */
class Proxy extends BaseEntity
{

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $toggle;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @Encrypted
     * @ORM\Column(type="string", length=4096, nullable=true)
     */
    private $proxies;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $noProxy = [];

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $other = [];


    public function getToggle(): ?bool
    {
        return $this->toggle;
    }

    public function setToggle(?bool $toggle): self
    {
        $this->toggle = $toggle;

        return $this;
    }

    public function getProxies(): ?string
    {
        return $this->proxies;
    }

    public function setProxies(?string $proxies): self
    {
        $this->proxies = $proxies;

        return $this;
    }

    public function getNoProxy(): ?array
    {
        return $this->noProxy;
    }

    public function setNoProxy(?array $noProxy): self
    {
        $this->noProxy = $noProxy;

        return $this;
    }

    public function getOther(): ?array
    {
        return $this->other;
    }

    public function setOther(?array $other): self
    {
        $this->other = $other;

        return $this;
    }
}
