<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Core;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\TagRepository;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="tags", "fsr_module"="application", "fsr_singular"="Tag"},
 *     denormalizationContext={"groups"={"fsr_all", "fsr_primary"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "path"="tags", "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "path"="tags", "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "path"="tags/{uuid}", "security" = "is_granted('read.application', object)" }
 *     }
 * )
 * @Doctrine\ORM\Mapping\Entity(repositoryClass=TagRepository::class)
 * @Doctrine\ORM\Mapping\Table(name="tags")
 */
class Tag
{
    /**
     * @Doctrine\ORM\Mapping\Id
     * @Doctrine\ORM\Mapping\Column(name="uuid", type="string", length=150, nullable=false)
     * @Groups({"fsr_primary", "fsr_all", "uuid"})
     */
    protected $uuid;

    /**
     * Set uuid
     *
     * @return BaseEntity
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;

        return $this;
    }

    /**
     * Get uuid
     *
     */
    public function getUuid()
    {
        return $this->uuid;
    }
}
