<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ModelMetadata;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use App\Repository\ClusterCommandRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="cluster_commands", "fsr_module"="clustercommand", "fsr_singular"="cluster_command"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.clustercommand', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.clustercommand', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.clustercommand', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=ClusterCommandRepository::class)
 * @ORM\Table(name="cyops.clustercommand")
 */
class ClusterCommand extends BaseEntity
{


    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=150, nullable=false)
     */
    private $nodeId;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $commandId;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $sequenceId;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=20, nullable=true)
     */
    private $status;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=50, nullable=true)
     */
    private $result;



    public function getNodeId(): ?string
    {
        return $this->nodeId;
    }

    public function setNodeId(?string $nodeId): self
    {
        $this->nodeId = $nodeId;

        return $this;
    }

    public function getCommandId(): ?string
    {
        return $this->commandId;
    }

    public function setCommandId(?string $commandId): self
    {
        $this->commandId = $commandId;

        return $this;
    }

    public function getSequenceId(): ?string
    {
        return $this->sequenceId;
    }

    public function setSequenceId(?string $sequenceId): self
    {
        $this->sequenceId = $sequenceId;

        return $this;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(?string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getResult(): ?string
    {
        return $this->result;
    }

    public function setResult(?string $result): self
    {
        $this->result = $result;

        return $this;
    }
}
