<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ModelMetadata;

use App\Traits\Importable;
use Doctrine\ORM\Mapping as ORM;
use App\Validator\KeywordConstraint;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Doctrine\Common\Collections\Collection;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use App\Entity\ModelMetadata\AttribModelMetadata;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="model_metadatas", "fsr_module"="application", "fsr_singular"="model_metadata"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass="\Doctrine\ORM\EntityRepository")
 * @ORM\Table(name="model_metadatas", uniqueConstraints={@UniqueConstraint(name="field_model_uniques", columns={"type"})})
 */
class ModelMetadata extends Metadatas
{
    use Importable;
    /**
     * @Groups({"fsr_primary", "fsr_all", "type"})
     * @KeywordConstraint(groups={"Default", "fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=500, nullable=true)
     * @Symfony\Component\Validator\Constraints\NotBlank(groups={"Default", "fsr_primary", "fsr_all"})
     */
    private $type;

    /**
     * @Groups({"fsr_primary", "fsr_all", "parentType"})
     * @KeywordConstraint(groups={"Default", "fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $parentType;

    /**
     * @Groups({"fsr_primary", "fsr_all", "tableName"})
     * @KeywordConstraint(groups={"Default", "fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=500, nullable=true)
     * @Symfony\Component\Validator\Constraints\NotBlank(groups={"Default", "fsr_primary", "fsr_all"})
     */
    private $tableName;

    /**
     * @Groups({"fsr_primary", "fsr_all", "ownable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $ownable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "userOwnable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $userOwnable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "queueable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $queueable = false;


    /**
     * @Groups({"fsr_primary", "fsr_all", "trackable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $trackable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "taggable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = true})
     */
    private $taggable = true;

    /**
     * @Groups({"fsr_primary", "fsr_all", "peerReplicable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $peerReplicable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "defaultSort"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $defaultSort = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "uniqueConstraint"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $uniqueConstraint = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "indexable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $indexable = true;

    /**
     * @Groups({"fsr_primary", "fsr_all", "writable"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $writable = true;

    /**
     * @Groups({"fsr_all", "attributes"})
     * @KeywordConstraint(groups={"Default", "fsr_primary", "fsr_all"})
     * @ORM\OneToMany(targetEntity="AttribModelMetadata", mappedBy="sattrib", cascade={"persist", "remove"})
     * @Doctrine\ORM\Mapping\OrderBy({
     *     "orderIndex"="ASC", "name"="ASC",
     * })
     */
    private $attributes;

    /**
     * @Groups({"fsr_primary", "fsr_all", "system"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $system = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "module"})
     * @ORM\Column(type="string", length=500, nullable=true)
     */
    private $module;

    /**
     * @Groups({"fsr_primary", "fsr_all", "softDeleteable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $softDeleteable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "archivable"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $archivable = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "paused"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = false})
     */
    private $paused = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "partitionBy"})
     * @ORM\Column(type="string", length=100, nullable=true)
     */
    private $partitionBy = 'modifyDate';

    /**
     * @Groups({"fsr_primary", "fsr_all", "archivalCriteria"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $archivalCriteria = ["keep_primary_data_for" => 24, "delete_primary_data_after" => 60];

    /**
     * @Groups({"fsr_primary", "fsr_all", "archivalFilters"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $archivalFilters = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "replicationFilters"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $replicationFilters = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "enableReplication"})
     * @ORM\Column(type="boolean", nullable=true , options={"default" = true})
     */
    private $enableReplication = true;

    public function __construct()
    {
        $this->attributes = new ArrayCollection();
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getParentType(): ?string
    {
        return $this->parentType;
    }

    public function setParentType(?string $parentType): self
    {
        $this->parentType = $parentType;

        return $this;
    }

    public function getTableName(): ?string
    {
        return $this->tableName;
    }

    public function setTableName(?string $tableName): self
    {
        $this->tableName = $tableName;

        return $this;
    }

    public function getOwnable(): ?bool
    {
        return $this->ownable;
    }

    public function setOwnable(bool $ownable): self
    {
        $this->ownable = $ownable;

        return $this;
    }

    public function getTrackable(): ?bool
    {
        return $this->trackable;
    }

    public function setTrackable(bool $trackable): self
    {
        $this->trackable = $trackable;

        return $this;
    }

    public function getTaggable(): ?bool
    {
        return $this->taggable;
    }

    public function setTaggable(bool $taggable): self
    {
        $this->taggable = $taggable;

        return $this;
    }

    public function getPeerReplicable(): ?bool
    {
        return $this->peerReplicable;
    }

    public function setPeerReplicable(bool $peerReplicable): self
    {
        $this->peerReplicable = $peerReplicable;

        return $this;
    }

    public function getDefaultSort(): ?array
    {
        return $this->defaultSort;
    }

    public function setDefaultSort(?array $defaultSort): self
    {
        $this->defaultSort = $defaultSort;

        return $this;
    }

    public function getUniqueConstraint(): ?array
    {
        return $this->uniqueConstraint;
    }

    public function setUniqueConstraint(?array $uniqueConstraint): self
    {
        $this->uniqueConstraint = $uniqueConstraint;

        return $this;
    }

    public function getIndexable(): ?bool
    {
        return $this->indexable;
    }

    public function setIndexable(bool $indexable): self
    {
        $this->indexable = $indexable;

        return $this;
    }

    public function getWritable(): ?bool
    {
        return $this->writable;
    }

    public function setWritable(bool $writable): self
    {
        $this->writable = $writable;

        return $this;
    }

    /**
     * @return Collection|AttribModelMetadata[]
     */
    public function getAttributes(): Collection
    {
        return $this->attributes;
    }

    public function addAttribute(AttribModelMetadata $attribute): self
    {
        if (!$this->attributes->contains($attribute)) {
            $this->attributes[] = $attribute;
            $attribute->setSattrib($this);
        }

        return $this;
    }

    public function removeAttribute(AttribModelMetadata $attribute): self
    {
        if ($this->attributes->contains($attribute)) {
            $this->attributes->removeElement($attribute);
            // set the owning side to null (unless already changed)
            if ($attribute->getSattrib() === $this) {
                $attribute->setSattrib(null);
            }
        }

        return $this;
    }

    /**
     * Get the value of system
     */
    public function getSystem()
    {
        return $this->system;
    }

    /**
     * Set the value of system
     *
     * @return  self
     */
    public function setSystem($system)
    {
        $this->system = $system;

        return $this;
    }

    /**
     * Get the value of module
     */
    public function getModule()
    {
        return $this->module;
    }

    /**
     * Set the value of module
     *
     * @return  self
     */
    public function setModule($module)
    {
        if (!is_null($module)) {
            $this->module = $module;
            return $this;
        }
        $this->module = $this->getType();
        return $this;
    }


    /**
     * Get the value of queueable
     */
    public function getQueueable()
    {
        return $this->queueable;
    }

    /**
     * Set the value of queueable
     *
     * @return  self
     */
    public function setQueueable($queueable)
    {
        $this->queueable = $queueable;

        return $this;
    }

    /**
     * Get the value of userOwnable
     */
    public function getUserOwnable()
    {
        return $this->userOwnable;
    }

    /**
     * Set the value of userOwnable
     *
     * @return  self
     */
    public function setUserOwnable($userOwnable)
    {
        $this->userOwnable = $userOwnable;

        return $this;
    }

    /**
     * Get the value of archivable
     */
    public function getArchivable()
    {
        return $this->archivable;
    }

    /**
     * Set the value of archivable
     *
     * @return  self
     */
    public function setArchivable($archivable)
    {
        $this->archivable = $archivable;

        return $this;
    }

    /**
     * Get the value of partitionBy
     */
    public function getPartitionBy()
    {
        return $this->partitionBy;
    }

    /**
     * Set the value of partitionBy
     *
     * @return  self
     */
    public function setPartitionBy($partitionBy)
    {
        $this->partitionBy = $partitionBy;

        return $this;
    }

    /**
     * Get the value of archivalCriteria
     */
    public function getArchivalCriteria()
    {
        return $this->archivalCriteria;
    }

    /**
     * Set the value of archivalCriteria
     *
     * @return  self
     */
    public function setArchivalCriteria($archivalCriteria)
    {
        $this->archivalCriteria = $archivalCriteria;

        return $this;
    }

    /**
     * Get the value of archivalFilters
     */
    public function getArchivalFilters()
    {
        return $this->archivalFilters;
    }

    /**
     * Set the value of archivalFilters
     *
     * @return  self
     */
    public function setArchivalFilters($archivalFilters)
    {
        $this->archivalFilters = $archivalFilters;

        return $this;
    }

    /**
     * Get the value of replicationFilters
     */
    public function getReplicationFilters()
    {
        return $this->replicationFilters;
    }

    /**
     * Set the value of replicationFilters
     *
     * @return  self
     */
    public function setReplicationFilters($replicationFilters)
    {
        $this->replicationFilters = $replicationFilters;

        return $this;
    }

    /**
     * Get the value of paused
     */
    public function getPaused()
    {
        return $this->paused;
    }

    /**
     * Set the value of paused
     *
     * @return  self
     */
    public function setPaused($paused)
    {
        $this->paused = $paused;
        return $this;
    }

    /** Get the value of softDeleteable
     */
    public function getSoftDeleteable()
    {
        return $this->softDeleteable;
    }

    /**
     * Set the value of softDeleteable
     *
     * @return  self
     */
    public function setSoftDeleteable($softDeleteable)
    {
        $this->softDeleteable = $softDeleteable;
        return $this;
    }

    /**
     * Get the value of enableReplication
     */
    public function getEnableReplication()
    {
        return $this->enableReplication;
    }

    /**
     * Set the value of enableReplication
     *
     * @return  self
     */
    public function setEnableReplication($enableReplication)
    {
        $this->enableReplication = $enableReplication;

        return $this;
    }
}
