<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\Query;


use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping\UniqueConstraint;
use App\Entity\ModelMetadata\ModelMetadata;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="queries", "fsr_module"="application", "fsr_singular"="Query"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Table(name="queries", uniqueConstraints={@UniqueConstraint(name="query_unique", columns={"name", "models", "assignee"})})
 * @ORM\Entity
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorMap({"UserQuery"="UserQuery", "SystemQuery"="SystemQuery"})
 * @ORM\DiscriminatorColumn(name="type", type="string", length=20)
 */
abstract class Query extends BaseEntity
{
    const KEYWORD_AGGREGATES = 'aggregates';
    const KEYWORD_SORT = 'sort';
    const KEYWORD_LOGIC = 'logic';
    const KEYWORD_FILTERS = 'filters';
    const SHOW_DELETED = '$showDeleted';

    /**
     * @var string
     *
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @Doctrine\ORM\Mapping\Column(name="`name`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     */
    protected $name;

    /**
     *
     * @Groups({"fsr_primary", "fsr_all", "models"})
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="App\Entity\ModelMetadata\ModelMetadata", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="models", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     */
    protected $models;

    /**
     * @var integer
     *
     * @Groups({"fsr_primary", "fsr_all", "resultCacheSeconds"})
     * @Doctrine\ORM\Mapping\Column(name="`resultcacheseconds`", type="integer", nullable=true, unique=false)
     */
    protected $resultCacheSeconds = 0;

    /**
     * @var string
     *
     * @Groups({"fsr_primary", "fsr_all", "query"})
     * @Doctrine\ORM\Mapping\Column(name="`query`", type="json_array", nullable=true, unique=false)
     */
    protected $query;

    /**
     * @var string
     *
     * @Groups({"fsr_primary", "fsr_all", "assignee"})
     * @ORM\Column(name="`assignee`", type="string", nullable=true, unique=false)
     */
    protected $assignee = '';

    /**
     * @var boolean
     * 
     * @Groups({"fsr_primary", "fsr_all", "advanced"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    protected $advanced;

    /**
     * @return string
     */
    public function getAssignee()
    {
        return $this->assignee;
    }

    /**
     * @param string $assignee
     */
    public function setAssignee($assignee)
    {
        $this->assignee = $assignee;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return ModelMetadata[]
     */
    public function getModels()
    {
        return $this->models;
    }

    /**
     * @param ModelMetadata[] $models
     */
    public function setModels($models)
    {
        $this->models = $models;
    }

    /**
     * @return int
     */
    public function getResultCacheSeconds()
    {
        return $this->resultCacheSeconds;
    }

    /**
     * @param int $resultCacheSeconds
     */
    public function setResultCacheSeconds($resultCacheSeconds)
    {
        $this->resultCacheSeconds = $resultCacheSeconds;
    }

    /**
     * @return array
     */
    public function getQuery()
    {
        return $this->query;
    }

    /**
     * @param array $query
     */
    public function setQuery($query)
    {
        $this->query = $query;
    }

    /**
     * @return boolean
     */
    public function getAdvanced()
    {
        return $this->advanced;
    }

    /**
     * @param boolean $advanced
     */
    public function setAdvanced($advanced)
    {
        $this->advanced = $advanced;
    }
}
