<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ViewTemplate;

use App\Traits\Trackable;
use App\Traits\Importable;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use App\Entity\Authorization\Role;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Doctrine\Common\Collections\Collection;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\GenericTemplateRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="generic_templates", "fsr_module"="application", "fsr_singular"="generic_template"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=GenericTemplateRepository::class)
 * @ORM\Table(name="generic_template")
 * @ORM\Entity
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorMap({"GenericTemplate"= "GenericTemplate","Dashboard"="Dashboard", "Reporting"="Reporting"})
 * @ORM\DiscriminatorColumn(name="genericType", type="string", length=20)
 */
class GenericTemplate extends BaseEntity
{

    use Trackable;
    use Importable;
    /**
     * @Groups({"fsr_primary", "fsr_all", "displayName"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $displayName;

    /**
     * @Groups({"fsr_primary", "fsr_all", "type"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $type;

    /**
     * @Groups({"fsr_primary", "fsr_all", "templateType"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $templateType;

    /**
     * @Groups({"fsr_all", "roles"})
     * @ORM\OrderBy({"name" = "ASC"})
     * @ORM\ManyToMany(targetEntity=Role::class)
     */
    private $roles;

    /**
     * @Groups({"fsr_primary", "fsr_all", "parentTemplateId"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $parentTemplateId;

    /**
     * @Groups({"fsr_primary", "fsr_all", "config"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $config = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "filterArray"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $filterArray = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "updateDate"})
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $updateDate;

    public function __construct()
    {
        $this->roles = new ArrayCollection();
    }

    public function getDisplayName(): ?string
    {
        return $this->displayName;
    }

    public function setDisplayName(?string $displayName): self
    {
        $this->displayName = $displayName;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getTemplateType(): ?string
    {
        return $this->templateType;
    }

    public function setTemplateType(?string $templateType): self
    {
        $this->templateType = $templateType;

        return $this;
    }

    /**
     * @return Collection|Role[]
     */
    public function getRoles(): Collection
    {
        return $this->roles;
    }

    public function addRole(Role $role): self
    {
        if (!$this->roles->contains($role)) {
            $this->roles[] = $role;
        }

        return $this;
    }

    public function removeRole(Role $role): self
    {
        if ($this->roles->contains($role)) {
            $this->roles->removeElement($role);
        }

        return $this;
    }

    public function getParentTemplateId(): ?string
    {
        return $this->parentTemplateId;
    }

    public function setParentTemplateId(?string $parentTemplateId): self
    {
        $this->parentTemplateId = $parentTemplateId;

        return $this;
    }

    public function getConfig(): ?array
    {
        return $this->config;
    }

    public function setConfig(?array $config): self
    {
        $this->config = $config;

        return $this;
    }

    public function getFilterArray(): ?array
    {
        return $this->filterArray;
    }

    public function setFilterArray(?array $filterArray): self
    {
        $this->filterArray = $filterArray;

        return $this;
    }

    public function getUpdateDate(): ?\DateTimeInterface
    {
        return $this->updateDate;
    }

    public function setUpdateDate(?\DateTimeInterface $updateDate): self
    {
        $this->updateDate = $updateDate;

        return $this;
    }
}
