<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ViewTemplate;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Entity\Actors\Actor;
use App\Entity\Base\BaseEntity;
use App\Entity\ViewTemplate\GenericTemplate;
use App\Repository\UserDefaultTemplateRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="user_default_templates", "fsr_module"="application", "fsr_singular"="user_default_template"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=UserDefaultTemplateRepository::class)
 * @ORM\Table(name="default_template", uniqueConstraints={@UniqueConstraint(name="default_user_unique", columns={"user_id", "templateId"})})
 */
class UserDefaultTemplate extends BaseEntity
{

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $type;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\ManyToOne(targetEntity="App\Entity\ViewTemplate\GenericTemplate")
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="templateId", referencedColumnName="uuid", onDelete="CASCADE")
     * })
     */
    private $templateId;

    /**
     * @Groups({"fsr_primary", "fsr_all"})
     * @ORM\ManyToOne(targetEntity=Actor::class)
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="user_id", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     */
    private $userId;

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getTemplateId(): ?GenericTemplate
    {
        return $this->templateId;
    }

    public function setTemplateId(?GenericTemplate $templateId): self
    {
        $this->templateId = $templateId;

        return $this;
    }

    public function getUserId(): ?Actor
    {
        return $this->userId;
    }

    public function setUserId(?Actor $userId): self
    {
        $this->userId = $userId;

        return $this;
    }
}
