<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Custom\Agent;
use App\Entity\Custom\Tenant;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\EntityManager;
use App\Entity\Custom\Appliance;
use App\Entity\Authorization\Team;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class AgentTenantOwnershipCascadingSubscriber implements EventSubscriberInterface
{
    public const POSTMAN_CONFIG_DIRECTORY = '/opt/cyops-routing-agent/postman/configs/agents/';
    public const AGENT_ROLE_UUID = '3006d2f2-2f3a-4ae8-a36c-03ae75bf17fb';

    /** @var DasClient */
    protected $dasClient;

    public function __construct(protected EntityManagerInterface $entityManager, protected ContainerInterface $container)
    {
        $this->dasClient = $this->container->get("app.proxy.client.das");
    }

    public static function getSubscribedEvents()
    {
        // Adding this on POST WRITE PRIORITY in order so data is persisted in database and then handling of data can be done.
        return [
            KernelEvents::VIEW => ['handleUpdate', 33],
        ];
    }

    protected function updateApplianceTeamsOnAgentOwnersChange($previousName, $teams, $agentName, $previousTeam)
    {
        $applianceRepo = $this->entityManager->getRepository(Appliance::class);
        $appliance = $applianceRepo->findBy(["name" => $previousName]);
        if ($appliance) {
            $appliance = $appliance[0];
            foreach ($previousTeam as $oldTeam) {
                $appliance->removeTeam($oldTeam);
            }
            foreach ($teams as $team) {
                $appliance->addTeam($team);
            }
            $appliance->setName($agentName);
            $this->entityManager->persist($appliance);
            $this->entityManager->flush();
        }
    }

    protected function propagateAgentTeamToTenant($teams, $agentId, $previousOwner)
    {
        $tenantRepo = $this->entityManager->getRepository(Tenant::class);
        $tenant = $tenantRepo->findBy(["tenantId" => $agentId]);
        if ($tenant) {
            $tenant = $tenant[0];
            foreach ($previousOwner as $oldOwner) {
                $tenant->removeOwner($oldOwner);
            }
            foreach ($teams as $team) {
                $tenant->addOwner($team);
            }
            $this->entityManager->persist($tenant);
            $this->entityManager->flush();
        }
    }

    protected function propagateTenantTeamtoAgent($tenantId, $teams, $tenantName, $agentName, $agentPreviousName, $previousOwner)
    {
        $agentRepo = $this->entityManager->getRepository(Agent::class);
        $agent = $agentRepo->findBy(array("agentId" => $tenantId));

        if ($agent) {
            $agent = $agent[0];
            foreach ($previousOwner as $oldOwner) {
                $agent->removeOwner($oldOwner);
            }
            foreach ($teams as $team) {
                $agent->addOwner($team);
            }
            $agent->setName($tenantName);
            $this->entityManager->persist($agent);
            $this->entityManager->flush();
            $this->updateApplianceTeamsOnAgentOwnersChange($agentPreviousName, $teams, $agentName, $previousOwner);
        }
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleUpdate(ViewEvent $event)
    {
        $data = $event->getControllerResult();
        $request = $event->getRequest();
        $method = $request->getMethod();
        if ($method != Request::METHOD_PUT || !$data instanceof BaseEntity) {
            return;
        }
        $previousData = $request->attributes->get('previous_data');
        $fsr_normalized_previous_data = $request->attributes->get('fsr_previous_data');
        $entityClass = $data::class;
        if (!($entityClass == Agent::class || $entityClass == Tenant::class) || is_null($previousData)) {
            return;
        }

        if ($entityClass == Agent::class) {
            $previousName = 'Agent' . '-' . $previousData->getName();
            $previousOwner = array_key_exists('owners', $fsr_normalized_previous_data) ? $this->getTeamPersistedObject($fsr_normalized_previous_data['owners']) : [];
            $agentId = $previousData->getAgentId();

            $name = $data->getName();
            $agentName = 'Agent' . '-' . $name;
            $teams = $data->getOwners();

            $this->updateApplianceTeamsOnAgentOwnersChange($previousName, $teams, $agentName, $previousOwner);
            $this->propagateAgentTeamToTenant($teams, $agentId, $previousOwner);
        }
        if ($entityClass == Tenant::class) {
            $tenantId = $previousData->getTenantId();
            $previousName = 'Agent' . '-' . $previousData->getName();
            $previousOwner = array_key_exists('owners', $fsr_normalized_previous_data) ? $this->getTeamPersistedObject($fsr_normalized_previous_data['owners']) : [];
            $tenantName = $data->getName();
            $name = $data->getName();
            $agentName = 'Agent' . '-' . $name;
            $teams = $data->getOwners();
            $this->propagateTenantTeamtoAgent($tenantId, $teams, $tenantName, $agentName, $previousName, $previousOwner);
        }
    }

    private function getTeamPersistedObject($ownerIris)
    {
        $ownerUuid = [];
        foreach ($ownerIris as $owner) {
            $explode = explode('/', $owner);
            array_push($ownerUuid, end($explode));
        }
        $TeamRepo = $this->entityManager->getRepository(Team::class);
        return $TeamRepo->findBy(["uuid" => $ownerUuid]);
    }
}
