<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Service\UtilityService;
use App\Entity\Core\PicklistName;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use ApiPlatform\Core\EventListener\EventPriorities;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

class CapturePreUpdateRelations implements EventSubscriberInterface
{
    public function __construct(private SerializerInterface $serializer, private ParameterBagInterface $parameters)
    {
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::REQUEST => ['handlePreUpdateData', EventPriorities::POST_READ]
        ];
    }

    public function handlePreUpdateData(RequestEvent $event)
    {
        $request = $event->getRequest();
        $request_method = $request->getMethod();
        if ($request_method == Request::METHOD_GET) {
            return;
        }
        $previousData = $request->attributes->get('previous_data');
        if (!is_null($previousData) && !empty($previousData)) {
            $requestBody = json_decode($request->getContent());
            if ($request_method == Request::METHOD_DELETE) {
                $groups = ["fsr_all", "fsr_primary", "password"];
            } else {
                $groups = UtilityService::identifyChangeFields($requestBody);
            }
            $max_relation_count = 100;
            if ($this->parameters->has('max_relation_count')) {
                $max_relation_count = $this->parameters->get('max_relation_count');
            }
            if (in_array($previousData::class, [PicklistName::class])) {
                $normalizedPreviousData = $this->serializer->normalize(
                    $previousData,
                    'jsonld',
                    ["relationships" => true, "groups" => $groups, "fsr_max_relation_count" => $max_relation_count, "ignore_html_escape" => true]
                );
            } else {
                $normalizedPreviousData = $this->serializer->normalize(
                    $previousData,
                    'jsonld',
                    ["asJson" => true, "relationships" => true, "groups" => $groups, "fsr_max_relation_count" => $max_relation_count, "ignore_html_escape" => true]
                );
            }
            $request->attributes->set('fsr_previous_data', $normalizedPreviousData);
            $request->attributes->set('fsr_change_fields', $groups);
        }
    }
}
