<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use Doctrine\Common\EventArgs;
use Gedmo\SoftDeleteable\SoftDeleteableListener;
use Symfony\Component\HttpFoundation\RequestStack;
use App\Entity\Settings\SystemSettings;
use Doctrine\ORM\EntityManagerInterface;
use App\Providers\PermissionsDictionaryBuilder;
use App\Security\Voter\FsrVoter;

class CustomSoftDeleteableListener extends SoftDeleteableListener
{
    public function __construct(private FsrVoter $fsrVoter, private EntityManagerInterface $entityManager, private RequestStack $requestStack)
    {
    }

    /**
     * If it's a SoftDeleteable object, and user has requested for hard delete
     * skip soft deleting of object and proceed with permanent delete
     *
     * @return void
     */
    public function onFlush(EventArgs $args)
    {
        $request = $this->requestStack->getCurrentRequest();
        if (!$request) {
            return;
        }
        $queryParams = $request->query->all();
        $systemRepository = $this->entityManager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();
        $publicValues = $systemSetting[0]->getPublicValues();
        $restrictHardDelete = false;
        if (is_array($publicValues) && array_key_exists('bin_ttl_config', $publicValues) && is_array($publicValues['bin_ttl_config']) && array_key_exists('restrictHardDelete', $publicValues['bin_ttl_config'])) {
            $restrictHardDelete = $publicValues['bin_ttl_config']['restrictHardDelete'];
        }
        $deletePerm = $this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_DELETE . '.' . 'application');
        if (array_key_exists('$hardDelete', $queryParams) && $queryParams['$hardDelete'] && (!$restrictHardDelete || $deletePerm)) {
            return;
        }
        parent::onFlush($args);
    }
}
