<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Core\PgFile;
use InvalidArgumentException;
use App\Traits\SoftDeleteable;
use App\Entity\Base\BaseEntity;
use App\Service\MetadataHelper;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class FileDeleteSubscriber implements EventSubscriberInterface
{
    private $inflector;
    public function __construct(protected ContainerInterface $containers, protected IriConverterInterface $iriConverter, protected EntityManagerInterface $entityManager, protected MetadataHelper $metadataHelper)
    {
        $this->inflector = InflectorFactory::create()->build();
    }


    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['FileDelete', 33]
        ];
    }

    public function isSoftDeleteable($entity_class)
    {
        return in_array(SoftDeleteable::class, class_uses($entity_class));
    }

    public function FileDelete(ViewEvent $event): void
    {
        $object = $event->getControllerResult();
        $request = $event->getRequest();
        $method = $request->getMethod();
        if (($method && $method != Request::METHOD_DELETE) || !$object instanceof BaseEntity) {
            return;
        }
        $entityClass = $entityClass = $object::class;

        if ($this->isSoftDeleteable($entityClass) && is_null($object->getDeletedAt())) {
            return;
        }
        $entityFieldMap = $this->metadataHelper->getEntityFields($entityClass) ?? null;
        if ($entityFieldMap && (is_countable($entityFieldMap['fileFields']) ? count($entityFieldMap['fileFields']) : 0)) {
            // Added extra check because of missing type map, can be removed once all entity are mapped
            foreach ($entityFieldMap['fileFields'] as $key => $value) {
                $camelized = $this->inflector->camelize($value);
                $getMethod = 'get' . $camelized;
                if ($object->{$getMethod}()) {
                    try {
                        $fileObject = $this->iriConverter->getItemFromIri($object->{$getMethod}());
                        $pgFileRepo = $this->entityManager->getRepository(PgFile::class);
                        if ($fileObject->getFile()) {
                            $pgFile = $pgFileRepo->findOneBy(["uuid" => $fileObject->getFile()]);
                            $this->entityManager->remove($pgFile);
                            $this->entityManager->flush();
                            $this->entityManager->remove($fileObject);
                            $this->entityManager->flush();
                        }
                    } catch (InvalidArgumentException | \Exception $e) {
                        $value = null;
                    }
                }
            }
        }
    }
}
