<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;


use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\Event\LifecycleEventArgs;
use App\Service\RuleService;
use App\Traits\SoftDeleteable;
use App\Entity\Custom\Queue;
use App\Entity\ViewTemplate\SystemViewTemplate;
use Doctrine\ORM\Mapping as ORM;

class QueueListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    private $ruleService;

    public function __construct(RuleService $ruleService)
    {
        $this->ruleService = $ruleService;
    }

    /**
     * @ORM\PostPersist
     */
    public function postPersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isQueue($object)) {
            $this->ruleService->upsertRulesForQueue($object);
        }
    }

    /**
     * @ORM\PostUpdate
     */
    public function postUpdate(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isQueue($object)) {
            $this->ruleService->upsertRulesForQueue($object);
        } else if ($this->isQueueManagementView($object)) {
            $this->ruleService->upsertRulesForQueueManagement($object);
        }
    }

    /**
     * @ORM\PreRemove
     */
    public function preRemove(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isQueue($object)) {
            $this->ruleService->deleteRulesForQueue($object->getUuid());
        }
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isQueue($object)
    {
        return $object instanceof Queue && (!in_array(SoftDeleteable::class, class_uses(get_class($object))) || !$object->getDeletedAt());
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isQueueManagementView($object)
    {
        return $object instanceof SystemViewTemplate && $object->getUuid() === 'cs_queue_management';
    }
}
