<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Traits\Ownable;
use App\Traits\Queueable;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use App\Entity\Custom\Queue;

class QueueableSubscriber implements EventSubscriberInterface
{

    protected $containers;
    protected $tokenStorage;
    protected $entityManager;
    protected $requeststack;
    protected $cache;
    protected $dataPersister;

    public function __construct(
        ContainerInterface $containers,
        TokenStorageInterface $tokenStorage,
        EntityManagerInterface $entityManager
    ) {
        $this->containers = $containers;
        $this->tokenStorage = $tokenStorage;
        $this->entityManager = $entityManager;
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['handleQueueableRecords', 33]
        ];
    }

    public function handleQueueableRecords(ViewEvent $event)
    {
        $object = $event->getControllerResult();
        $request = $event->getRequest();
        $requestMethod = $request->getMethod();
        if (!in_array($requestMethod, [Request::METHOD_PUT, Request::METHOD_POST])
            || !$this->isQueueable($object)
            || !$this->isOwnable($object)) {
                return;
        }
        
        $previousData = $request->attributes->get('fsr_previous_data');
        $previousQueueUuid = $previousData['fsrQueue'] ? end(explode('/', $previousData['fsrQueue'])) : null;

        if ($object->getFsrQueue() && $object->getFsrQueue()->getUuid() != $previousQueueUuid) {
            /** @var Queue $queue */
            $queue = $object->getFsrQueue();
            $assignmentParameters = $queue->getAssignmentParameters();
            if ($assignmentParameters && array_key_exists('updateRecordOwnership', $assignmentParameters) && $assignmentParameters['updateRecordOwnership']) {
                $queueOwners = $queue->getOwners();
                foreach ($object->getOwners() as $owner) {
                    $object->removeOwner($owner);
                }
                foreach ($queueOwners as $queueOwner) {
                    $object->addOwner($queueOwner);
                }
            }
        }
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isQueueable($object)
    {
        return in_array(Queueable::class, class_uses(get_class($object)));
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isOwnable($object)
    {
        return in_array(Ownable::class, class_uses(get_class($object)));
    }
}
