<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\Core\SolutionPack;
use App\Service\SolutionPackUtilityService;
use Doctrine\ORM\EntityManagerInterface;

class SolutionPackListener
{
    private $utilityService;
    private $entityManager;

    const SOLUTIONPACK_TYPE = "solutionpack";

    public function __construct(SolutionPackUtilityService $utilityService, EntityManagerInterface $entityManager)
    {
        $this->utilityService = $utilityService;
        $this->entityManager = $entityManager;
    }

    private function checkEmptyContents($contents)
    {
        $empty = true;
        if (!empty($contents)) {
            foreach ($contents as $key => $value) {
                if (!empty($value)) {
                    return false;
                }
            }
        }
        return $empty;
    }

    public function prePersist(SolutionPack $solutionPack)
    {
        if ($solutionPack->getType() == self::SOLUTIONPACK_TYPE) {
            $infoContent = $solutionPack->getInfoContent();
            if ($this->checkEmptyContents($infoContent["contents"])) {
                $template = $solutionPack->getTemplate();
                if ($template) {
                    $options = $template->getOptions();
                    $contents = $this->utilityService->getInfoContent($options);
                    $infoContent["contents"] = $contents;
                    $solutionPack->setInfoContent($infoContent);
                }
            }
        }
    }

    public function preUpdate(SolutionPack $solutionPack)
    {
        if ($solutionPack->getType() == self::SOLUTIONPACK_TYPE) {
            $template = $solutionPack->getTemplate();
            if ($template && $solutionPack->getLocal()) {
                $options = $template->getOptions();

                $infoContents = $solutionPack->getInfoContent();
                $oldContents = $infoContents["contents"];
                $newContents = $this->utilityService->getInfoContent($options);

                if ($oldContents != $newContents) {
                    $infoContents["contents"] = $newContents;
                    $solutionPack->setInfoContent($infoContents);
                }
            }
        }
    }
}
