<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Entity\ViewTemplate\SystemViewTemplate;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpFoundation\Request;
use App\Service\ViewTemplateService;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class SystemViewTemplateSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => [
                ['backupSystemViewTemplate', 31],
            ]
        ];
    }

    public function __construct(
        private EntityManagerInterface $entityManager,
        private ViewTemplateService $viewTemplateService
    ) {
        $this->entityManager = $entityManager;
        $this->viewTemplateService = $viewTemplateService;
    }

    public function backupSystemViewTemplate(ViewEvent $event)
    {
        $newData = $event->getRequest()->attributes->get('data');
        $previousData = $event->getRequest()->attributes->get('previous_data');
        $method = $event->getRequest()->getMethod();
        if (in_array($method, [Request::METHOD_GET, Request::METHOD_POST, Request::METHOD_DELETE])) {
            return;
        }
        if ($method == Request::METHOD_PUT && $newData instanceof SystemViewTemplate) {
            try {
                $name = $newData->getUuid();
                $backupSystemViewTemplate = $this->viewTemplateService->getViewTemplate($name . '-backup');
                $this->saveTemplate($previousData, $backupSystemViewTemplate, $name);
            } catch (NotFoundHttpException $nfe) {
                $backupSystemViewTemplate = new SystemViewTemplate();
                $this->saveTemplate($previousData, $backupSystemViewTemplate, $name);
            }
        }
    }

    private function saveTemplate($data, $template, $name)
    {
        $template->setType($data->getType());
        $template->setImportedBy($data->getImportedBy());
        $template->setConfig($data->getConfig());
        $this->viewTemplateService->saveSystemViewTemplate($name . '-backup', $template);
    }
}
