<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Service\UtilityService;
use App\Constants\CacheConstants;
use App\Entity\Authorization\Role;
use Doctrine\ORM\Event\LifecycleEventArgs;
use App\Entity\Authorization\FieldPermission;
use App\Entity\Authorization\ModulePermission;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class UserPermissionCacheListener
{
    protected $cache;
    protected $utilityService;

    public function __construct(UtilityService $utilityService, TagAwareCacheInterface $cache)
    {
        $this->utilityService = $utilityService;
        $this->cache = $cache;
    }

    private function invalidateCache($args)
    {
        $entity = get_class($args->getObject());
        if (in_array($entity, [Role::class, ModulePermission::class, FieldPermission::class])) {
            $this->cache->invalidateTags([CacheConstants::USER_PERMISSIONS]);
            $cmd = sprintf('sudo -u nginx php /opt/cyops-api/bin/console app:util %s', CacheConstants::USER_PERMISSIONS);
            $this->utilityService->broadcastToHA($cmd);
        }
    }

    public function postUpdate(LifecycleEventArgs $args)
    {
        $this->invalidateCache($args);
    }

    public function postPersist(LifecycleEventArgs $args)
    {
        $this->invalidateCache($args);
    }

    public function postRemove(LifecycleEventArgs $args)
    {
        $this->invalidateCache($args);
    }
}
