<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations\DefaultMigrations;

use Ramsey\Uuid\Uuid;
use App\Constants\AppConstants;
use App\Entity\Core\ConfigProp;
use App\Utility\PayloadUtils;
use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20220923145557 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return 'Default Value for Config Prop for Required TIM Solution Pack';
    }

    public function up(Schema $schema): void
    {
        $now = new \DateTime('NOW', new \DateTimeZone('UTC'));
        $usedValue = 0;
        $fetchTimResults = $this->connection->fetchAllAssociative("select uuid from staging_model_metadatas where type ='threat_intel_feeds'");
        if (!empty($fetchTimResults)) {
            $lastCreateDate = $this->connection->fetchAllAssociative("select createdate from threat_intel_feeds order by createdate desc limit 1");
            if (count($lastCreateDate)) {
                $totalFeedsCreated = $this->connection->fetchAllAssociative("select count(*) as total from threat_intel_feeds where date(createdate)=(select date(createdate) from threat_intel_feeds order by createdate desc limit 1);");
                $usedValue = $totalFeedsCreated[0]['total'];

                $createDate = $lastCreateDate[0]['createdate'];
                $epoch = strtotime($createDate);
                $now = new \DateTime("@$epoch");
            }
        }

        $managerRegistry = $this->container->get('doctrine');
        $entityManager = $managerRegistry->getManagerForClass(ConfigProp::class);
        $configPropRepo = $entityManager->getRepository(ConfigProp::class);
        if (!empty($configPropRepo->findBy(['key' => AppConstants::FEED_COUNT]))) {
            return;
        }

        $feedInfo = new ConfigProp();
        $feedInfo->setKey(AppConstants::FEED_COUNT);
        $feedInfo->setValue($usedValue);
        $feedInfo->setModifiedAt($now);
        $randomUuid = Uuid::uuid4()->toString();
        $feedInfo->setChecksum(PayloadUtils::computeHash($randomUuid, $usedValue, $now));
        $feedInfo->setSalt($randomUuid);
        $entityManager->persist($feedInfo);
        $entityManager->flush();
    }

    public function down(Schema $schema): void
    {
        // Not required
    }
}
