<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210212145231 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        // $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }



    public function getDescription(): string
    {
        return '';
    }

    private $playbooks = '
    [
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Notify New Announcement",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990332,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/0dbeaf35-834d-4943-b75a-0f2d69908f5a",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Set email body",
                    "description": null,
                    "arguments": {
                        "emailBody": "<p>An announcement was sent from War Room #{{ vars.warRoom.id }}:</p>\n<p>&nbsp;</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Announcement Title</strong></td>\n<td>{{vars.input.records[0].title}}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Type</strong></td>\n<td>{{vars.input.records[0].announcementType.itemValue}}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Announcement Description</strong></td>\n<td>{{vars.input.records[0].description}}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Sent By</strong></td>\n<td>{{vars.input.records[0].createUser.firstname}} {{vars.input.records[0].createUser.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<p>&nbsp;</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>War Room TItle</strong></td>\n<td>{{ vars.warRoom.title }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Summary</strong></td>\n<td>{{ vars.warRoom.summary }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Status</strong></td>\n<td>{{ vars.warRoom.status.itemValue }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Launched By</strong></td>\n<td>{{ vars.warRoom.launchedBy.firstname}} {{ vars.warRoom.launchedBy.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<p>&nbsp;</p>\n<p><a href=\"{{ vars.warRoomLink }}\">Join War Room</a></p>\n<p>Thanks,<br />FortiSOAR&trade;</p>"
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "354a72c1-7bde-4648-a023-0a5905637992",
                    "id": 4738
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send announcement notification",
                    "description": null,
                    "arguments": {
                        "arguments": {
                            "warRoom": "{{vars.warRoom}}",
                            "emailBody": "{{vars.emailBody}}",
                            "emailSubject": "[War Room {{vars.warRoom.id }}] New announcement:{{vars.input.records[0].title}}",
                            "emailAttachmentIris": ""
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/e1fd3374-1f75-4f0b-8252-050aabd0a93b"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "2146fd4d-24db-4d05-b5d4-4332c16ca057",
                    "id": 4739
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "resource": "announcements",
                        "resources": [
                            "announcements"
                        ],
                        "step_variables": {
                            "input": {
                                "records": [
                                    "{{vars.input.records[0]}}"
                                ]
                            }
                        },
                        "fieldbasedtrigger": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "notifyOnEmail",
                                    "value": true,
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ]
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/ea155646-3821-4542-9702-b246da430a8d",
                    "uuid": "0dbeaf35-834d-4943-b75a-0f2d69908f5a",
                    "id": 4740
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load war room",
                    "description": null,
                    "arguments": {
                        "query": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "announcements.uuid",
                                    "value": "{{vars.input.records[0][\'@id\'].split(\'/\')[-1]}}",
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ]
                        },
                        "module": "warrooms?$limit=30",
                        "step_variables": {
                            "warRoom": "{{vars.result[0]}}"
                        }
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
                    "uuid": "0ee329f0-6bfa-4e9c-963d-fd8782adfbda",
                    "id": 4741
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Wait 1 second",
                    "description": null,
                    "arguments": {
                        "delay": {
                            "days": 0,
                            "hours": 0,
                            "weeks": 0,
                            "minutes": 0,
                            "seconds": 1
                        }
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/6832e556-b9c7-497a-babe-feda3bd27dbf",
                    "uuid": "39c8ef80-7edc-485f-981a-78d3334d4715",
                    "id": 4742
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Load war room -> Set email body",
                    "targetStep": "/api/3/workflow_steps/354a72c1-7bde-4648-a023-0a5905637992",
                    "sourceStep": "/api/3/workflow_steps/0ee329f0-6bfa-4e9c-963d-fd8782adfbda",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "3e189338-2dff-43a1-9064-bcb164584722"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set email body -> Send announcement notification",
                    "targetStep": "/api/3/workflow_steps/2146fd4d-24db-4d05-b5d4-4332c16ca057",
                    "sourceStep": "/api/3/workflow_steps/354a72c1-7bde-4648-a023-0a5905637992",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "b855116e-f308-41e1-8cc4-a2c92e2f11bf"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Wait 1 second",
                    "targetStep": "/api/3/workflow_steps/39c8ef80-7edc-485f-981a-78d3334d4715",
                    "sourceStep": "/api/3/workflow_steps/0dbeaf35-834d-4943-b75a-0f2d69908f5a",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "2cb9e9f1-9f51-44ff-bb07-6cb0fb9301b2"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Wait 1 second -> Load war room",
                    "targetStep": "/api/3/workflow_steps/0ee329f0-6bfa-4e9c-963d-fd8782adfbda",
                    "sourceStep": "/api/3/workflow_steps/39c8ef80-7edc-485f-981a-78d3334d4715",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "041f8d4e-645f-48c1-b652-39b45ac34fd8"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "0e268c4e-33f4-49ae-bdd2-6a946d99dbb1",
            "recordTags": [
                "system"
            ],
            "id": 1141,
            "createUser": null,
            "createDate": 1619169976,
            "modifyUser": null,
            "modifyDate": 1619169976,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Set up War Room from Incidents",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1619252476,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/c61c48a5-967f-4433-a598-7f84fbb0cbc9",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Load relationships",
                    "description": null,
                    "arguments": {
                        "query": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "id",
                                    "value": "{{vars.input.records[0].id}}",
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ]
                        },
                        "module": "incidents?$limit=30&$relationships=true",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
                    "uuid": "5e1c0dd7-464e-48b5-9578-1668d4e99337",
                    "id": 4743
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Create War Room",
                    "description": null,
                    "arguments": {
                        "message": {
                            "tags": [],
                            "type": "/api/3/picklists/ff599189-3eeb-4c86-acb0-a7915e85ac3b",
                            "tenant": "",
                            "content": "War Room created\nName: {{vars.input.params.warRoomTitle}}\nStatus: {{vars.input.params.status.itemValue}}\n\n[Go to War Room]({{ vars.result[\'@id\'] | iriToLink(\'\') }})\n",
                            "records": "{{vars.input.records[0][\'@id\']}}"
                        },
                        "resource": {
                            "tasks": "{{vars.steps.Load_relationships[0].tasks}}",
                            "title": "{{vars.input.params.warRoomTitle}}",
                            "alerts": "{{vars.steps.Load_relationships[0].alerts}}",
                            "assets": "{{vars.steps.Load_relationships[0].assets}}",
                            "owners": "{{vars.steps.Load_relationships[0].owners}}",
                            "status": "{{vars.input.params.status[\'@id\']}}",
                            "summary": "{{vars.input.params.warRoomDescription}}",
                            "__replace": "",
                            "incidents": "{{vars.input.records[0][\'@id\']}}",
                            "indicators": "{{vars.steps.Load_relationships[0].indicators}}"
                        },
                        "_showJson": false,
                        "operation": "Overwrite",
                        "collection": "/api/3/warrooms",
                        "__recommend": [],
                        "fieldOperation": {
                            "recordTags": "Overwrite"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/2597053c-e718-44b4-8394-4d40fe26d357",
                    "uuid": "6fd05526-b423-48d2-b9be-fcb9d9957041",
                    "id": 4744
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "route": "ff085e32-f73f-4600-9ad6-663f23f0f52e",
                        "title": "Set up War Room",
                        "resources": [
                            "incidents"
                        ],
                        "inputVariables": [
                            {
                                "name": "status",
                                "type": "picklists",
                                "label": "Status",
                                "title": "Picklist",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "picklist",
                                "formType": "picklist",
                                "required": true,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "dataSource": {
                                    "model": "picklists",
                                    "query": {
                                        "sort": [
                                            {
                                                "field": "orderIndex",
                                                "direction": "ASC"
                                            }
                                        ],
                                        "logic": "AND",
                                        "filters": [
                                            {
                                                "field": "listName__name",
                                                "value": "WarRoomStatus",
                                                "operator": "eq"
                                            }
                                        ]
                                    },
                                    "displayConditions": {
                                        "1c4fe134-de72-40d9-b35f-deb8dadc6800": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "4584faf4-fe11-4039-8c73-157e37e477f4": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "55a2f3e7-8354-4c41-9120-f6c3559903f8": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "5d1a13e5-0b83-4e8c-ac42-2bab13385b58": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "cc063ee2-b0ac-47a0-a123-e40c8bcea3a5": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "f433458e-0cf8-4130-94d3-31b60ed4add7": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        }
                                    }
                                },
                                "searchable": false,
                                "templateUrl": "app/components/form/fields/typeahead.html",
                                "defaultValue": {
                                    "id": 5512,
                                    "@id": "/api/3/picklists/55a2f3e7-8354-4c41-9120-f6c3559903f8",
                                    "icon": null,
                                    "@type": "Picklist",
                                    "color": "#cfcfcf",
                                    "display": "Draft",
                                    "listName": "/api/3/picklist_names/ab8bbdb4-2cf9-4695-90ef-9f1f825d2b1f",
                                    "itemValue": "Draft",
                                    "orderIndex": 0
                                },
                                "displayTemplate": "{{ itemValue }}",
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            },
                            {
                                "name": "warRoomTitle",
                                "type": "string",
                                "label": "War Room Title",
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "text",
                                "required": true,
                                "_expanded": false,
                                "moduleField": "name",
                                "useRecordFieldDefault": true
                            },
                            {
                                "name": "warRoomDescription",
                                "type": "string",
                                "label": "War Room Description",
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "text",
                                "required": false,
                                "_expanded": true,
                                "moduleField": "description",
                                "useRecordFieldDefault": true
                            },
                            {
                                "name": "includeAnalysts",
                                "type": "boolean",
                                "label": "Include all analysts involved with this incident as War Room responders",
                                "title": "Checkbox",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "checkbox",
                                "formType": "checkbox",
                                "required": false,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/checkbox.html",
                                "_nameTouched": true,
                                "defaultValue": true,
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            }
                        ],
                        "step_variables": {
                            "input": {
                                "params": {
                                    "status": "{{vars.request.data[\"status\"]}}",
                                    "warRoomTitle": "{{vars.request.data[\"warRoomTitle\"]}}",
                                    "includeAnalysts": "{{vars.request.data[\"includeAnalysts\"]}}",
                                    "warRoomDescription": "{{vars.request.data[\"warRoomDescription\"]}}"
                                },
                                "records": "{{vars.input.records}}"
                            }
                        },
                        "_promptexpanded": true,
                        "displayConditions": {
                            "alerts": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            },
                            "incidents": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            }
                        },
                        "executeButtonText": "Execute",
                        "noRecordExecution": false,
                        "singleRecordExecution": true
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
                    "uuid": "c61c48a5-967f-4433-a598-7f84fbb0cbc9",
                    "id": 4745
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Load relationships",
                    "targetStep": "/api/3/workflow_steps/5e1c0dd7-464e-48b5-9578-1668d4e99337",
                    "sourceStep": "/api/3/workflow_steps/c61c48a5-967f-4433-a598-7f84fbb0cbc9",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "84aaa31d-0295-44ab-8548-1a955d699191"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load relationships -> Create War Room",
                    "targetStep": "/api/3/workflow_steps/6fd05526-b423-48d2-b9be-fcb9d9957041",
                    "sourceStep": "/api/3/workflow_steps/5e1c0dd7-464e-48b5-9578-1668d4e99337",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "be571cdb-8faa-440d-bd85-bd68f9ce3f19"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "575ec2d1-9317-460b-ab13-0586fd73d082",
            "recordTags": [],
            "id": 1142,
            "createUser": null,
            "createDate": 1619169976,
            "modifyUser": "/api/3/people/3451141c-bac6-467c-8d72-85e0fab569ce",
            "modifyDate": 1619252475,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Send War Room Summary Email",
            "aliasName": null,
            "tag": null,
            "description": "Email contains war room summary report",
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990843,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/6f3106df-2c3b-4a03-8157-d41d4084933e",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "route": "62bdcb92-cb0a-48ca-b1f9-02e0d058bc42",
                        "title": "Send Email Update",
                        "resources": [
                            "warrooms"
                        ],
                        "inputVariables": [
                            {
                                "name": "sendToResponseTeam",
                                "type": "boolean",
                                "label": "Send to Response Team",
                                "title": "Checkbox",
                                "usable": true,
                                "tooltip": "Send the update to all users and teams with access to the record",
                                "dataType": "checkbox",
                                "formType": "checkbox",
                                "required": false,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/checkbox.html",
                                "defaultValue": true,
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            },
                            {
                                "name": "customRecipientEmails",
                                "type": "string",
                                "label": "Custom Recipient Emails",
                                "title": "Text Area",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "textarea",
                                "required": false,
                                "_expanded": false,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/textarea.html",
                                "defaultValue": null,
                                "visibilityQuery": {
                                    "sort": [],
                                    "limit": 30,
                                    "logic": "AND",
                                    "filters": [
                                        {
                                            "type": "primitive",
                                            "field": "sendToResponseTeam",
                                            "value": false,
                                            "operator": "eq",
                                            "_operator": "eq"
                                        }
                                    ]
                                },
                                "lengthConstraint": true,
                                "allowedEncryption": true,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false,
                                "_addVisibilityConditions": true
                            },
                            {
                                "name": "attachWarRoomSummaryReport",
                                "type": "boolean",
                                "label": "Attach War Room Summary report",
                                "title": "Checkbox",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "checkbox",
                                "formType": "checkbox",
                                "required": false,
                                "_expanded": false,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/checkbox.html",
                                "defaultValue": true,
                                "visibilityQuery": {
                                    "sort": [],
                                    "limit": 30,
                                    "logic": "AND",
                                    "filters": []
                                },
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false,
                                "_addVisibilityConditions": true
                            }
                        ],
                        "step_variables": {
                            "input": {
                                "params": {
                                    "sendToResponseTeam": "{{vars.request.data[\"sendToResponseTeam\"]}}",
                                    "customRecipientEmails": "{{vars.request.data[\"customRecipientEmails\"]}}",
                                    "attachWarRoomSummaryReport": "{{vars.request.data[\"attachWarRoomSummaryReport\"]}}"
                                },
                                "records": "{{vars.input.records}}"
                            }
                        },
                        "_promptexpanded": true,
                        "displayConditions": {
                            "alerts": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            },
                            "warrooms": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            }
                        },
                        "executeButtonText": "Send Email",
                        "noRecordExecution": false,
                        "singleRecordExecution": true
                    },
                    "status": null,
                    "top": "30",
                    "left": "300",
                    "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
                    "uuid": "6f3106df-2c3b-4a03-8157-d41d4084933e",
                    "id": 4746
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Set email parameters",
                    "description": null,
                    "arguments": {
                        "emailBody": "<p>You were added to War Room #{{ vars.warRoom.id }}:</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Title</strong></td>\n<td>{{ vars.warRoom.title }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Summary</strong></td>\n<td>{{ vars.warRoom.summary }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Status</strong></td>\n<td>{{ vars.warRoom.status.itemValue }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Launched By</strong></td>\n<td>{{ vars.warRoom.launchedBy.firstname}} {{ vars.warRoom.launchedBy.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<br />\n<p><a href=\"{{ vars.warRoomLink }}\">Join War Room</a></p>\n<p>Thanks,<br />FortiSOAR&trade;</p>",
                        "emailSubject": "War Room #{{ vars.warRoom.id }}: {{ vars.warRoom.title }}",
                        "emailAttachments": "{{vars.steps.Generate_War_Room_Summary_Report.data[\'@id\']}}"
                    },
                    "status": null,
                    "top": "435",
                    "left": "300",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "54eb3b7b-b0f4-4921-9a40-af37a5285845",
                    "id": 4747
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Configuration",
                    "description": null,
                    "arguments": {
                        "warRoom": "{{vars.input.records[0]}}",
                        "warRoomLink": "https://{{globalVars.Server_fqhn}}{{vars.input.records[0][\'@id\'] | iriToLink}}"
                    },
                    "status": null,
                    "top": "165",
                    "left": "300",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "d55ad50c-0d37-4f5f-a14e-3663584fda90",
                    "id": 4748
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send email",
                    "description": null,
                    "arguments": {
                        "arguments": {
                            "warRoom": "{{vars.input.records[0]}}",
                            "emailBody": "{{vars.emailBody}}",
                            "emailSubject": "{{vars.emailSubject}}",
                            "emailAttachmentIris": "{{vars.emailAttachments}}"
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/e1fd3374-1f75-4f0b-8252-050aabd0a93b"
                    },
                    "status": null,
                    "top": "705",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "30c45bd7-4840-46af-8fbd-5fb06f47e2a8",
                    "id": 4749
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send to response team or custom",
                    "description": null,
                    "arguments": {
                        "conditions": [
                            {
                                "option": "Response Team",
                                "step_iri": "/api/3/workflow_steps/30c45bd7-4840-46af-8fbd-5fb06f47e2a8",
                                "condition": "{{ vars.input.params.sendToResponseTeam }}",
                                "step_name": "Send email"
                            },
                            {
                                "option": "Custom",
                                "default": true,
                                "step_iri": "/api/3/workflow_steps/8eacab0c-66bb-4b5d-b0be-d36916bf62bf",
                                "step_name": "Send to custom recipients"
                            }
                        ]
                    },
                    "status": null,
                    "top": "570",
                    "left": "300",
                    "stepType": "/api/3/workflow_step_types/12254cf5-5db7-4b1a-8cb1-3af081924b28",
                    "uuid": "0ecd65c5-44d3-4bba-9456-ee35eef95d78",
                    "id": 4750
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send to custom recipients",
                    "description": null,
                    "arguments": {
                        "config": "f221bd8b-e626-4513-8f41-615f99d56281",
                        "params": {
                            "cc": "",
                            "to": "{{vars.input.params.customRecipientEmails}}",
                            "bcc": "",
                            "from": "",
                            "type": "Manual Input",
                            "content": "{{vars.emailBody}}",
                            "subject": "{{vars.emailSubject}}",
                            "iri_list": "{{vars.emailAttachments}}",
                            "body_type": "Plain Text",
                            "file_name": "",
                            "file_path": ""
                        },
                        "version": "2.4.0",
                        "from_str": "admin@example.com",
                        "connector": "smtp",
                        "operation": "send_email_new",
                        "operationTitle": "Send Email",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "705",
                    "left": "475",
                    "stepType": "/api/3/workflow_step_types/4c0019b2-055c-44d0-968c-678a0c2d762e",
                    "uuid": "8eacab0c-66bb-4b5d-b0be-d36916bf62bf",
                    "id": 4751
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Generate War Room Summary Report",
                    "description": null,
                    "arguments": {
                        "name": "Report Engine",
                        "params": {
                            "user_id": "{{vars.currentUser}}",
                            "timezone": "{{vars.request.headers.tz}}",
                            "report_id": "60f0c032-90e8-4026-b430-176efd5560b7",
                            "query_params": "{\"qparam\": {\"warRoomID\": {{ vars.warRoom.id }} } }"
                        },
                        "version": "1.2.0",
                        "connector": "cyops-schedule-report",
                        "operation": "schedule_report_manual",
                        "operationTitle": "Generate Report From Report ID",
                        "pickFromTenant": false,
                        "step_variables": []
                    },
                    "status": null,
                    "top": "300",
                    "left": "300",
                    "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
                    "uuid": "f1a56775-8dc4-4d6f-8423-4718cd1ed7c1",
                    "id": 4752
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Configuration",
                    "targetStep": "/api/3/workflow_steps/d55ad50c-0d37-4f5f-a14e-3663584fda90",
                    "sourceStep": "/api/3/workflow_steps/6f3106df-2c3b-4a03-8157-d41d4084933e",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "b70f8650-3127-4777-9be4-d30a2ba4606b"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set email body -> Send to response team or custom",
                    "targetStep": "/api/3/workflow_steps/0ecd65c5-44d3-4bba-9456-ee35eef95d78",
                    "sourceStep": "/api/3/workflow_steps/54eb3b7b-b0f4-4921-9a40-af37a5285845",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "d692e66a-4970-4dd9-8b5b-cf7557c92f07"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Send to response team or custom -> Send email",
                    "targetStep": "/api/3/workflow_steps/30c45bd7-4840-46af-8fbd-5fb06f47e2a8",
                    "sourceStep": "/api/3/workflow_steps/0ecd65c5-44d3-4bba-9456-ee35eef95d78",
                    "label": "Response Team",
                    "isExecuted": false,
                    "uuid": "799f75cb-a71d-4590-b3b1-54c23cf0b94c"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Send to response team or custom -> Send to custom recipients",
                    "targetStep": "/api/3/workflow_steps/8eacab0c-66bb-4b5d-b0be-d36916bf62bf",
                    "sourceStep": "/api/3/workflow_steps/0ecd65c5-44d3-4bba-9456-ee35eef95d78",
                    "label": "Custom",
                    "isExecuted": false,
                    "uuid": "f79d36af-90db-486f-bd23-0a1e704e3b49"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Configuration -> Generate War Room Summary Report",
                    "targetStep": "/api/3/workflow_steps/f1a56775-8dc4-4d6f-8423-4718cd1ed7c1",
                    "sourceStep": "/api/3/workflow_steps/d55ad50c-0d37-4f5f-a14e-3663584fda90",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "f1fb2090-5835-485b-8591-832901de0f72"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Generate War Room Summary Report -> Set email body and subject",
                    "targetStep": "/api/3/workflow_steps/54eb3b7b-b0f4-4921-9a40-af37a5285845",
                    "sourceStep": "/api/3/workflow_steps/f1a56775-8dc4-4d6f-8423-4718cd1ed7c1",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "c327db63-eb68-44ca-afcd-a03ee8e92000"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "5d2bce7e-2f83-4336-9012-2515257452f3",
            "recordTags": [],
            "id": 1143,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Set War Room Live and Notify Responders",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990603,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/21727b68-95be-43be-adcf-400085d713aa",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Configuration",
                    "description": null,
                    "arguments": {
                        "warRoom": "{{ vars.input.records[0] }}",
                        "warRoomLink": "https://{{globalVars.Server_fqhn}}{{vars.input.records[0][\'@id\'] | iriToLink}}",
                        "externalCollaborationLink": "{{vars.input.params.externalCollaborationLink}}"
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "a3b47329-e305-472a-90a2-9a2b63f791b0",
                    "id": 4753
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Set email body",
                    "description": null,
                    "arguments": {
                        "params": {
                            "value": "<p>You were added to War Room #{{ vars.warRoom.id }}:</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Title</strong></td>\n<td>{{ vars.warRoom.title }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Summary</strong></td>\n<td>{{ vars.warRoom.summary }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Status</strong></td>\n<td>{{ vars.warRoom.status.itemValue }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Launched By</strong></td>\n<td>{{ vars.warRoom.launchedBy.firstname}} {{ vars.warRoom.launchedBy.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<br />\n<p><a href=\"{{ vars.warRoomLink }}\">Join War Room</a></p>\n<p>Thanks,<br />FortiSOAR&trade;</p>"
                        },
                        "version": "3.1.0",
                        "connector": "cyops_utilities",
                        "operation": "format_richtext",
                        "operationTitle": "Utils: Format as RichText",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "361cf638-c747-438e-b68c-58ed133d14e9",
                    "id": 4754
                },
                {
                    "@type": "WorkflowStep",
                    "name": "War room send notifications",
                    "description": null,
                    "arguments": {
                        "arguments": {
                            "warRoom": "{{vars.input.records[0]}}",
                            "emailBody": "{{vars.steps.Set_email_body.data[\'formatted_string\']}}",
                            "emailSubject": "You were added to War Room #{{ vars.warRoom.id }}: {{ vars.warRoom.title }}"
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/e1fd3374-1f75-4f0b-8252-050aabd0a93b"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "f989a0f6-97bd-473b-bb31-3253e4988d1a",
                    "id": 4755
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "route": "b44916bc-6a88-4123-9ccd-78315244763b",
                        "title": "Go Live",
                        "resources": [
                            "warrooms"
                        ],
                        "inputVariables": [
                            {
                                "name": "notifyAllResponders",
                                "type": "boolean",
                                "label": "Notify All Responders",
                                "title": "Checkbox",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "checkbox",
                                "formType": "checkbox",
                                "required": false,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/checkbox.html",
                                "defaultValue": true,
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            },
                            {
                                "name": "externalCollaborationLink",
                                "type": "string",
                                "label": "External Collaboration Link",
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "text",
                                "required": false,
                                "_expanded": true,
                                "moduleField": "collaboration",
                                "_previousName": "externalOllaborationLink",
                                "useRecordFieldDefault": true
                            }
                        ],
                        "step_variables": {
                            "input": {
                                "params": {
                                    "notifyAllResponders": "{{vars.request.data[\"notifyAllResponders\"]}}",
                                    "externalCollaborationLink": "{{vars.request.data[\"externalCollaborationLink\"]}}"
                                },
                                "records": "{{vars.input.records}}"
                            }
                        },
                        "_promptexpanded": true,
                        "displayConditions": {
                            "warrooms": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": [
                                    {
                                        "type": "object",
                                        "field": "status",
                                        "value": "/api/3/picklists/55a2f3e7-8354-4c41-9120-f6c3559903f8",
                                        "_value": {
                                            "@id": "/api/3/picklists/55a2f3e7-8354-4c41-9120-f6c3559903f8",
                                            "itemValue": "Draft"
                                        },
                                        "operator": "eq"
                                    }
                                ]
                            }
                        },
                        "executeButtonText": "Go Live",
                        "noRecordExecution": false,
                        "singleRecordExecution": true
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
                    "uuid": "21727b68-95be-43be-adcf-400085d713aa",
                    "id": 4756
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Set status to Live",
                    "description": null,
                    "arguments": {
                        "resource": {
                            "status": {
                                "id": 5513,
                                "@id": "/api/3/picklists/1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                "icon": null,
                                "uuid": "1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                "@type": "Picklist",
                                "color": "#28B35C",
                                "display": "Live",
                                "listName": "/api/3/picklist_names/ab8bbdb4-2cf9-4695-90ef-9f1f825d2b1f",
                                "itemValue": "Live",
                                "orderIndex": 1
                            },
                            "launchedBy": "{{vars.currentUser}}",
                            "activeSince": "{{arrow.utcnow().int_timestamp}}",
                            "collaboration": "{{vars.externalCollaborationLink}}"
                        },
                        "_showJson": false,
                        "operation": "Append",
                        "collection": "{{vars.warRoom[\'@id\']}}",
                        "__recommend": [],
                        "collectionType": "/api/3/warrooms",
                        "fieldOperation": {
                            "recordTags": "Append"
                        },
                        "step_variables": {
                            "warRoom": "{{vars.result}}"
                        }
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                    "uuid": "a9673021-6174-433b-9860-765648a519c9",
                    "id": 4757
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Set status to Active -> Set email body",
                    "targetStep": "/api/3/workflow_steps/361cf638-c747-438e-b68c-58ed133d14e9",
                    "sourceStep": "/api/3/workflow_steps/a9673021-6174-433b-9860-765648a519c9",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "ba193374-e595-4989-b53e-8dc61f538ec0"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set email body -> War room send notifications",
                    "targetStep": "/api/3/workflow_steps/f989a0f6-97bd-473b-bb31-3253e4988d1a",
                    "sourceStep": "/api/3/workflow_steps/361cf638-c747-438e-b68c-58ed133d14e9",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "992ad323-29a4-4070-bb4c-d72e8091cf9e"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Configuration",
                    "targetStep": "/api/3/workflow_steps/a3b47329-e305-472a-90a2-9a2b63f791b0",
                    "sourceStep": "/api/3/workflow_steps/21727b68-95be-43be-adcf-400085d713aa",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "824bb163-82ad-462f-ac55-a8fa17e471d9"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Configuration -> Set status to Active",
                    "targetStep": "/api/3/workflow_steps/a9673021-6174-433b-9860-765648a519c9",
                    "sourceStep": "/api/3/workflow_steps/a3b47329-e305-472a-90a2-9a2b63f791b0",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "efebba09-8015-444e-9869-29cb60354335"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "66752c80-743a-457d-943d-b9fc996baf23",
            "recordTags": [],
            "id": 1144,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Update War Room Close Date",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990887,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/07228b5b-7ab6-4df8-bf32-7c83f7994080",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "resource": "warrooms",
                        "resources": [
                            "warrooms"
                        ],
                        "step_variables": {
                            "input": {
                                "records": [
                                    "{{vars.input.records[0]}}"
                                ]
                            }
                        },
                        "fieldbasedtrigger": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "object",
                                    "field": "status",
                                    "value": null,
                                    "_value": {
                                        "@id": null,
                                        "display": "",
                                        "itemValue": ""
                                    },
                                    "operator": "changed"
                                },
                                {
                                    "type": "object",
                                    "field": "status",
                                    "value": "/api/3/picklists/f433458e-0cf8-4130-94d3-31b60ed4add7",
                                    "_value": {
                                        "@id": "/api/3/picklists/f433458e-0cf8-4130-94d3-31b60ed4add7",
                                        "display": "Closed",
                                        "itemValue": "Closed"
                                    },
                                    "operator": "eq"
                                }
                            ]
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
                    "uuid": "07228b5b-7ab6-4df8-bf32-7c83f7994080",
                    "id": 4758
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Update War Room",
                    "description": null,
                    "arguments": {
                        "resource": {
                            "closedOn": "{{arrow.utcnow().int_timestamp}}"
                        },
                        "_showJson": false,
                        "operation": "Append",
                        "collection": "{{vars.input.records[0][\'@id\']}}",
                        "__recommend": [],
                        "collectionType": "/api/3/warrooms",
                        "fieldOperation": {
                            "recordTags": "Append"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                    "uuid": "63070bed-e97f-4fb3-a35a-fa6fef055d2b",
                    "id": 4759
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Update War Room",
                    "targetStep": "/api/3/workflow_steps/63070bed-e97f-4fb3-a35a-fa6fef055d2b",
                    "sourceStep": "/api/3/workflow_steps/07228b5b-7ab6-4df8-bf32-7c83f7994080",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "972d594b-4fa2-44de-a34b-830415be1dd0"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "7031053b-59c5-4444-ba51-90ad76f434f3",
            "recordTags": [
                "system"
            ],
            "id": 1145,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Cascade Ownership for Newly Linked Records",
            "aliasName": null,
            "tag": null,
            "description": "Assign war room responders as owners on all newly linked records such as alerts, incident, indicators, e",
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1619255488,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/d08533e5-dbc4-4f50-a216-85e46680a187",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "resource": "warrooms",
                        "resources": [
                            "warrooms"
                        ],
                        "step_variables": {
                            "input": {
                                "params": [],
                                "records": [
                                    "{{vars.input.records[0]}}"
                                ]
                            }
                        },
                        "fieldbasedtrigger": {
                            "sort": [],
                            "limit": 30,
                            "logic": "OR",
                            "filters": [
                                {
                                    "type": "array",
                                    "field": "alerts",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "incidents",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "indicators",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "announcements",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "attachments",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "tasks",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "owners",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "array",
                                    "field": "userOwners",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                }
                            ]
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
                    "uuid": "d08533e5-dbc4-4f50-a216-85e46680a187",
                    "id": 4760
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Find Related Records",
                    "description": null,
                    "arguments": {
                        "record_metadata": "{{(vars.input.records[0][\'@id\'] + \"?$relationships=true\") | fromIRI}}"
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "dcc3922e-1b29-45d9-a0ed-ca07cc7d7638",
                    "id": 4761
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Create Data List",
                    "description": null,
                    "arguments": {
                        "tasks": "{{vars.record_metadata.tasks | json_query(\'[*][\"@id\"][]\') or []}}",
                        "alerts": "{{vars.record_metadata.alerts | json_query(\'[*][\"@id\"][]\') or []}}",
                        "assets": "{{vars.record_metadata.assets | json_query(\'[*][\"@id\"][]\') or []}}",
                        "owners": "{{vars.record_metadata.owners | json_query(\'[*][\"@id\"][]\') or []}}",
                        "incidents": "{{vars.record_metadata.incidents | json_query(\'[*][\"@id\"][]\') or []}}",
                        "indicators": "{{vars.record_metadata.indicators | json_query(\'[*][\"@id\"][]\') or []}}",
                        "attachments": "{{vars.record_metadata.attachments | json_query(\'[*][\"@id\"][]\') or []}}",
                        "user_owners": "{{vars.record_metadata.userOwners | json_query(\'[*][\"@id\"][]\') or []}}",
                        "announcements": "{{vars.record_metadata.announcements | json_query(\'[*][\"@id\"][]\') or []}}"
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "378b866f-31f8-4b93-a422-a3ee022fa1fc",
                    "id": 4762
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Combine Data Set",
                    "description": null,
                    "arguments": {
                        "iri_list": "{{vars.linked_alerts | union(vars.linked_announcements) | union(vars.linked_assets) | union(vars.linked_attachments) | union(vars.linked_incidents) | union(vars.linked_indicators) | union(vars.linked_tasks)}}"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "6a80f6ac-3191-4dfc-a8f2-d204d34f7d11",
                    "id": 4763
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Update Ownership",
                    "description": null,
                    "arguments": {
                        "for_each": {
                            "item": "{{vars.iri_list}}",
                            "__bulk": true,
                            "condition": "",
                            "batch_size": 100
                        },
                        "resource": {
                            "__link": {
                                "owners": "{{vars.owners}}",
                                "userOwners": "{{vars.user_owners}}"
                            }
                        },
                        "_showJson": false,
                        "operation": "Append",
                        "collection": "{{vars.item}}",
                        "__recommend": [],
                        "collectionType": "/api/3/alerts",
                        "fieldOperation": {
                            "recordTags": "Append"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "705",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                    "uuid": "c67a65ef-c3b2-4aa2-a795-70d9562d40d0",
                    "id": 4764
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Identify Changed Entities",
                    "description": null,
                    "arguments": {
                        "linked_tasks": "{% if vars.previous.data.tasks %}{{vars.tasks | difference(vars.previous.data.tasks) | list}}{% else %}[]{% endif %}",
                        "linked_alerts": "{% if vars.previous.data.alerts %}{{vars.alerts | difference(vars.previous.data.alerts) | list}}{% else %}[]{% endif %}",
                        "linked_assets": "{% if vars.previous.data.assets %}{{vars.assets | difference(vars.previous.data.assets) | list}}{% else %}[]{% endif %}",
                        "linked_incidents": "{% if vars.previous.data.incidents %}{{vars.incidents | difference(vars.previous.data.incidents) | list}}{% else %}[]{% endif %}",
                        "linked_indicators": "{% if vars.previous.data.indicators %}{{vars.indicators | difference(vars.previous.data.indicators) | list}}{% else %}[]{% endif %}",
                        "linked_attachments": "{% if vars.previous.data.attachments %}{{vars.attachments | difference(vars.previous.data.attachments) | list}}{% else %}[]{% endif %}",
                        "linked_announcements": "{% if vars.previous.data.announcements %}{{vars.announcements | difference(vars.previous.data.announcements) | list}}{% else %}[]{% endif %}"
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "9a8a388d-8665-4295-8e05-19bf86570de9",
                    "id": 4765
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Find Related Records",
                    "targetStep": "/api/3/workflow_steps/dcc3922e-1b29-45d9-a0ed-ca07cc7d7638",
                    "sourceStep": "/api/3/workflow_steps/d08533e5-dbc4-4f50-a216-85e46680a187",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "3ba309fc-06b0-4bdd-a31a-9e73ee2ca5d9"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Find Related Records -> Create Data List",
                    "targetStep": "/api/3/workflow_steps/378b866f-31f8-4b93-a422-a3ee022fa1fc",
                    "sourceStep": "/api/3/workflow_steps/dcc3922e-1b29-45d9-a0ed-ca07cc7d7638",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "40833033-e11f-4abd-a5d0-0dbda9dae054"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Combine Data Set -> Update Ownership",
                    "targetStep": "/api/3/workflow_steps/c67a65ef-c3b2-4aa2-a795-70d9562d40d0",
                    "sourceStep": "/api/3/workflow_steps/6a80f6ac-3191-4dfc-a8f2-d204d34f7d11",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "0bba9c8a-b079-45cf-aceb-ddb308d711f8"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Create Data List -> Copy  of Identify Changed Entities",
                    "targetStep": "/api/3/workflow_steps/9a8a388d-8665-4295-8e05-19bf86570de9",
                    "sourceStep": "/api/3/workflow_steps/378b866f-31f8-4b93-a422-a3ee022fa1fc",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "a38ba7b6-c50b-45d6-b59e-255a3fdf9a13"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Copy  of Identify Changed Entities -> Combine Data Set",
                    "targetStep": "/api/3/workflow_steps/6a80f6ac-3191-4dfc-a8f2-d204d34f7d11",
                    "sourceStep": "/api/3/workflow_steps/9a8a388d-8665-4295-8e05-19bf86570de9",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "ceb2c402-5a37-4eb2-8450-2e925a41dc8d"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "91facde8-08e1-4700-9be1-3c835a1a17e3",
            "recordTags": [
                "system"
            ],
            "id": 1146,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": "/api/3/people/3451141c-bac6-467c-8d72-85e0fab569ce",
            "modifyDate": 1619255487,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Generate War Room Report",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990549,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/02e6ebea-4536-46fb-bd0a-494fa679b467",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "route": "69ba561c-3be5-49e6-918b-d41252555f5d",
                        "title": "Generate Report",
                        "resources": [
                            "warrooms"
                        ],
                        "inputVariables": [],
                        "step_variables": {
                            "input": {
                                "records": "{{vars.input.records}}"
                            }
                        },
                        "_promptexpanded": true,
                        "displayConditions": {
                            "warrooms": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            }
                        },
                        "executeButtonText": "Execute",
                        "noRecordExecution": false,
                        "singleRecordExecution": true
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
                    "uuid": "02e6ebea-4536-46fb-bd0a-494fa679b467",
                    "id": 4766
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Update war room file",
                    "description": null,
                    "arguments": {
                        "resource": {
                            "report": "{{vars.steps.Generate_Report_By_Manual_Trigger.data.file[\"@id\"]}}"
                        },
                        "_showJson": false,
                        "operation": "Append",
                        "collection": "{{vars.input.records[0][\'@id\']}}",
                        "collectionType": "/api/3/warrooms",
                        "fieldOperation": {
                            "recordTags": "Append"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                    "uuid": "bf1f086a-f2da-49c3-b51c-ddd28e8b7fd6",
                    "id": 4767
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Generate Report By Manual Trigger",
                    "description": null,
                    "arguments": {
                        "name": "Report Engine",
                        "params": {
                            "user_id": "{{vars.currentUser}}",
                            "timezone": "{{vars.request.headers.tz}}",
                            "report_id": "60f0c032-90e8-4026-b430-176efd5560b7",
                            "query_params": "{\"qparam\": {\"warRoomID\": {{vars.input.records[0].id}} } }"
                        },
                        "version": "1.2.0",
                        "connector": "cyops-schedule-report",
                        "operation": "schedule_report_manual",
                        "operationTitle": "Generate Report From Report ID",
                        "step_variables": {
                            "reportObject": "{{vars.result.data}}"
                        }
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
                    "uuid": "2e753f53-7925-4c4c-bae7-a8e20c06c6a8",
                    "id": 4768
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Insert Link for Report",
                    "description": null,
                    "arguments": {
                        "resource": {
                            "file": "{{vars.steps.Generate_Report_By_Manual_Trigger.data.file[\"@id\"]}}",
                            "type": {
                                "id": 208,
                                "@id": "/api/3/picklists/ff599189-3eeb-4c86-acb0-a7915e85ac3b",
                                "icon": null,
                                "@type": "Picklist",
                                "color": null,
                                "display": "Comment",
                                "listName": "/api/3/picklist_names/0841c1eb-a0a3-4abd-b29c-9f68e4d9b14f",
                                "itemValue": "Comment",
                                "orderIndex": 1
                            },
                            "people": [],
                            "content": "<p>War Room Summary Report available for download</p>",
                            "warrooms": "{{vars.input.records[0][\'@id\']}}",
                            "__replace": "",
                            "isImportant": false,
                            "peopleUpdated": false
                        },
                        "_showJson": false,
                        "operation": "Overwrite",
                        "collection": "/api/3/comments",
                        "__recommend": [],
                        "fieldOperation": {
                            "recordTags": "Overwrite"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/2597053c-e718-44b4-8394-4d40fe26d357",
                    "uuid": "3e8f0b7e-7937-4704-96bc-7411c263daf4",
                    "id": 4769
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Generate Report By Manual Trigger",
                    "targetStep": "/api/3/workflow_steps/2e753f53-7925-4c4c-bae7-a8e20c06c6a8",
                    "sourceStep": "/api/3/workflow_steps/02e6ebea-4536-46fb-bd0a-494fa679b467",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "9562c6b0-d5c6-4495-9931-2fc88442a51f"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Generate Report By Manual Trigger -> Update war room file",
                    "targetStep": "/api/3/workflow_steps/bf1f086a-f2da-49c3-b51c-ddd28e8b7fd6",
                    "sourceStep": "/api/3/workflow_steps/2e753f53-7925-4c4c-bae7-a8e20c06c6a8",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "8278ce0b-140b-449f-9533-7a1ecdf80de6"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Update war room file -> Insert Link for Report",
                    "targetStep": "/api/3/workflow_steps/3e8f0b7e-7937-4704-96bc-7411c263daf4",
                    "sourceStep": "/api/3/workflow_steps/bf1f086a-f2da-49c3-b51c-ddd28e8b7fd6",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "4106cae5-93e6-412e-a66b-7e78b8f60c0a"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "9bde7776-97c1-4d79-ae65-f38bd1317fd1",
            "recordTags": [
                "system"
            ],
            "id": 1147,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Send Email",
            "aliasName": null,
            "tag": null,
            "description": "Send individual notifications",
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [
                "userIri",
                "warRoom",
                "emailBody",
                "emailSubject",
                "emailAttachmentIris"
            ],
            "synchronous": false,
            "lastModifyDate": 1618991033,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/688b9e36-8293-4f19-99e1-68c207052558",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Load user email settings",
                    "description": null,
                    "arguments": {
                        "userNotificationsEmail": "{{ (vars.userObject[\'@settings\'] + \'/user/notifications/email\') | fromIRI }}"
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "1cfd0562-fc9f-4da4-af28-e5de485597de",
                    "id": 4770
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load userObject",
                    "description": null,
                    "arguments": {
                        "warRoom": "{{vars.input.params.warRoom}}",
                        "userObject": "{{ vars.input.params.userIri | fromIRI }}"
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "f779b3fa-28a9-4e60-a590-22895a3ae5c4",
                    "id": 4771
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "step_variables": {
                            "input": {
                                "params": {
                                    "userIri": "{{ vars.userIri }}",
                                    "warRoom": "{{ vars.warRoom }}",
                                    "emailBody": "{{ vars.emailBody }}",
                                    "emailSubject": "{{ vars.emailSubject }}",
                                    "emailAttachmentIris": "{{ vars.emailAttachmentIris }}"
                                }
                            }
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
                    "uuid": "688b9e36-8293-4f19-99e1-68c207052558",
                    "id": 4772
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send email if user wants to receive email",
                    "description": null,
                    "arguments": {
                        "when": "{{vars.userNotificationsEmail}}",
                        "config": "8f7f5c5b-c136-4e2f-9d43-46fb9ad8b999",
                        "params": {
                            "cc": "",
                            "to": "{{vars.userObject.email}}",
                            "bcc": "",
                            "from": "",
                            "type": "Manual Input",
                            "content": "<p>{{vars.input.params.emailBody}}</p>",
                            "subject": "{{vars.input.params.emailSubject}}",
                            "iri_list": "{{vars.input.params.emailAttachmentIris}}",
                            "body_type": "Rich Text",
                            "file_name": "",
                            "file_path": ""
                        },
                        "version": "2.4.0",
                        "from_str": "admin@example.com",
                        "connector": "smtp",
                        "operation": "send_email_new",
                        "operationTitle": "Send Email",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/4c0019b2-055c-44d0-968c-678a0c2d762e",
                    "uuid": "91626bc0-62d1-4ec6-898d-cce31d6f6caa",
                    "id": 4773
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Set uuid",
                    "targetStep": "/api/3/workflow_steps/f779b3fa-28a9-4e60-a590-22895a3ae5c4",
                    "sourceStep": "/api/3/workflow_steps/688b9e36-8293-4f19-99e1-68c207052558",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "e2fd30b8-80fe-4711-a1ff-20596e383c7f"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set uuid -> Load user email settings",
                    "targetStep": "/api/3/workflow_steps/1cfd0562-fc9f-4da4-af28-e5de485597de",
                    "sourceStep": "/api/3/workflow_steps/f779b3fa-28a9-4e60-a590-22895a3ae5c4",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "ca1e80b4-a2d1-4b03-b926-23ca587da165"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load user email settings -> Send email if user wants to receive email",
                    "targetStep": "/api/3/workflow_steps/91626bc0-62d1-4ec6-898d-cce31d6f6caa",
                    "sourceStep": "/api/3/workflow_steps/1cfd0562-fc9f-4da4-af28-e5de485597de",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "0b585739-0c3f-4817-ab65-31b8bfb3a470"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "aa73744c-fb61-4a04-bc31-1046d56912b7",
            "recordTags": [],
            "id": 1148,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Notify Newly Linked Team",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990980,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/e765911a-1d25-4687-9b88-52b365ec5406",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "resource": "warrooms",
                        "resources": [
                            "warrooms"
                        ],
                        "step_variables": {
                            "input": {
                                "records": [
                                    "{{vars.input.records[0]}}"
                                ]
                            }
                        },
                        "fieldbasedtrigger": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "array",
                                    "field": "owners",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "object",
                                    "field": "status",
                                    "value": "/api/3/picklists/1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                    "_value": {
                                        "@id": "/api/3/picklists/1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                        "display": "Active",
                                        "itemValue": "Active"
                                    },
                                    "operator": "eq"
                                }
                            ]
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
                    "uuid": "e765911a-1d25-4687-9b88-52b365ec5406",
                    "id": 4774
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Get difference of owners",
                    "description": null,
                    "arguments": {
                        "newTeams": "{{ vars.steps.Load_war_room_relationships[0].owners | json_query(\'[].actors[].\"@id\"\') | difference(vars.previous.data.owners) | list}}"
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "9890c3f9-06c6-4afc-abbe-6970c2debced",
                    "id": 4775
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load war room relationships",
                    "description": null,
                    "arguments": {
                        "query": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "id",
                                    "value": "{{vars.input.records[0].id}}",
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ],
                            "__selectFields": [
                                "owners",
                                "userOwners"
                            ]
                        },
                        "module": "warrooms?$limit=1&$relationships=true",
                        "checkboxFields": true,
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
                    "uuid": "cc81ea7b-b845-414a-bd2c-610f04d01e47",
                    "id": 4776
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Set email body",
                    "description": null,
                    "arguments": {
                        "params": {
                            "value": "<p>You were added to War Room #{{ vars.warRoom.id }}:</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Title</strong></td>\n<td>{{ vars.warRoom.title }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Summary</strong></td>\n<td>{{ vars.warRoom.summary }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Status</strong></td>\n<td>{{ vars.warRoom.status.itemValue }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Launched By</strong></td>\n<td>{{ vars.warRoom.launchedBy.firstname}} {{ vars.warRoom.launchedBy.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<br />\n<p><a href=\"{{ vars.warRoomLink }}\">Join War Room</a></p>\n<p>Thanks,<br />FortiSOAR&trade;</p>"
                        },
                        "version": "3.1.0",
                        "connector": "cyops_utilities",
                        "operation": "format_richtext",
                        "operationTitle": "Utils: Format as RichText",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "705",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "7fb81855-c21c-45d7-bd50-44fdb0ac08da",
                    "id": 4777
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send new notifications",
                    "description": null,
                    "arguments": {
                        "for_each": {
                            "item": "{{vars.uniqueUsers}}",
                            "condition": "{{\'people\' in vars.item}}"
                        },
                        "arguments": {
                            "userIri": "{{vars.item}}",
                            "warRoom": "{{vars.input.records[0]}}",
                            "emailBody": "{{vars.steps.Set_email_body.data[\'formatted_string\']}}",
                            "emailSubject": "You were added to War Room #{{ vars.warRoom.id }}: {{ vars.warRoom.title }}"
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/aa73744c-fb61-4a04-bc31-1046d56912b7"
                    },
                    "status": null,
                    "top": "840",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "3b7ca055-b288-414b-84a9-91fc6f48748f",
                    "id": 4778
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load owners from war room",
                    "description": null,
                    "arguments": {
                        "params": {
                            "iri": "{{vars.item}}?$relationships=true",
                            "body": "",
                            "method": "GET"
                        },
                        "version": "3.0.4",
                        "for_each": {
                            "item": "{{vars.newTeams}}",
                            "condition": ""
                        },
                        "connector": "cyops_utilities",
                        "operation": "make_cyops_request",
                        "operationTitle": "FSR: Make FortiSOAR API Call",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "683483f4-890d-4167-937d-c57a592e8331",
                    "id": 4779
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Combine teams and unique",
                    "description": null,
                    "arguments": {
                        "warRoom": "{{vars.input.records[0]}}",
                        "uniqueUsers": "{{ (vars.steps.Load_owners_from_war_room | json_query(\'[].data[].actors[]\')) | unique | list }}",
                        "warRoomLink": "https://{{globalVars.Server_fqhn}}{{vars.input.records[0][\'@id\'] | iriToLink}}"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "446faa3c-ed36-44ba-a9b7-1573008ff549",
                    "id": 4780
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Load war room relationships",
                    "targetStep": "/api/3/workflow_steps/cc81ea7b-b845-414a-bd2c-610f04d01e47",
                    "sourceStep": "/api/3/workflow_steps/e765911a-1d25-4687-9b88-52b365ec5406",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "b168e02a-e2bd-4854-a590-832c384a3764"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load war room relationships -> Get difference of userOwners and owners",
                    "targetStep": "/api/3/workflow_steps/9890c3f9-06c6-4afc-abbe-6970c2debced",
                    "sourceStep": "/api/3/workflow_steps/cc81ea7b-b845-414a-bd2c-610f04d01e47",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "8a81748d-7a07-4954-915f-f15195d8dd4e"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Get difference of owners -> Load owners from war room",
                    "targetStep": "/api/3/workflow_steps/683483f4-890d-4167-937d-c57a592e8331",
                    "sourceStep": "/api/3/workflow_steps/9890c3f9-06c6-4afc-abbe-6970c2debced",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "cf0b8b1f-cbd2-45c7-bc5d-2f46614bdd30"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load owners from war room -> Combine teams and unique",
                    "targetStep": "/api/3/workflow_steps/446faa3c-ed36-44ba-a9b7-1573008ff549",
                    "sourceStep": "/api/3/workflow_steps/683483f4-890d-4167-937d-c57a592e8331",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "82ef0a60-b703-4071-a5cb-afc27fb9910c"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Combine teams and unique -> Set email body",
                    "targetStep": "/api/3/workflow_steps/7fb81855-c21c-45d7-bd50-44fdb0ac08da",
                    "sourceStep": "/api/3/workflow_steps/446faa3c-ed36-44ba-a9b7-1573008ff549",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "0e841dd9-478c-4bc0-9cb1-0ff7ef2bc3cf"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set email body -> Send new notifications",
                    "targetStep": "/api/3/workflow_steps/3b7ca055-b288-414b-84a9-91fc6f48748f",
                    "sourceStep": "/api/3/workflow_steps/7fb81855-c21c-45d7-bd50-44fdb0ac08da",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "78ed7d72-c5ca-4903-bd42-856f337b85f6"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "c2aead2c-e312-4405-a44d-42fbd8077912",
            "recordTags": [
                "system"
            ],
            "id": 1149,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Notify Newly Linked User(s)",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1619252226,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/d6973514-cabc-4732-bbb0-fe1b5f07b999",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "resource": "warrooms",
                        "resources": [
                            "warrooms"
                        ],
                        "step_variables": {
                            "input": {
                                "records": [
                                    "{{vars.input.records[0]}}"
                                ]
                            }
                        },
                        "fieldbasedtrigger": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "array",
                                    "field": "userOwners",
                                    "value": null,
                                    "_value": null,
                                    "operator": "changed"
                                },
                                {
                                    "type": "object",
                                    "field": "status",
                                    "value": "/api/3/picklists/1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                    "_value": {
                                        "@id": "/api/3/picklists/1c4fe134-de72-40d9-b35f-deb8dadc6800",
                                        "display": "Active",
                                        "itemValue": "Active"
                                    },
                                    "operator": "eq"
                                }
                            ]
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
                    "uuid": "d6973514-cabc-4732-bbb0-fe1b5f07b999",
                    "id": 4781
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Get difference of userOwners and owners",
                    "description": null,
                    "arguments": {
                        "warRoom": "{{vars.input.records[0]}}",
                        "newUsers": "{{ vars.steps.Load_war_room_relationships[0].userOwners | json_query(\'[].\"@id\"\') | difference(vars.previous.data.userOwners) | list}}",
                        "warRoomLink": "https://{{globalVars.Server_fqhn}}{{vars.input.records[0][\'@id\'] | iriToLink}}"
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "07066686-ec08-4e69-a996-de9beb04ad84",
                    "id": 4782
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send new notifications",
                    "description": null,
                    "arguments": {
                        "for_each": {
                            "item": "{{vars.newUsers}}",
                            "condition": ""
                        },
                        "arguments": {
                            "userIri": "{{vars.item}}",
                            "warRoom": "{{vars.input.records[0]}}",
                            "emailBody": "{{vars.steps.Set_email_body.data[\'formatted_string\']}}",
                            "emailSubject": "You were added to War Room #{{ vars.warRoom.id }}: {{ vars.warRoom.title }}",
                            "emailAttachmentIris": ""
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/aa73744c-fb61-4a04-bc31-1046d56912b7"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "b0c92621-8381-4d19-867f-90bb4b7d8a07",
                    "id": 4783
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Set email body",
                    "description": null,
                    "arguments": {
                        "params": {
                            "value": "<p>You were added to War Room #{{ vars.warRoom.id }}:</p>\n<table style=\"min-width: 300px; max-width: 800px; border: 1px solid #dddddd;\">\n<tbody>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Title</strong></td>\n<td>{{ vars.warRoom.title }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Summary</strong></td>\n<td>{{ vars.warRoom.summary }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Status</strong></td>\n<td>{{ vars.warRoom.status.itemValue }}</td>\n</tr>\n<tr style=\"border-bottom: 1px solid #ddd;\">\n<td style=\"text-align: right; padding: 5px; border-right: 1px solid #ddd;\"><strong>Launched By</strong></td>\n<td>{{ vars.warRoom.launchedBy.firstname}} {{ vars.warRoom.launchedBy.lastname}}</td>\n</tr>\n</tbody>\n</table>\n<br />\n<p><a href=\"{{ vars.warRoomLink }}\">Join War Room</a></p>\n<p>Thanks,<br />FortiSOAR&trade;</p>"
                        },
                        "version": "3.1.0",
                        "connector": "cyops_utilities",
                        "operation": "format_richtext",
                        "operationTitle": "Utils: Format as RichText",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "6c0ac3ec-e75a-4004-80bf-be08d5c0f067",
                    "id": 4784
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load war room relationships",
                    "description": null,
                    "arguments": {
                        "query": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "id",
                                    "value": "{{vars.input.records[0].id}}",
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ],
                            "__selectFields": [
                                "owners",
                                "userOwners"
                            ]
                        },
                        "module": "warrooms?$limit=1&$relationships=true",
                        "checkboxFields": true,
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
                    "uuid": "58960633-87a8-43f3-a1ac-48d0818bd9e2",
                    "id": 4785
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Load war room relationships",
                    "targetStep": "/api/3/workflow_steps/58960633-87a8-43f3-a1ac-48d0818bd9e2",
                    "sourceStep": "/api/3/workflow_steps/d6973514-cabc-4732-bbb0-fe1b5f07b999",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "32d73eab-968f-4482-b091-2df12248f762"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load war room relationships -> Get difference of userOwners and owners",
                    "targetStep": "/api/3/workflow_steps/07066686-ec08-4e69-a996-de9beb04ad84",
                    "sourceStep": "/api/3/workflow_steps/58960633-87a8-43f3-a1ac-48d0818bd9e2",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "fed26cec-4880-4baa-bff4-28c6e1deb11c"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Get difference of userOwners and owners -> Set email body",
                    "targetStep": "/api/3/workflow_steps/6c0ac3ec-e75a-4004-80bf-be08d5c0f067",
                    "sourceStep": "/api/3/workflow_steps/07066686-ec08-4e69-a996-de9beb04ad84",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "60fdee19-2f6f-49d3-b70f-af71cb202bc4"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Set email body -> Send new notifications",
                    "targetStep": "/api/3/workflow_steps/b0c92621-8381-4d19-867f-90bb4b7d8a07",
                    "sourceStep": "/api/3/workflow_steps/6c0ac3ec-e75a-4004-80bf-be08d5c0f067",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "6c6d3446-715e-4412-a7da-1267a789f6a1"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "d5eb1767-524a-420a-9b66-fc277453b556",
            "recordTags": [
                "system"
            ],
            "id": 1150,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": "/api/3/people/3451141c-bac6-467c-8d72-85e0fab569ce",
            "modifyDate": 1619252225,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Send Email Notification",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [
                "warRoom",
                "emailBody",
                "emailSubject",
                "emailAttachmentIris"
            ],
            "synchronous": false,
            "lastModifyDate": 1618990657,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/381f8403-a5b8-4449-b406-7b1064e6836b",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "step_variables": {
                            "input": {
                                "params": {
                                    "warRoom": "{{ vars.warRoom }}",
                                    "warroom": "{{ vars.warroom }}",
                                    "emailBody": "{{ vars.emailBody }}",
                                    "emailSubject": "{{ vars.emailSubject }}",
                                    "emailAttachmentIris": "{{ vars.emailAttachmentIris }}"
                                }
                            }
                        }
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
                    "uuid": "381f8403-a5b8-4449-b406-7b1064e6836b",
                    "id": 4786
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load owners from war room",
                    "description": null,
                    "arguments": {
                        "params": {
                            "iri": "{{vars.input.params.warRoom[\'@id\']}}/owners?$relationships=true",
                            "body": "",
                            "method": "GET"
                        },
                        "version": "3.0.4",
                        "connector": "cyops_utilities",
                        "operation": "make_cyops_request",
                        "operationTitle": "FSR: Make FortiSOAR API Call",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "c99319d9-9880-41ab-8859-c3b64202033c",
                    "id": 4787
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Load user owners from war room",
                    "description": null,
                    "arguments": {
                        "params": {
                            "iri": "{{vars.input.params.warRoom[\'@id\']}}/userOwners?$relationships=true",
                            "body": "",
                            "method": "GET"
                        },
                        "version": "3.0.4",
                        "connector": "cyops_utilities",
                        "operation": "make_cyops_request",
                        "operationTitle": "FSR: Make FortiSOAR API Call",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                    "uuid": "ef1a81db-883a-4f68-8f1b-1534a512f60e",
                    "id": 4788
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Combine lists and uniq",
                    "description": null,
                    "arguments": {
                        "users": "{{ ((vars.steps.Load_owners_from_war_room.data[\'hydra:member\'] | json_query(\'[].actors[].\"@id\"\')) + (vars.steps.Load_user_owners_from_war_room.data[\'hydra:member\'] | json_query(\'[].\"@id\"\'))) | unique | list }}"
                    },
                    "status": null,
                    "top": "435",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
                    "uuid": "17c4bc6e-e597-492d-b856-3a1f3034872a",
                    "id": 4789
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Send each email",
                    "description": null,
                    "arguments": {
                        "for_each": {
                            "item": "{{vars.users}}",
                            "condition": "{{\'people\' in vars.item}}"
                        },
                        "arguments": {
                            "userIri": "{{vars.item}}",
                            "warRoom": "{{vars.input.params.warRoom}}",
                            "emailBody": "{{vars.input.params.emailBody}}",
                            "emailSubject": "{{vars.input.params.emailSubject}}",
                            "emailAttachmentIris": "{{vars.input.params.emailAttachmentIris}}"
                        },
                        "apply_async": false,
                        "step_variables": [],
                        "workflowReference": "/api/3/workflows/aa73744c-fb61-4a04-bc31-1046d56912b7"
                    },
                    "status": null,
                    "top": "570",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
                    "uuid": "e32a1c2d-c320-44d0-b54f-0f07e89bdb23",
                    "id": 4790
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Load owners from war room",
                    "targetStep": "/api/3/workflow_steps/c99319d9-9880-41ab-8859-c3b64202033c",
                    "sourceStep": "/api/3/workflow_steps/381f8403-a5b8-4449-b406-7b1064e6836b",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "c9fc86b0-8649-43ad-aa21-9ca80f362802"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load owners from war room -> Load user owners from war room",
                    "targetStep": "/api/3/workflow_steps/ef1a81db-883a-4f68-8f1b-1534a512f60e",
                    "sourceStep": "/api/3/workflow_steps/c99319d9-9880-41ab-8859-c3b64202033c",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "1a783d06-ec98-4abb-9a40-46b0d5d69161"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load user owners from war room -> Combine lists and uniq",
                    "targetStep": "/api/3/workflow_steps/17c4bc6e-e597-492d-b856-3a1f3034872a",
                    "sourceStep": "/api/3/workflow_steps/ef1a81db-883a-4f68-8f1b-1534a512f60e",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "873b498a-8dac-4337-a785-94b1b69f3ea6"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Combine lists and uniq -> Send each email",
                    "targetStep": "/api/3/workflow_steps/e32a1c2d-c320-44d0-b54f-0f07e89bdb23",
                    "sourceStep": "/api/3/workflow_steps/17c4bc6e-e597-492d-b856-3a1f3034872a",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "7eb93fac-c84e-4e2e-95e9-1f082854dc32"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "e1fd3374-1f75-4f0b-8252-050aabd0a93b",
            "recordTags": [],
            "id": 1151,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        },
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Set up War Room from Alerts",
            "aliasName": null,
            "tag": null,
            "description": null,
            "isActive": false,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1618990564,
            "collection": "/api/3/workflow_collections/67ee8089-6620-47a8-ad25-6cb7e20cf16e",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/d473f9af-77cc-439b-888c-1f70b1c292b2",
            "steps": [
                {
                    "@type": "WorkflowStep",
                    "name": "Load relationships",
                    "description": null,
                    "arguments": {
                        "query": {
                            "sort": [],
                            "limit": 30,
                            "logic": "AND",
                            "filters": [
                                {
                                    "type": "primitive",
                                    "field": "id",
                                    "value": "{{vars.input.records[0].id}}",
                                    "operator": "eq",
                                    "_operator": "eq"
                                }
                            ]
                        },
                        "module": "alerts?$limit=30&$relationships=true",
                        "step_variables": []
                    },
                    "status": null,
                    "top": "165",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
                    "uuid": "3594a16b-1d61-4258-9b9f-349e1472e8cc",
                    "id": 4791
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Create War Room",
                    "description": null,
                    "arguments": {
                        "message": {
                            "tags": [],
                            "type": "/api/3/picklists/ff599189-3eeb-4c86-acb0-a7915e85ac3b",
                            "tenant": "",
                            "content": "War Room created\nName: {{vars.input.params.warRoomTitle}}\nStatus: {{vars.input.params.status.itemValue}}\n\n[Go to War Room]({{ vars.result[\'@id\'] | iriToLink(\'\') }})\n",
                            "records": "{{vars.input.records[0][\'@id\']}}"
                        },
                        "resource": {
                            "tasks": "{{vars.steps.Load_relationships[0].tasks}}",
                            "title": "{{vars.input.params.warRoomTitle}}",
                            "alerts": "{{vars.input.records[0][\'@id\']}}",
                            "assets": "{{vars.steps.Load_relationships[0].assets}}",
                            "owners": "{{vars.steps.Load_relationships[0].owners}}",
                            "status": "{{vars.input.params.status[\'@id\']}}",
                            "summary": "{{vars.input.params.warRoomDescription}}",
                            "__replace": "",
                            "incidents": "{{vars.steps.Load_relationships[0].incidents}}",
                            "indicators": "{{vars.steps.Load_relationships[0].indicators}}"
                        },
                        "_showJson": false,
                        "operation": "Overwrite",
                        "collection": "/api/3/warrooms",
                        "__recommend": [],
                        "fieldOperation": {
                            "recordTags": "Overwrite"
                        },
                        "step_variables": []
                    },
                    "status": null,
                    "top": "300",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/2597053c-e718-44b4-8394-4d40fe26d357",
                    "uuid": "40546cf7-662d-48a4-b384-fe4ac24faa0f",
                    "id": 4792
                },
                {
                    "@type": "WorkflowStep",
                    "name": "Start",
                    "description": null,
                    "arguments": {
                        "route": "91fd1e0a-6780-448c-957a-91cdf74314bd",
                        "title": "Set up War Room",
                        "resources": [
                            "alerts"
                        ],
                        "inputVariables": [
                            {
                                "name": "status",
                                "type": "picklists",
                                "label": "Status",
                                "title": "Picklist",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "picklist",
                                "formType": "picklist",
                                "required": true,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "dataSource": {
                                    "model": "picklists",
                                    "query": {
                                        "sort": [
                                            {
                                                "field": "orderIndex",
                                                "direction": "ASC"
                                            }
                                        ],
                                        "logic": "AND",
                                        "filters": [
                                            {
                                                "field": "listName__name",
                                                "value": "WarRoomStatus",
                                                "operator": "eq"
                                            }
                                        ]
                                    },
                                    "displayConditions": {
                                        "1c4fe134-de72-40d9-b35f-deb8dadc6800": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "4584faf4-fe11-4039-8c73-157e37e477f4": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "55a2f3e7-8354-4c41-9120-f6c3559903f8": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "5d1a13e5-0b83-4e8c-ac42-2bab13385b58": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "cc063ee2-b0ac-47a0-a123-e40c8bcea3a5": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        },
                                        "f433458e-0cf8-4130-94d3-31b60ed4add7": {
                                            "conditions": null,
                                            "visibility": "visible"
                                        }
                                    }
                                },
                                "searchable": false,
                                "templateUrl": "app/components/form/fields/typeahead.html",
                                "defaultValue": {
                                    "id": 5512,
                                    "@id": "/api/3/picklists/55a2f3e7-8354-4c41-9120-f6c3559903f8",
                                    "icon": null,
                                    "@type": "Picklist",
                                    "color": "#cfcfcf",
                                    "display": "Draft",
                                    "listName": "/api/3/picklist_names/ab8bbdb4-2cf9-4695-90ef-9f1f825d2b1f",
                                    "itemValue": "Draft",
                                    "orderIndex": 0
                                },
                                "displayTemplate": "{{ itemValue }}",
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            },
                            {
                                "name": "warRoomTitle",
                                "type": "string",
                                "label": "War Room Title",
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "text",
                                "required": true,
                                "_expanded": true,
                                "moduleField": "name",
                                "useRecordFieldDefault": true
                            },
                            {
                                "name": "warRoomDescription",
                                "type": "string",
                                "label": "War Room Description",
                                "tooltip": "",
                                "dataType": "text",
                                "formType": "text",
                                "required": false,
                                "_expanded": true,
                                "moduleField": "description",
                                "useRecordFieldDefault": true
                            },
                            {
                                "name": "includeAnalysts",
                                "type": "boolean",
                                "label": "Include all analysts involved with this alert as War Room responders",
                                "title": "Checkbox",
                                "usable": true,
                                "tooltip": "",
                                "dataType": "checkbox",
                                "formType": "checkbox",
                                "required": false,
                                "_expanded": true,
                                "mmdUpdate": true,
                                "collection": false,
                                "searchable": true,
                                "templateUrl": "app/components/form/fields/checkbox.html",
                                "_nameTouched": true,
                                "defaultValue": true,
                                "lengthConstraint": false,
                                "allowedEncryption": false,
                                "allowedGridColumn": true,
                                "useRecordFieldDefault": false
                            }
                        ],
                        "step_variables": {
                            "input": {
                                "params": {
                                    "status": "{{vars.request.data[\"status\"]}}",
                                    "warRoomTitle": "{{vars.request.data[\"warRoomTitle\"]}}",
                                    "includeAnalysts": "{{vars.request.data[\"includeAnalysts\"]}}",
                                    "warRoomDescription": "{{vars.request.data[\"warRoomDescription\"]}}"
                                },
                                "records": "{{vars.input.records}}"
                            }
                        },
                        "_promptexpanded": true,
                        "displayConditions": {
                            "alerts": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            },
                            "incidents": {
                                "sort": [],
                                "limit": 30,
                                "logic": "AND",
                                "filters": []
                            }
                        },
                        "executeButtonText": "Execute",
                        "noRecordExecution": false,
                        "singleRecordExecution": true
                    },
                    "status": null,
                    "top": "30",
                    "left": "125",
                    "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
                    "uuid": "d473f9af-77cc-439b-888c-1f70b1c292b2",
                    "id": 4793
                }
            ],
            "routes": [
                {
                    "@type": "WorkflowRoute",
                    "name": "Start -> Load relationships",
                    "targetStep": "/api/3/workflow_steps/3594a16b-1d61-4258-9b9f-349e1472e8cc",
                    "sourceStep": "/api/3/workflow_steps/d473f9af-77cc-439b-888c-1f70b1c292b2",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "7bd2076f-4612-4480-9df1-5e611345df6b"
                },
                {
                    "@type": "WorkflowRoute",
                    "name": "Load relationships -> Create War Room",
                    "targetStep": "/api/3/workflow_steps/40546cf7-662d-48a4-b384-fe4ac24faa0f",
                    "sourceStep": "/api/3/workflow_steps/3594a16b-1d61-4258-9b9f-349e1472e8cc",
                    "label": null,
                    "isExecuted": false,
                    "uuid": "6c9d59bc-c0df-4737-803f-340bd99bccd0"
                }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "f032baef-0fde-4622-8706-4a89a7715e78",
            "recordTags": [],
            "id": 1152,
            "createUser": null,
            "createDate": 1619169977,
            "modifyUser": null,
            "modifyDate": 1619169977,
            "owners": [],
            "isPrivate": false
        }
    ]';

    public function up(Schema $schema): void
    {
        $playbook = json_decode($this->playbooks, true);
        foreach ($playbook as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    public function down(Schema $schema): void
    {
        $playbooks = json_decode($this->playbooks, true);
        foreach ($playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
