<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210225030622 extends AbstractMigration implements ContainerAwareInterface
{
  /** @var ContainerInterface */
  private $container;

  public function setContainer(ContainerInterface $container = null)
  {
    $this->container = $container;
  }

  private function deleteWorkflow($workflow)
  {
    /** @var ResourceInterface $workflowResource */
    $workflowResource = Workflow::class;
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
    $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
    if ($workflowToDelete == null || sizeof($workflowToDelete) == 0) {
      $workflowManager->flush();
      return;
    }
    $workflowManager->remove($workflowToDelete);
    $workflowManager->flush();
  }


  private function addWorkflow($workflow)
  {
    $workflowResource = Workflow::class;
    // $context = $workflowResource->getDenormalizationContext();
    $context["ignore_field_authorization"] = true;
    $context["ignore_authorization"] = true;
    $context["groups"] = ["fsr_all", "password"];
    $context["fetch_data"] = false;
    $workflowObject = $this->container->get('serializer')->deserialize(
      json_encode($workflow),
      $workflowResource,
      'jsonld',
      $context
    );
    /** @var ManagerRegistry $objectManager */
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
    $workflowManager->persist($workflowObject);
    $workflowManager->flush();
  }



  public function getDescription(): string
  {
    return 'Comment Workflows';
  }

  private $playbooks = '
  [
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "> Comment - Send Email Notification",
      "aliasName": null,
      "description": null,
      "collection": "/api/3/workflow_collections/94126e6b-16a4-46c0-8ebe-228cc013afac",
      "tag": null,
      "isActive": true,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [
        "comment",
        "userIri",
        "parentRecord"
      ],
      "synchronous": false,
      "triggerStep": "/api/3/workflow_steps/bdeb1709-fd00-4b60-892a-ff0d6443cadf",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "step_variables": {
              "input": {
                "params": {
                  "comment": "{{ vars.comment }}",
                  "userIri": "{{ vars.userIri }}",
                  "parentRecord": "{{ vars.parentRecord }}"
                }
              }
            }
          },
          "status": null,
          "left": "125",
          "top": "30",
          "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
          "uuid": "bdeb1709-fd00-4b60-892a-ff0d6443cadf"
        },
        {
          "@type": "WorkflowStep",
          "name": "Get User Details",
          "description": null,
          "arguments": {
            "params": {
              "iri": "{{vars.input.params.userIri}}?$relationships=true",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.0",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": {
              "userEmail": "{{vars.result.data.email}}"
            }
          },
          "status": null,
          "left": "125",
          "top": "165",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "0d56d720-9e45-4f94-aacc-87c186505616"
        },
        {
          "@type": "WorkflowStep",
          "name": "Get User Email Settings",
          "description": null,
          "arguments": {
            "params": {
              "iri": "{{vars.steps.Get_User_Details.data[\'@settings\']}}",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.0",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": []
          },
          "status": null,
          "left": "125",
          "top": "300",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "678210e6-e0a6-4049-af44-c0d76ca6a6a8"
        },
        {
          "@type": "WorkflowStep",
          "name": "Send Email Notification",
          "description": null,
          "arguments": {
            "when": "{{vars.steps.Get_User_Email_Settings.data.user.notifications.enableCommentMentionNotifications if vars.steps.Get_User_Email_Settings.data.user.notifications.enableCommentMentionNotifications is defined else True}}",
            "config": "88c3d39c-2fa9-4731-b00d-29815008f17c",
            "params": {
              "cc": "",
              "to": "{{vars.userEmail}}",
              "bcc": "",
              "from": "FortiSOAR",
              "type": "Manual Input",
              "content": "<table style=\"font-family: Verdana, Geneva, sans-serif; border: 3px solid #cccccc; border-radius: 3px;\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n<tbody>\n<tr>\n<td style=\"padding: 20px; width: 100%;\">Hi,<br /><br />\n<table style=\"font-family: Verdana, Geneva, sans-serif; width: 100%;\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n<tbody>\n<tr>\n<td><a style=\"text-decoration: none;\" href=\"https://{{globalVars.Server_fqhn}}/security/people/{{vars.input.params.comment.createUser[\'@id\'].split(\'/\')[-1]}}\" target=\"_blank\" rel=\"noopener\">{{vars.input.params.comment.createUser.firstname}} {{vars.input.params.comment.createUser.lastname}}</a> has mentioned you in a FortiSOAR {{vars.input.params.parentRecord[\'@type\']}}, <a style=\"text-decoration: none;\" href=\"https://{{globalVars.Server_fqhn}}{{vars.input.params.parentRecord[\'@id]\'] | iriToLink}}\" target=\"_blank\" rel=\"noopener\">{{vars.input.params.parentRecord.name}}</a><br /><br /></td>\n</tr>\n</tbody>\n</table>\n<table style=\"font-family: Verdana, Geneva, sans-serif; border: 2px solid #cccccc; background-color: #e4e0e0; padding: 10px; margin-bottom: 20px; border-radius: 3px; width: 100%;\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n<tbody>\n<tr>\n<td style=\"font-weight: 500; font-size: 16px;\">{{vars.input.params.comment.createUser.firstname}} {{vars.input.params.comment.createUser.lastname}}</td>\n</tr>\n<tr>\n<td>\n<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n<tbody>\n<tr>\n<td style=\"font-size: 14px;\">{{vars.formatted_content.data.html}}</td>\n</tr>\n</tbody>\n</table>\n</td>\n</tr>\n</tbody>\n</table>\n<table style=\"font-family: Verdana, Geneva, sans-serif; margin-bottom: 20px; border-radius: 3px;\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n<tbody>\n<tr>\n<td style=\"background-color: #22a6af; padding: 6px;\"><a style=\"text-decoration: none; color: #ffffff;\" href=\"https://{{globalVars.Server_fqhn}}{{vars.input.params.parentRecord[\'@id\'] | iriToLink}}\" target=\"_blank\" rel=\"noopener\">View {{vars.input.params.parentRecord[\'@type\']}} In FortiSOAR <svg fill=\"#fdfcfc\" height=\"14px\" width=\"14px\" viewbox=\"0 0 512 512\" xmlns=\"http://www.w3.org/2000/svg\"><path d=\"M432,320H400a16,16,0,0,0-16,16V448H64V128H208a16,16,0,0,0,16-16V80a16,16,0,0,0-16-16H48A48,48,0,0,0,0,112V464a48,48,0,0,0,48,48H400a48,48,0,0,0,48-48V336A16,16,0,0,0,432,320ZM488,0h-128c-21.37,0-32.05,25.91-17,41l35.73,35.73L135,320.37a24,24,0,0,0,0,34L157.67,377a24,24,0,0,0,34,0L435.28,133.32,471,169c15,15,41,4.5,41-17V24A24,24,0,0,0,488,0Z\"></path></svg></a></td>\n</tr>\n</tbody>\n</table>\n</td>\n</tr>\n<tr style=\"background-color: #e4e0e0;\">\n<td style=\"padding: 20px; font-size: 12px; color: #a7a4a4;\">Please do not reply this mail as this is an automated mail service.<br />Copyright &copy; {{arrow.utcnow().year}} Fortinet Inc. All Rights Reserved<br /><img style=\"padding-top: 20px;\" title=\"FortiSOAR\" src=\"https://marvel-b1-cdn.bc0a.com/f00000000216283/www.fortinet.com/content/dam/fortinet/images/icons/Fortinet-Logo-Black.png\" width=\"150\" height=\"16\" /></td>\n</tr>\n</tbody>\n</table>",
              "subject": "FortiSOAR: {{vars.input.params.comment.createUser.firstname}} {{vars.input.params.comment.createUser.lastname}} has mentioned you in a comment",
              "iri_list": "",
              "body_type": "Rich Text",
              "file_name": "",
              "file_path": ""
            },
            "version": "2.4.0",
            "from_str": "admin@example.com",
            "connector": "smtp",
            "operation": "send_email_new",
            "operationTitle": "Send Email",
            "step_variables": []
          },
          "status": null,
          "left": "125",
          "top": "570",
          "stepType": "/api/3/workflow_step_types/4c0019b2-055c-44d0-968c-678a0c2d762e",
          "uuid": "09f4897b-515c-48fa-a2ca-65f0cacfcbef"
        },
        {
          "@type": "WorkflowStep",
          "name": "Format Comment Content to HTML",
          "description": null,
          "arguments": {
            "params": {
              "markdown_string": "{{vars.input.params.comment.content}}"
            },
            "version": "3.1.0",
            "connector": "cyops_utilities",
            "operation": "markdown_to_html",
            "operationTitle": "Utils: Convert Markdown string to HTML",
            "step_variables": {
              "formatted_content": "{{vars.result}}"
            }
          },
          "status": null,
          "left": "125",
          "top": "435",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "61ac60d3-f13e-430d-872a-85f305ea8ca4"
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "uuid": "17fc2414-b230-44a2-8c68-45d050414ada",
          "name": "Start -> Get User Settings",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/0d56d720-9e45-4f94-aacc-87c186505616",
          "sourceStep": "/api/3/workflow_steps/bdeb1709-fd00-4b60-892a-ff0d6443cadf"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "9b90c947-ec77-4d6b-b450-6f769dafb7d2",
          "name": "Get User Details -> Get User Email Settings",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/678210e6-e0a6-4049-af44-c0d76ca6a6a8",
          "sourceStep": "/api/3/workflow_steps/0d56d720-9e45-4f94-aacc-87c186505616"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "a178aaea-97af-45a0-a964-bb5f028a01fe",
          "name": "Get User Email Settings -> Format Comment Content to HTML",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/61ac60d3-f13e-430d-872a-85f305ea8ca4",
          "sourceStep": "/api/3/workflow_steps/678210e6-e0a6-4049-af44-c0d76ca6a6a8"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "1b6a57dd-36a5-45e9-bf9e-940bd5bd1408",
          "name": "Format Comment Content to HTML -> Send Email Notification",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/09f4897b-515c-48fa-a2ca-65f0cacfcbef",
          "sourceStep": "/api/3/workflow_steps/61ac60d3-f13e-430d-872a-85f305ea8ca4"
        }
      ],
      "versions": null,
      "lastModifyDate": 1612551779,
      "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
      "uuid": "1049dfef-c955-43e1-8933-00f80553f5d0",
      "recordTags": [],
      "isPrivate": false,
      "owners": []
    },
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "Comment > Notify Mentioned/Tagged People on Comment Update",
      "aliasName": null,
      "description": null,
      "collection": "/api/3/workflow_collections/94126e6b-16a4-46c0-8ebe-228cc013afac",
      "tag": null,
      "isActive": true,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [],
      "synchronous": false,
      "triggerStep": "/api/3/workflow_steps/cbb31391-d058-4f07-bbbe-03e8b3ec18b5",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "resource": "comments",
            "resources": [
              "comments"
            ],
            "step_variables": {
              "input": {
                "records": [
                  "{{vars.input.records[0]}}"
                ]
              }
            },
            "fieldbasedtrigger": {
              "sort": [],
              "limit": 30,
              "logic": "AND",
              "filters": [
                {
                  "type": "primitive",
                  "field": "peopleUpdated",
                  "value": true,
                  "operator": "eq",
                  "_operator": "eq"
                }
              ]
            }
          },
          "status": null,
          "left": "125",
          "top": "30",
          "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
          "uuid": "cbb31391-d058-4f07-bbbe-03e8b3ec18b5"
        },
        {
          "@type": "WorkflowStep",
          "name": "Fetch Comment Record",
          "description": null,
          "arguments": {
            "params": {
              "iri": "{{vars.input.records[0][\'@id\']}}?$relationships=true",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.0",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": {
              "taggedPeople": "{{vars.result.data.people}}",
              "commentParentRecord": "{%set list_objects={} %}{% for key, value in vars.result.data.items() %}{% if value is iterable and value is not string and value is not mapping and value|length > 0 and key not in [\'recordTags\',\'userOwners\',\'people\',\'owners\',\'replies\'] %}{% set temp=list_objects.update({key: value}) %}{% endif %}{% endfor %}{{ (list_objects.values()|list)[0][0]}}"
            }
          },
          "status": null,
          "left": "125",
          "top": "165",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "89f8ea19-b1b4-4df5-8d7d-b94abfdbf760"
        },
        {
          "@type": "WorkflowStep",
          "name": "Send Notification",
          "description": null,
          "arguments": {
            "for_each": {
              "item": "{{vars.taggedPeople}}",
              "condition": ""
            },
            "arguments": {
              "comment": "{{vars.steps.Fetch_Comment_Record.data}}",
              "userIri": "{{vars.item[\'@id\']}}",
              "parentRecord": "{{vars.commentParentRecord}}"
            },
            "apply_async": false,
            "step_variables": [],
            "workflowReference": "/api/3/workflows/1049dfef-c955-43e1-8933-00f80553f5d0"
          },
          "status": null,
          "left": "125",
          "top": "300",
          "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
          "uuid": "5d319684-762d-4f5f-8001-e57bd2bb8e54"
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "uuid": "b1b56521-1379-46ff-bdef-6800dc2c2691",
          "name": "Get Comment Record -> Send Notification",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/5d319684-762d-4f5f-8001-e57bd2bb8e54",
          "sourceStep": "/api/3/workflow_steps/89f8ea19-b1b4-4df5-8d7d-b94abfdbf760"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "4fbf8636-78b5-401b-8373-5bac5192509e",
          "name": "Start -> Get Comment Record",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/89f8ea19-b1b4-4df5-8d7d-b94abfdbf760",
          "sourceStep": "/api/3/workflow_steps/cbb31391-d058-4f07-bbbe-03e8b3ec18b5"
        }
      ],
      "versions": null,
      "lastModifyDate": 1612219270,
      "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
      "uuid": "4ce3a111-363e-4c55-83a1-4bc6823f3252",
      "recordTags": [],
      "isPrivate": false,
      "owners": []
    },
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "Comment > Notify Mentioned/Tagged People on Comment Create",
      "aliasName": null,
      "description": null,
      "collection": "/api/3/workflow_collections/94126e6b-16a4-46c0-8ebe-228cc013afac",
      "tag": null,
      "isActive": true,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [],
      "synchronous": false,
      "triggerStep": "/api/3/workflow_steps/95563b79-0d67-4060-b586-d4033ad132fe",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "resource": "comments",
            "resources": [
              "comments"
            ],
            "step_variables": {
              "input": {
                "records": [
                  "{{vars.input.records[0]}}"
                ]
              }
            },
            "fieldbasedtrigger": {
              "sort": [],
              "limit": 30,
              "logic": "AND",
              "filters": [
                {
                  "type": "primitive",
                  "field": "peopleUpdated",
                  "value": true,
                  "operator": "eq",
                  "_operator": "eq"
                }
              ]
            }
          },
          "status": null,
          "left": "125",
          "top": "30",
          "stepType": "/api/3/workflow_step_types/ea155646-3821-4542-9702-b246da430a8d",
          "uuid": "95563b79-0d67-4060-b586-d4033ad132fe"
        },
        {
          "@type": "WorkflowStep",
          "name": "Fetch Comment Record",
          "description": null,
          "arguments": {
            "params": {
              "iri": "{{vars.input.records[0][\'@id\']}}?$relationships=true",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.0",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": {
              "taggedPeople": "{{vars.result.data.people}}",
              "commentParentRecord": "{%set list_objects={} %}{% for key, value in vars.result.data.items() %}{% if value is iterable and value is not string and value is not mapping and value|length > 0 and key not in [\'recordTags\',\'userOwners\',\'people\',\'owners\',\'replies\'] %}{% set temp=list_objects.update({key: value}) %}{% endif %}{% endfor %}{{ (list_objects.values()|list)[0][0]}}"
            }
          },
          "status": null,
          "left": "125",
          "top": "165",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "85395c3f-11df-4a49-b138-e56f29e07033"
        },
        {
          "@type": "WorkflowStep",
          "name": "Send Notification",
          "description": null,
          "arguments": {
            "for_each": {
              "item": "{{vars.taggedPeople}}",
              "condition": ""
            },
            "arguments": {
              "comment": "{{vars.steps.Fetch_Comment_Record.data}}",
              "userIri": "{{vars.item[\'@id\']}}",
              "parentRecord": "{{vars.commentParentRecord}}"
            },
            "apply_async": false,
            "step_variables": [],
            "workflowReference": "/api/3/workflows/1049dfef-c955-43e1-8933-00f80553f5d0"
          },
          "status": null,
          "left": "125",
          "top": "300",
          "stepType": "/api/3/workflow_step_types/74932bdc-b8b6-4d24-88c4-1a4dfbc524f3",
          "uuid": "3e4ce543-3640-4e6b-9e27-49c74fc0908b"
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "uuid": "6020997c-8102-49b6-95ba-7a80fd1746c9",
          "name": "Start -> Get Comment Record",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/85395c3f-11df-4a49-b138-e56f29e07033",
          "sourceStep": "/api/3/workflow_steps/95563b79-0d67-4060-b586-d4033ad132fe"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "2f7313d1-7ffc-4558-bb5f-8c34c37cfb0e",
          "name": "Get Comment Record -> Send Notification",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/3e4ce543-3640-4e6b-9e27-49c74fc0908b",
          "sourceStep": "/api/3/workflow_steps/85395c3f-11df-4a49-b138-e56f29e07033"
        }
      ],
      "versions": null,
      "lastModifyDate": 1612219290,
      "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
      "uuid": "92071320-e6a8-4598-a04e-727ea98585fb",
      "recordTags": [],
      "isPrivate": false,
      "owners": []
    }
  ]';

  public function up(Schema $schema): void
  {
    $playbooks = json_decode($this->playbooks, true);
    foreach ($playbooks as $playbook) {
      $this->addWorkflow($playbook);
    }
  }

  public function down(Schema $schema): void
  {
    $playbooks = json_decode($this->playbooks, true);
    foreach ($playbooks as $playbook) {
      $this->deleteWorkflow($playbook);
    }
  }
}
