<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use App\Entity\ModelMetadata\ModelMetadata;
use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210415195343 extends AbstractMigration implements ContainerAwareInterface
{

    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return 'Migration for adding default value to ID field for all the external module';
    }

    public function up(Schema $schema): void
    {
        $idPerModule = false;
        if ($this->container->hasParameter('id_per_module') && $this->container->getParameter('id_per_module')) {
            $idPerModule = true;
        }
        $managerRegistry = $this->container->get('doctrine');
        $entityManager = $managerRegistry->getManagerForClass(ModelMetadata::class);
        $entityRepository = $entityManager->getRepository(ModelMetadata::class);

        $queryBuilder = $entityRepository->createQueryBuilder('o');
        $modelObj = $queryBuilder->andWhere('o.indexable = :indexable')
            ->setParameter('indexable', 'true')
            ->andWhere($queryBuilder->expr()->isNull('o.parentType'))
            ->getQuery()
            ->execute();
        foreach ($modelObj as $model) {
            $tableName = $model->getTableName();
            $seqName = $tableName . '_id_seq';
            if ($idPerModule) {
                $this->addSql("alter table $tableName alter column id set default nextval('$seqName')");
            } else {
                $this->addSql("alter table $tableName alter column id set default nextval('global_id_seq')");
            }
        }

        $system_module_table = [
            'actors', 'workflow_step_types', 'workflow_versions', 'workflows',
            'workflow_collections', 'picklists', 'widgets', 'picklist_names', 'import_jobs', 'export_templates', 'export_jobs'
        ];
        foreach ($system_module_table as $tableName) {
            $seqName = $tableName . '_id_seq';
            if ($idPerModule) {
                $this->addSql("alter table $tableName alter column id set default nextval('$seqName')");
            } else {
                $this->addSql("alter table $tableName alter column id set default nextval('global_id_seq')");
            }
        }
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs

    }
}
