<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210504205718 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        // $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    public function getDescription() : string
    {
        return '';
    }

    private $playbooks = '
    [
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Tasks > Post-Update: Assign user owner",
            "aliasName": null,
            "tag": "#system",
            "recordTags": [
              "/api/3/tags/system"
            ],
            "description": "Sets the user owner of an updated task to the assigned user",
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1620159419,
            "collection": "/api/3/workflow_collections/6368061c-c6ea-462b-b33f-75437efb0e3b",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/d0cfa693-132c-4055-a5df-b234f909bb9d",
            "steps": [
              {
                "@type": "WorkflowStep",
                "name": "Add assignedTo as userOwner",
                "description": null,
                "arguments": {
                  "resource": {
                    "__link": {
                      "userOwners": "{{vars.input.records[0].assignedToPerson[\'@id\']}}"
                    }
                  },
                  "_showJson": false,
                  "operation": "Append",
                  "collection": "{{vars.input.records[0][\'@id\']}}",
                  "__recommend": [],
                  "collectionType": "/api/3/tasks",
                  "fieldOperation": {
                    "recordTags": "Append"
                  },
                  "step_variables": []
                },
                "status": null,
                "top": "30",
                "left": "450",
                "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                "uuid": "2a3f0cbc-884f-4588-904b-bdfc4c628652",
                "id": 504
              },
              {
                "@type": "WorkflowStep",
                "name": "Start",
                "description": null,
                "arguments": {
                  "resource": "tasks",
                  "resources": [
                    "tasks"
                  ],
                  "step_variables": {
                    "input": {
                      "params": [],
                      "records": [
                        "{{vars.input.records[0]}}"
                      ]
                    }
                  },
                  "fieldbasedtrigger": {
                    "sort": [],
                    "limit": 30,
                    "logic": "AND",
                    "filters": [
                      {
                        "type": "object",
                        "field": "assignedToPerson",
                        "value": "",
                        "_value": {
                          "@id": "",
                          "display": "",
                          "itemValue": ""
                        },
                        "operator": "changed"
                      },
                      {
                        "type": "object",
                        "field": "assignedToPerson",
                        "value": "false",
                        "_value": {
                          "@id": "false",
                          "display": "",
                          "itemValue": ""
                        },
                        "operator": "isnull"
                      }
                    ]
                  }
                },
                "status": null,
                "top": "30",
                "left": "125",
                "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
                "uuid": "d0cfa693-132c-4055-a5df-b234f909bb9d",
                "id": 505
              }
            ],
            "routes": [
              {
                "@type": "WorkflowRoute",
                "name": "Start -> Add assignedTo as userOwner",
                "targetStep": "/api/3/workflow_steps/2a3f0cbc-884f-4588-904b-bdfc4c628652",
                "sourceStep": "/api/3/workflow_steps/d0cfa693-132c-4055-a5df-b234f909bb9d",
                "label": null,
                "isExecuted": false,
                "uuid": "2deca04c-cdcc-48e4-9d1d-356602f12303"
              }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "53e64ff1-62d6-467e-8711-c40cb960402e",
            "id": 115,
            "createDate": 1620159303,
            "modifyDate": 1620159419,
            "owners": [],
            "isPrivate": false
          },
          {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Tasks > Post-Create: Assign user owner",
            "aliasName": null,
            "tag": "#system",
            "recordTags": [
              "/api/3/tags/system"
            ],
            "description": "Sets the user owner of a new task to the assigned user",
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "lastModifyDate": 1620159277,
            "collection": "/api/3/workflow_collections/6368061c-c6ea-462b-b33f-75437efb0e3b",
            "versions": [],
            "triggerStep": "/api/3/workflow_steps/38cde9d4-e2ce-4b61-857a-fcf61e1aa798",
            "steps": [
              {
                "@type": "WorkflowStep",
                "name": "Start",
                "description": null,
                "arguments": {
                  "resource": "tasks",
                  "resources": [
                    "tasks"
                  ],
                  "step_variables": {
                    "input": {
                      "params": [],
                      "records": [
                        "{{vars.input.records[0]}}"
                      ]
                    }
                  },
                  "fieldbasedtrigger": {
                    "sort": [],
                    "limit": 30,
                    "logic": "AND",
                    "filters": [
                      {
                        "type": "object",
                        "field": "assignedToPerson",
                        "value": "false",
                        "_value": {
                          "@id": "false",
                          "display": "",
                          "itemValue": ""
                        },
                        "operator": "isnull"
                      }
                    ]
                  }
                },
                "status": null,
                "top": "30",
                "left": "125",
                "stepType": "/api/3/workflow_step_types/ea155646-3821-4542-9702-b246da430a8d",
                "uuid": "38cde9d4-e2ce-4b61-857a-fcf61e1aa798",
                "id": 501
              },
              {
                "@type": "WorkflowStep",
                "name": "Add assignedTo as userOwner",
                "description": null,
                "arguments": {
                  "resource": {
                    "__link": {
                      "userOwners": "{{vars.input.records[0].assignedToPerson[\'@id\']}}"
                    }
                  },
                  "_showJson": false,
                  "operation": "Append",
                  "collection": "{{vars.input.records[0][\'@id\']}}",
                  "__recommend": [],
                  "collectionType": "/api/3/tasks",
                  "fieldOperation": {
                    "recordTags": "Append"
                  },
                  "step_variables": []
                },
                "status": null,
                "top": "30",
                "left": "450",
                "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
                "uuid": "5a5fe2df-e1c1-446f-b5e9-3c2aa619aa96",
                "id": 502
              }
            ],
            "routes": [
              {
                "@type": "WorkflowRoute",
                "name": "Start -> Add assignedTo as userOwner",
                "targetStep": "/api/3/workflow_steps/5a5fe2df-e1c1-446f-b5e9-3c2aa619aa96",
                "sourceStep": "/api/3/workflow_steps/38cde9d4-e2ce-4b61-857a-fcf61e1aa798",
                "label": null,
                "isExecuted": false,
                "uuid": "10d1f642-d522-432a-b25c-32518e4451ac"
              }
            ],
            "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
            "uuid": "ba6d3691-8cf1-4a08-9141-065a437fce80",
            "id": 114,
            "createDate": 1620159198,
            "modifyDate": 1620159279,
            "owners": [],
            "isPrivate": false
        }
    ]';

    public function up(Schema $schema): void
    {
        $this->addSql("UPDATE staging_model_metadatas SET userownable = true WHERE type = 'tasks';");
        $this->addSql("UPDATE staging_model_metadatas SET userownable = true WHERE type = 'attachments';");
        $this->addSql("UPDATE attribute_metadatas SET system = true WHERE name = 'uuid';");
        $playbook = json_decode($this->playbooks, true);
        foreach ($playbook as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    public function down(Schema $schema): void
    {
        $this->addSql("UPDATE staging_model_metadatas SET userownable = false WHERE type = 'tasks';");
        $this->addSql("UPDATE staging_model_metadatas SET userownable = false WHERE type = 'attachments';");
        $playbooks = json_decode($this->playbooks, true);
        foreach ($playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
