<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Ramsey\Uuid\Uuid;
use App\Entity\Core\Widget;
use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210707170607 extends AbstractMigration implements ContainerAwareInterface
{
    const WIDGET_PATH = '/opt/cyops-ui/widgets/installed/';
    const WIDGET_OLD_PATH = '/opt/cyops-ui/widgets/';
    const WIDGET_DEV_PATH = '/opt/cyops-ui/widgets/development/';
    const INGNORE_FOLDER_LIST = [".DS_Store", ".blank", ".", "..", "README.txt", "development", "installed", "templates", "generic"];
    protected $utilityService;

    public function getDescription(): string
    {
        return '';
    }

    /** @var ContainerInterface */
    private $container;

    /** @var ManagerRegistry $objectManager */
    private $widgetsManager;

    private $widgetsFileSystem;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
        $managerRegistry = $this->container->get('doctrine');
        $this->widgetsManager = $managerRegistry->getManagerForClass(Widget::class);
        $this->widgetsFileSystem = new Filesystem();
    }

    public function up(Schema $schema): void
    {
        // this up() migration is auto-generated, please modify it to your need
        if (!is_dir(self::WIDGET_PATH)) {
            mkdir(self::WIDGET_PATH);
        }

        if (!is_dir(self::WIDGET_DEV_PATH)) {
            mkdir(self::WIDGET_DEV_PATH);
        }

        $localPublishedWidgetList = [];
        $client = new \GuzzleHttp\Client();
        $widgetJsonRepoLink = sprintf('https://' . getenv("product_yum_server") . '/fsr-widgets/widgets.json');
        $widgetJsonObject = $client->request('GET', $widgetJsonRepoLink, ['verify' => false]);
        $widgetJson = json_decode($widgetJsonObject->getBody()->getContents(), true);
        $repoWidgets = array_keys($widgetJson);
        $repoWidgetNames = [];

        foreach ($repoWidgets as $repoWidget) {
            array_push($repoWidgetNames, explode('-', $repoWidget)[0]);
        }

        //Repo Widgets
        $widgetList = $this->connection->fetchAllAssociative("SELECT name, version, uuid FROM widgets WHERE installed=true");
        $id = $this->connection->fetchAllAssociative("select id from widgets WHERE id = (select max(id) from widgets)")[0]['id'];
        foreach ($widgetList as $widget) {
            if (in_array($widget['name'], $repoWidgetNames)) {
                $widgetOldFolderPath = self::WIDGET_OLD_PATH . $widget['name'];
                $id++;
                try {
                    $this->installRepoWidget($widget['name'], '2.0.0', $id);
                } catch (\Exception $e) {
                } finally {
                    $uuid = "'" . $widget['uuid'] . "'";
                    $this->widgetsFileSystem->remove($widgetOldFolderPath);
                    $this->addSql("delete from widgets where uuid = $uuid");
                }
            } else {
                array_push($localPublishedWidgetList, $widget);
            }
        }

        // Local Published Widgets
        try {
            $this->dumpLocalWidgets($localPublishedWidgetList, false);
        } catch (\Exception $e) {
        }

        // Local Unpublished Widgets
        try {
            $localUnpublishedWidgetList = $this->connection->fetchAllAssociative("SELECT uuid FROM widgets WHERE draft=true AND installed =false");
            $this->dumpLocalWidgets($localUnpublishedWidgetList, true);
        } catch (\Exception $e) {
        }
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs

    }

    protected function installRepoWidget($widgetName, $widgetVersion, $id)
    {
        $client = new \GuzzleHttp\Client();
        $widgetRepoName = $widgetName . '-' . $widgetVersion;
        $widgetTgzRepoLink = sprintf('https://' . getenv("product_yum_server") . '/fsr-widgets/%s/%s.tgz', $widgetRepoName, $widgetRepoName);
        $responseObject = $client->request('GET', $widgetTgzRepoLink, ['verify' => false]);
        $tgzFileData = $responseObject->getBody()->getContents();

        $widgetPath = self::WIDGET_PATH . $widgetRepoName;
        $tgzFileName = sprintf('%s.tgz', $widgetRepoName);
        try {
            $fileSystem = new Filesystem();
            $widgetTgzPath = sprintf('%s.tgz', $widgetPath);
            $fileSystem->dumpFile($widgetTgzPath, $tgzFileData);
            $phar = new \PharData($widgetTgzPath);
            $phar->decompress();
            $phar->extractTo(self::WIDGET_PATH);

            // Save the widget in DB
            $widgetInfoContent = json_decode(file_get_contents(self::WIDGET_PATH . $widgetRepoName . '/info.json'));
            $widgetInfoContent->draft = false;
            $widgetInfoContent->installed = true;
            $i = "'" . $id . "'";
            $uuid = "'" . Uuid::uuid4()->toString() . "'";
            $name = "'" . $widgetName . "'";
            $version = "'" . $widgetVersion . "'";
            $title = "'" . $widgetInfoContent->title . "'";
            $subTitle = "'" . $widgetInfoContent->subTitle . "'";
            $metadata = "'" . json_encode($widgetInfoContent->metadata, JSON_HEX_APOS) . "'";
            $this->addSql("insert into widgets (id, uuid, name, version, title, subTitle, metadata, draft, installed, enablePublish) 
                values ($i, $uuid, $name, $version, $title, $subTitle, $metadata, false, true, false)");
        } catch (\Exception $e) {
            // Code to clear any files that got unzipped
            $this->removeTarFiles(self::WIDGET_PATH, pathinfo($tgzFileName, PATHINFO_FILENAME));
        } finally {
            $this->removeTarFiles(self::WIDGET_PATH, pathinfo($tgzFileName, PATHINFO_FILENAME));
        }
    }


    private function removeTarFiles($path, $name)
    {
        $fileSystem = new Filesystem();
        $filePaths = [sprintf('%s/%s.tar.gz', $path, $name), sprintf('%s/%s.tgz', $path, $name), sprintf('%s/%s.tar', $path, $name)];
        foreach ($filePaths as $key => $value) {
            if ($fileSystem->exists($value)) {
                $fileSystem->remove($value);
            }
        }
    }

    private function dumpLocalWidgets($widgetList, $development = false)
    {
        if (count($widgetList)) {
            foreach ($widgetList as $key => $value) {
                $widgetsToUpdate = $this->widgetsManager->find(Widget::class, $value['uuid']);
                if ($widgetsToUpdate) {
                    $widgetOldFolderPath = self::WIDGET_OLD_PATH . $widgetsToUpdate->getName();
                    if (!$this->widgetsFileSystem->exists($widgetOldFolderPath)) {
                        $this->widgetsFileSystem->mkdir($widgetOldFolderPath, 0755);
                        $this->widgetsFileSystem->mkdir($widgetOldFolderPath . "/html", 0755);
                        $this->widgetsFileSystem->mkdir($widgetOldFolderPath . "/js", 0755);
                    }

                    $widgetsPath = sprintf("/opt/cyops-ui/widgets/%s", $widgetsToUpdate->getName());
                    $this->widgetsFileSystem->dumpFile(sprintf('%s/html/view.html', $widgetsPath), $widgetsToUpdate->getViewHtml());
                    $this->widgetsFileSystem->dumpFile(sprintf('%s/html/edit.html', $widgetsPath), $widgetsToUpdate->getEditHtml());
                    $this->widgetsFileSystem->dumpFile(sprintf('%s/js/view.controller.js', $widgetsPath), $widgetsToUpdate->getViewController());
                    $this->widgetsFileSystem->dumpFile(sprintf('%s/js/edit.controller.js', $widgetsPath), $widgetsToUpdate->getEditController());

                    $infoData['metadata'] = $widgetsToUpdate->getMetadata();
                    if (empty($infoData['metadata']['compatibility'])) {
                        $infoData['metadata']['compatibility'] = [];
                        array_push($infoData['metadata']['compatibility'], '7.0.2');
                    } else {
                        if (!in_array('7.0.2', $infoData['metadata']['compatibility'], true)) {
                            array_push($infoData['metadata']['compatibility'], '7.0.2');
                        }
                    }

                    $infoData['name'] = $widgetsToUpdate->getName();
                    $infoData['title'] = $widgetsToUpdate->getTitle();
                    $infoData['subTitle'] = $widgetsToUpdate->getSubTitle();
                    $infoData['version'] = $widgetsToUpdate->getVersion();

                    if ($widgetsToUpdate->getPublishedDate()) {
                        $infoData['published_date'] = $widgetsToUpdate->getPublishedDate();
                    }
                    $this->widgetsFileSystem->dumpFile(sprintf('%s/info.json', $widgetsPath), json_encode($infoData, JSON_PRETTY_PRINT));
                    $infoContent = json_decode(file_get_contents($widgetOldFolderPath . '/info.json'), true);
                    $widgetNewFolderPath = ($development ? self::WIDGET_DEV_PATH : self::WIDGET_PATH) . $widgetsToUpdate->getName() . '-' . $infoContent['version'];

                    if (!is_dir($widgetNewFolderPath)) {
                        mkdir($widgetNewFolderPath);
                    }
                    $this->copyWidgetFiles($widgetOldFolderPath, $widgetNewFolderPath, $infoContent['name'], str_replace('.', '', $infoContent['version']), $development);
                    $metadata = "'" . json_encode($infoContent['metadata'], JSON_HEX_APOS) . "'";
                    $uuid = "'" .  $value['uuid'] . "'";
                    if ($development) {
                        $infoContent['enablePublish'] = true;
                        $this->addSql("update widgets set metadata = $metadata, enablePublish = true where uuid = $uuid");
                    } else {

                        $this->addSql("update widgets set metadata = $metadata where uuid = $uuid");
                    }
                }

                // remove old folder path
                $this->widgetsFileSystem->remove($widgetOldFolderPath);
            }
        }
    }

    private function copyWidgetFiles($src, $dst, $widgetName, $widgetVersion, $addDevString)
    {

        $dirName = pathinfo($src)['dirname'];
        if (!is_dir($dst) && $dirName != "." && $dirName != ".." && $dirName != 'js' && $dirName != 'html') {
            mkdir($dst);
        }
        if (is_dir($src)) {
            $dir = scandir($src);
            foreach ($dir as $file) {
                if ($file != "." && $file != "..") {
                    if (is_dir($src . "/" . $file)) {
                        if ($file == 'js' && count(explode('/', $src)) == 5) {
                            $editControllerMatchString = 'edit' . ucfirst($widgetName) . 'Ctrl';
                            $editControllerReplaceString = 'edit' . ucfirst($widgetName) . $widgetVersion . ($addDevString ? 'DevCtrl' : 'Ctrl');
                            $editControllerFileSrcPath = $src . '/js/edit.controller.js';
                            $editControllerFileDstPath = $dst . '/edit.controller.js';

                            $viewControllerMatchString = $widgetName . 'Ctrl';
                            $viewControllerReplaceString = $widgetName . $widgetVersion . ($addDevString ? 'DevCtrl' : 'Ctrl');
                            $viewControllerFileSrcPath = $src . '/js/view.controller.js';
                            $viewControllerFileDstPath = $dst . '/view.controller.js';
                            $widgetBase = "widgets/" . $widgetName;
                            $widgetAssetBase = "widgets/installed/" . $widgetName . "-" . $widgetVersion;


                            file_put_contents($editControllerFileDstPath, str_replace($editControllerMatchString, $editControllerReplaceString, file_get_contents($editControllerFileSrcPath)));
                            file_put_contents($viewControllerFileDstPath, str_replace($viewControllerMatchString, $viewControllerReplaceString, file_get_contents($viewControllerFileSrcPath)));
                            file_put_contents($editControllerFileDstPath, str_replace($widgetBase, $widgetAssetBase, file_get_contents($editControllerFileDstPath)));
                            file_put_contents($viewControllerFileDstPath, str_replace($widgetBase, $widgetAssetBase, file_get_contents($viewControllerFileDstPath)));
                        } else if ($file == 'html' && count(explode('/', $src)) == 5) {
                            $matchString = 'widgets/' . $widgetName;
                            $replaceString = $widgetName . '-' . implode('.', str_split($widgetVersion));
                            $htmlSrcPath = $src . '/html/view.html';
                            $htmlDstPath = $dst . '/view.html';

                            $editHtmlSrcPath = $src . '/html/edit.html';
                            $editHtmlDstPath = $dst . '/edit.html';

                            file_put_contents($htmlDstPath, str_replace($matchString, $replaceString, file_get_contents($htmlSrcPath)));
                            file_put_contents($editHtmlDstPath, str_replace($matchString, $replaceString, file_get_contents($editHtmlSrcPath)));
                        } else {
                            $this->copyWidgetFiles($src . "/" . $file, $dst . "/" . $file, $widgetName, $widgetVersion, $addDevString);
                        }
                    } else {
                        copy($src . "/" . $file, $dst . "/" . $file);
                    }
                }
            }
        } else {
            copy($src, $dst);
        }

        return true;
    }
}
