<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20210719081245 extends AbstractMigration implements ContainerAwareInterface
{
  /** @var ContainerInterface */
  private $container;

  public function setContainer(ContainerInterface $container = null)
  {
    $this->container = $container;
  }

  private function deleteWorkflow($workflow)
  {
    /** @var ResourceInterface $workflowResource */
    $workflowResource = Workflow::class;
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
    $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
    if ($workflowToDelete == null) {
      $workflowManager->flush();
      return;
    }
    $workflowManager->remove($workflowToDelete);
    $workflowManager->flush();
  }


  private function addWorkflow($workflow)
  {
    $workflowResource = Workflow::class;
    // $context = $workflowResource->getDenormalizationContext();
    $context["ignore_field_authorization"] = true;
    $context["ignore_authorization"] = true;
    $context["groups"] = ["fsr_all", "password"];
    $context["fetch_data"] = false;
    $workflowObject = $this->container->get('serializer')->deserialize(
      json_encode($workflow),
      $workflowResource,
      'jsonld',
      $context
    );
    /** @var ManagerRegistry $objectManager */
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
    $workflowManager->persist($workflowObject);
    $workflowManager->flush();
  }



  public function getDescription(): string
  {
    return 'Comment Workflows';
  }

  private $playbooks = '
  [
    {
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Reclaim Disk Space (Playbook Logs)",
        "aliasName": null,
        "tag": "#system",
        "description": "Workflow for Scheduling Reclaim Disk Space of Playbook Execution Logs",
        "isActive": true,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "lastModifyDate": 1625766785,
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/747556b8-3042-432f-990b-ea1b791f7788",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": []
            },
            "status": null,
            "top": "20",
            "left": "20",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "747556b8-3042-432f-990b-ea1b791f7788",
            "id": 4764
          },
          {
            "@type": "WorkflowStep",
            "name": "run vacuum",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/wf/api/workflows/auto_vacuum/?format=json",
                "body": "",
                "method": "POST"
              },
              "version": "3.1.1",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "top": "140",
            "left": "20",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "7fe6a5d3-e996-4d2f-aa8c-fea9e3c8a6f4",
            "id": 4765
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "name": "Start -> run vacuum",
            "targetStep": "/api/3/workflow_steps/7fe6a5d3-e996-4d2f-aa8c-fea9e3c8a6f4",
            "sourceStep": "/api/3/workflow_steps/747556b8-3042-432f-990b-ea1b791f7788",
            "label": null,
            "isExecuted": false,
            "uuid": "d2805d29-43e0-4a23-b277-2b931cf8d1be"
          }
        ],
        "priority": null,
        "uuid": "095624fb-23ab-4a4c-b4f2-918d9d0c528f",
        "recordTags": [
          "system"
        ],
        "id": 1102,
        "createDate": 1625766593,
        "modifyDate": 1625766784,
        "owners": [],
        "isPrivate": false
      }
  ]';

  public function up(Schema $schema): void
  {
    $playbooks = json_decode($this->playbooks, true);
    foreach ($playbooks as $playbook) {
      $this->deleteWorkflow($playbook);
    }
    foreach ($playbooks as $playbook) {
      $this->addWorkflow($playbook);
    }
  }

  public function down(Schema $schema): void
  {
    $playbooks = json_decode($this->playbooks, true);
    foreach ($playbooks as $playbook) {
      $this->deleteWorkflow($playbook);
    }
  }
}
