<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20211007045212 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;

    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $commandToExecute = [
          sprintf("delete from workflows where uuid='%s'", $workflow['uuid']),
          "delete from workflow_steps where uuid not in  (select workflowstep_uuid from workflow_workflowstep );"
        ];
        foreach ($commandToExecute as $command) {
            $workflowManager->getConnection()->prepare($command)->execute();
        }
    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }



    public function getDescription(): string
    {
        return 'Bin Clean Up Playbook';
    }

    private $playbooks = '
  [
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "Recycle Bin Cleanup",
      "aliasName": null,
      "tag": "#system",
      "description": "Scheduled workflow for Recycle Bin Cleanup",
      "isActive": true,
      "debug": true,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [
        "moduleList"
      ],
      "synchronous": false,
      "lastModifyDate": 1636443913,
      "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
      "versions": [],
      "triggerStep": "/api/3/workflow_steps/b5e7067e-85d1-4dd8-9a44-65ab3206336a",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Exit the playbook",
          "description": null,
          "arguments": {
            "params": [],
            "version": "3.1.2",
            "connector": "cyops_utilities",
            "operation": "no_op",
            "operationTitle": "Utils: No Operation",
            "step_variables": []
          },
          "status": null,
          "top": "435",
          "left": "125",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "00c04532-2d97-446f-b930-6320410957ee",
          "id": 97
        },
        {
          "@type": "WorkflowStep",
          "name": "Get Soft Deletable Modules",
          "description": null,
          "arguments": {
            "params": {
              "iri": "/api/3/model_metadatas?$limit=500&softDeleteable=true&__selectFields=type",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.2",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": {
              "modulesList": "{{vars.result.data[\"hydra:member\"]}}"
            }
          },
          "status": null,
          "top": "435",
          "left": "475",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "15c23e00-3604-4952-aa3c-ac274bfe0d53",
          "id": 100
        },
        {
          "@type": "WorkflowStep",
          "name": "Appending Workflow and Workflow Collections to list of Module",
          "description": null,
          "arguments": {
            "temp": "{{vars.modulesList.extend([{\"type\": \"workflows\"}, {\"type\": \"workflow_collections\"}]) }}"
          },
          "status": null,
          "top": "570",
          "left": "475",
          "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
          "uuid": "43afe063-a196-4c36-9e28-515ac138480c",
          "id": 2570
        },
        {
          "@type": "WorkflowStep",
          "name": "Fetch Bin Purge Settings",
          "description": null,
          "arguments": {
            "params": {
              "iri": "/api/3/system_settings",
              "body": "",
              "method": "GET"
            },
            "version": "3.1.2",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": {
              "bin_ttl_config": "{{vars.result.data[\"hydra:member\"][0][\"publicValues\"][\"bin_ttl_config\"] | default({\"enabled\": false })}}"
            }
          },
          "status": null,
          "top": "165",
          "left": "300",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "55394a0d-a970-4b9a-a88b-0e43201db149",
          "id": 98
        },
        {
          "@type": "WorkflowStep",
          "name": "Delete Records",
          "description": null,
          "arguments": {
            "params": {
              "iri": "/api/3/delete-with-query/{{vars.item.type}}?$showDeleted=true",
              "body": "{\"logic\":\"AND\",\"filters\":[{\"field\":\"deletedAt\",\"operator\":\"isnull\",\"value\":\"false\",\"type\":\"object\"},{\"field\":\"modifyDate\",\"operator\":\"gte\",\"value\":\"{{ arrow.utcnow().shift(days=-vars.bin_ttl_config.ttl).int_timestamp}}\",\"type\":\"datetime\"}]}",
              "method": "DELETE"
            },
            "version": "3.1.2",
            "for_each": {
              "item": "{{vars.modulesList}}",
              "__bulk": false,
              "parallel": true,
              "condition": ""
            },
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": []
          },
          "status": null,
          "top": "700",
          "left": "480",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "5f90a1d9-9520-4ee6-b25b-2b8650a2a148",
          "id": 102
        },
        {
          "@type": "WorkflowStep",
          "name": "Check if purge enabled",
          "description": null,
          "arguments": {
            "conditions": [
              {
                "step_iri": "/api/3/workflow_steps/15c23e00-3604-4952-aa3c-ac274bfe0d53",
                "condition": "{{ vars.bin_ttl_config.enabled }}",
                "step_name": "Get Soft Deletable Modules"
              },
              {
                "default": true,
                "step_iri": "/api/3/workflow_steps/00c04532-2d97-446f-b930-6320410957ee",
                "step_name": "Terminate the playbook"
              }
            ]
          },
          "status": null,
          "top": "300",
          "left": "300",
          "stepType": "/api/3/workflow_step_types/12254cf5-5db7-4b1a-8cb1-3af081924b28",
          "uuid": "8c8a40cc-a11b-4ee1-9e8b-07e2124ba28a",
          "id": 99
        },
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "step_variables": {
              "input": {
                "params": []
              }
            }
          },
          "status": null,
          "top": "30",
          "left": "300",
          "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
          "uuid": "b5e7067e-85d1-4dd8-9a44-65ab3206336a",
          "id": 101
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "name": "Fetch Bin Purge Settings -> Check if purge enabled",
          "targetStep": "/api/3/workflow_steps/8c8a40cc-a11b-4ee1-9e8b-07e2124ba28a",
          "sourceStep": "/api/3/workflow_steps/55394a0d-a970-4b9a-a88b-0e43201db149",
          "label": null,
          "isExecuted": false,
          "uuid": "31feacf1-9c9f-4905-ac12-b3c252399d6e"
        },
        {
          "@type": "WorkflowRoute",
          "name": "Appending Workflow and Workflow Collections to list of Module -> Delete Records",
          "targetStep": "/api/3/workflow_steps/5f90a1d9-9520-4ee6-b25b-2b8650a2a148",
          "sourceStep": "/api/3/workflow_steps/43afe063-a196-4c36-9e28-515ac138480c",
          "label": null,
          "isExecuted": false,
          "uuid": "34a14ef4-dbe5-4cc5-a15d-c0c06867c8b1"
        },
        {
          "@type": "WorkflowRoute",
          "name": "Start -> Fetch Bin Purge Settings",
          "targetStep": "/api/3/workflow_steps/55394a0d-a970-4b9a-a88b-0e43201db149",
          "sourceStep": "/api/3/workflow_steps/b5e7067e-85d1-4dd8-9a44-65ab3206336a",
          "label": null,
          "isExecuted": false,
          "uuid": "87b57497-189f-4daf-82e0-2819e3c7ec47"
        },
        {
          "@type": "WorkflowRoute",
          "name": "Get Soft Deletable Modules -> Appending Workflow and Workflow Collections to list of Module",
          "targetStep": "/api/3/workflow_steps/43afe063-a196-4c36-9e28-515ac138480c",
          "sourceStep": "/api/3/workflow_steps/15c23e00-3604-4952-aa3c-ac274bfe0d53",
          "label": null,
          "isExecuted": false,
          "uuid": "a4c8e434-37d5-41f7-9a80-ba8e6dbfb3cf"
        },
        {
          "@type": "WorkflowRoute",
          "name": "Check if purge enabled -> Terminate the playbook",
          "targetStep": "/api/3/workflow_steps/00c04532-2d97-446f-b930-6320410957ee",
          "sourceStep": "/api/3/workflow_steps/8c8a40cc-a11b-4ee1-9e8b-07e2124ba28a",
          "label": null,
          "isExecuted": false,
          "uuid": "ae8c745e-88c6-4d85-8923-207e640289c9"
        },
        {
          "@type": "WorkflowRoute",
          "name": "Check if purge enabled -> Get Soft Deletable Modules",
          "targetStep": "/api/3/workflow_steps/15c23e00-3604-4952-aa3c-ac274bfe0d53",
          "sourceStep": "/api/3/workflow_steps/8c8a40cc-a11b-4ee1-9e8b-07e2124ba28a",
          "label": null,
          "isExecuted": false,
          "uuid": "e05ab056-798f-4899-97b0-05e3a73e0629"
        }
      ],
      "priority": null,
      "uuid": "0546c27c-15dc-4320-9063-00f40e44abd3",
      "recordTags": [
        "system"
      ],
      "id": 28,
      "createDate": 1636351927,
      "modifyDate": 1636443913,
      "owners": [],
      "isPrivate": false,
      "deletedAt": null
    }  
  ]';
 
    public function up(Schema $schema): void
    {
        $migrated_playbooks = json_decode($this->playbooks, true);
        foreach ($migrated_playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
        foreach ($migrated_playbooks as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    public function down(Schema $schema): void
    {
        $migrated_playbooks = json_decode($this->playbooks, true);
        foreach ($migrated_playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
