<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20211019162042 extends AbstractMigration implements ContainerAwareInterface
{

    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;

    }
    
    public function getDescription(): string
    {
        return 'Archival Schedule Playbooks';
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        // Soft Deleted
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
        // Permannently Deleted
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    public function up(Schema $schema): void
    {
        $migrated_playbooks = json_decode($this->playbooks, true);
        foreach ($migrated_playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
        foreach ($migrated_playbooks as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    public function down(Schema $schema): void
    {
        $migrated_playbooks = json_decode($this->playbooks, true);
        foreach ($migrated_playbooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }

  private $playbooks = '[
    {
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Archived Data",
        "aliasName": null,
        "tag": null,
        "description": "Triggers Data Archival for module marked for archived",
        "isActive": true,
        "debug": false,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "lastModifyDate": 1634660086,
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/61d1c4ca-172a-4cf7-a203-b0cca520c469",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": {
                "input": {
                  "params": []
                }
              }
            },
            "status": null,
            "top": "40",
            "left": "40",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "61d1c4ca-172a-4cf7-a203-b0cca520c469",
            "id": 208
          },
          {
            "@type": "WorkflowStep",
            "name": "Trigger Archival",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/archival/trigger/start",
                "body": "",
                "method": "POST"
              },
              "version": "3.1.2",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "top": "160",
            "left": "360",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "edacd7e5-7d3f-4e8c-b8bf-d0f508be9f7d",
            "id": 209
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "name": "Start -> Initiate Archive",
            "targetStep": "/api/3/workflow_steps/edacd7e5-7d3f-4e8c-b8bf-d0f508be9f7d",
            "sourceStep": "/api/3/workflow_steps/61d1c4ca-172a-4cf7-a203-b0cca520c469",
            "label": null,
            "isExecuted": false,
            "uuid": "ae6364f9-922a-4a7e-bb42-9ad93b2696ba"
          }
        ],
        "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
        "uuid": "907f8e2b-d071-49a8-a73d-daa3b3d60a17",
        "recordTags": [
          "system"
        ],
        "id": 57,
        "createDate": 1634650391,
        "modifyDate": 1634660087,
        "owners": [],
        "isPrivate": false,
        "deletedAt": null
    },
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "Purge Expired Feeds",
      "aliasName": null,
      "tag": null,
      "description": "Scheduled Workflow Purge expired feeds",
      "isActive": true,
      "debug": false,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [],
      "synchronous": false,
      "lastModifyDate": 1638450218,
      "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
      "versions": [],
      "triggerStep": "/api/3/workflow_steps/5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "step_variables": {
              "input": {
                "params": []
              }
            }
          },
          "status": null,
          "top": "30",
          "left": "125",
          "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
          "uuid": "5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd",
          "id": 209
        },
        {
          "@type": "WorkflowStep",
          "name": "Delete Expired Feeds",
          "description": null,
          "arguments": {
            "params": {
              "iri": "/api/3/delete-with-query/threat_intel_feeds?$showDeleted=true",
              "body": "{\"logic\":\"AND\",\"filters\":[{\"field\":\"lastSeen\",\"operator\":\"isnull\",\"value\":true},{\"field\":\"expiresOn\",\"operator\":\"lte\",\"value\":\"{{ arrow.utcnow().shift(days=-vars.bin_ttl_config.ttl).int_timestamp}}\",\"type\":\"datetime\"}]}",
              "method": "DELETE"
            },
            "version": "3.1.2",
            "connector": "cyops_utilities",
            "operation": "make_cyops_request",
            "operationTitle": "FSR: Make FortiSOAR API Call",
            "step_variables": []
          },
          "status": null,
          "top": "165",
          "left": "125",
          "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
          "uuid": "8a7159de-5438-4c9c-91b2-e02af3aaaacc",
          "id": 210
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "name": "Start -> Delete Expired Feeds",
          "targetStep": "/api/3/workflow_steps/8a7159de-5438-4c9c-91b2-e02af3aaaacc",
          "sourceStep": "/api/3/workflow_steps/5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd",
          "label": null,
          "isExecuted": false,
          "uuid": "3ca47a63-f0c4-43dc-ae32-8fe897c03b7f"
        }
      ],
      "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
      "uuid": "14e1e918-2472-40e0-9557-a31d2c0a57fc",
      "recordTags": [
        "system"
      ],
      "id": 57,
      "createDate": 1638448712,
      "modifyDate": 1638450218,
      "owners": [],
      "isPrivate": false,
      "deletedAt": null
    }
  ]';
}
