<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use PDOException;
use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20220711104057 extends AbstractMigration implements ContainerAwareInterface
{

    /** @var ContainerInterface */
    private $container;

    private $playbooks = '[
      {
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Purge Expired Feeds",
        "aliasName": null,
        "tag": null,
        "description": "Scheduled Workflow Purge expired feeds",
        "isActive": true,
        "debug": false,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "lastModifyDate": 1658125751,
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": {
                "input": {
                  "params": []
                }
              }
            },
            "status": null,
            "top": "30",
            "left": "125",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "group": null,
            "uuid": "5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd"
          },
          {
            "@type": "WorkflowStep",
            "name": "Delete Expired Feeds",
            "description": null,
            "arguments": {
              "when": "{{vars.module_exists == 1}}",
              "params": {
                "iri": "/api/3/delete-with-query/threat_intel_feeds?$showDeleted=true",
                "body": "{\"logic\":\"AND\",\"filters\":[{\"field\":\"expiresOn\",\"operator\":\"lte\",\"value\":\"{{ arrow.utcnow().int_timestamp}}\",\"type\":\"datetime\"}]}",
                "method": "DELETE"
              },
              "version": "3.2.1",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "top": "300",
            "left": "125",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "group": null,
            "uuid": "8a7159de-5438-4c9c-91b2-e02af3aaaacc"
          },
          {
            "@type": "WorkflowStep",
            "name": "Fetch Threat Intel Feed Modules details",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/3/modules?type=threat_intel_feeds",
                "body": "",
                "method": "GET"
              },
              "version": "3.2.1",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": {
                "module_exists": "{{vars.steps.Fetch_Threat_Intel_Feed_Modules_details.data[\"hydra:totalItems\"]}}"
              }
            },
            "status": null,
            "top": "165",
            "left": "125",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "group": null,
            "uuid": "644055e6-2b2e-4bcb-a5f6-db5fefb68fec"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "name": "Start -> Check if feeds module enabled",
            "targetStep": "/api/3/workflow_steps/644055e6-2b2e-4bcb-a5f6-db5fefb68fec",
            "sourceStep": "/api/3/workflow_steps/5300f6f3-9a49-4e2b-8e78-d1bfb2a848cd",
            "label": null,
            "isExecuted": false,
            "uuid": "d39824fd-f257-4577-b8c6-4fd67c19c01f"
          },
          {
            "@type": "WorkflowRoute",
            "name": "Fetch Threat Intel Feed Modules details -> Delete Expired Feeds",
            "targetStep": "/api/3/workflow_steps/8a7159de-5438-4c9c-91b2-e02af3aaaacc",
            "sourceStep": "/api/3/workflow_steps/644055e6-2b2e-4bcb-a5f6-db5fefb68fec",
            "label": null,
            "isExecuted": false,
            "uuid": "a773745d-f9db-4f62-b04a-0e2741a3f0e4"
          }
        ],
        "groups": [],
        "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
        "uuid": "14e1e918-2472-40e0-9557-a31d2c0a57fc",
        "id": 16,
        "createDate": 1657865652.170304,
        "modifyDate": 1658125748.151757,
        "owners": [],
        "isPrivate": false,
        "deletedAt": null,
        "importedBy": [],
        "recordTags": [
          "system"
        ]
      }
      ]';

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return '';
    }

    public function up(Schema $schema): void
    {
        $playbooksToAdd = json_decode($this->playbooks, true);
        foreach ($playbooksToAdd as $playbook) {
            $this->deleteWorkflow($playbook);
        }
        foreach ($playbooksToAdd as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $conn = $workflowManager->getConnection();
        try {
            $context["ignore_field_authorization"] = true;
            $context["ignore_authorization"] = true;
            $context["groups"] = ["fsr_all", "password"];
            $context["fetch_data"] = false;
            $workflowObject = $this->container->get('serializer')->deserialize(
                json_encode($workflow),
                $workflowResource,
                'jsonld',
                $context
            );
            $conn->beginTransaction();
            $workflowManager->persist($workflowObject);
            $conn->commit();
            $workflowManager->flush();
        } catch (\Exception | PDOException $e) {
            $conn->rollback();
            $workflowManager->clear();
            return true;
        }
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null || empty($workflowToDelete)) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs
        $lPlaybooks = json_decode($this->playbooks, true);
        foreach ($lPlaybooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
