<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20220928121921 extends AbstractMigration implements ContainerAwareInterface
{

    /** @var ContainerInterface */
    private $container;

    private $playbooks = '[
        {
            "@type": "Workflow",
            "triggerLimit": null,
            "name": "Playbook execution history cleanup",
            "description": "Workflow for Scheduling Playbook Execution History Cleanup",
            "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
            "tag": "#system",
            "recordTags": ["/api/3/tags/system"],
            "isActive": true,
            "singleRecordExecution": false,
            "remoteExecutableFlag": false,
            "parameters": [],
            "synchronous": false,
            "triggerStep": "/api/3/workflow_steps/00d37ec1-f828-46a3-8d8d-3bf7d0a8af02",
            "steps": [
              {
                "@type": "WorkflowStep",
                "name": "Start",
                "description": null,
                "arguments": {
                  "step_variables": []
                },
                "status": null,
                "left": "20",
                "top": "20",
                "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
                "uuid": "00d37ec1-f828-46a3-8d8d-3bf7d0a8af02"
              },
              {
                "@type": "WorkflowStep",
                "name": "cleanup",
                "description": null,
                "arguments": {
                  "params": {
                    "iri": "/api/wf/api/workflows/delete/?format=json",
                    "body": "{{ vars.ttl_config }}",
                    "method": "DELETE"
                  },
                  "version": "3.0.1",
                  "connector": "cyops_utilities",
                  "operation": "make_cyops_request",
                  "operationTitle": "FSR: Make FortiSOAR API Call",
                  "step_variables": []
                },
                "status": null,
                "left": "20",
                "top": "260",
                "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                "uuid": "77cbf88d-7e72-438e-a00e-9b7056d095c0"
              },
              {
                "@type": "WorkflowStep",
                "name": "Fetch PB Log TTL Settings",
                "description": null,
                "arguments": {
                  "params": {
                    "iri": "/api/3/system_settings",
                    "body": "",
                    "method": "GET"
                  },
                  "version": "3.0.1",
                  "connector": "cyops_utilities",
                  "operation": "make_cyops_request",
                  "operationTitle": "FSR: Make FortiSOAR API Call",
                  "step_variables": {
                    "ttl_config": "{{vars.result.data[\"hydra:member\"][0][\"publicValues\"][\"workflow_ttl_config\"]}}"
                  }
                },
                "status": null,
                "left": "20",
                "top": "140",
                "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                "uuid": "4408958d-fc31-4ce8-928b-afebd061a3d5"
              }
            ],
            "routes": [
              {
                "@type": "WorkflowRoute",
                "uuid": "41cee7b1-d6dc-4cd5-b2cd-6a823dd7890c",
                "name": "Start -> Fetch PB Log TTL Settings",
                "label": null,
                "isExecuted": false,
                "targetStep": "/api/3/workflow_steps/4408958d-fc31-4ce8-928b-afebd061a3d5",
                "sourceStep": "/api/3/workflow_steps/00d37ec1-f828-46a3-8d8d-3bf7d0a8af02"
              },
              {
                "@type": "WorkflowRoute",
                "uuid": "653875c4-d571-4ec7-b589-b11e34ad307a",
                "name": "Fetch PB Log TTL Settings -> cleanup",
                "label": null,
                "isExecuted": false,
                "targetStep": "/api/3/workflow_steps/77cbf88d-7e72-438e-a00e-9b7056d095c0",
                "sourceStep": "/api/3/workflow_steps/4408958d-fc31-4ce8-928b-afebd061a3d5"
              }
            ],
            "versions": [],
            "uuid": "26f1077a-c334-43bb-a10a-77b2593a121f"
          }
      ]';

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return '';
    }

    public function up(Schema $schema): void
    {
        $playbooksToAdd = json_decode($this->playbooks, true);
        foreach ($playbooksToAdd as $playbook) {
            $this->deleteWorkflow($playbook);
        }
        foreach ($playbooksToAdd as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $conn = $workflowManager->getConnection();
        try {
            $context["ignore_field_authorization"] = true;
            $context["ignore_authorization"] = true;
            $context["groups"] = ["fsr_all", "password"];
            $context["fetch_data"] = false;
            $workflowObject = $this->container->get('serializer')->deserialize(
                json_encode($workflow),
                $workflowResource,
                'jsonld',
                $context
            );
            $conn->beginTransaction();
            $workflowManager->persist($workflowObject);
            $conn->commit();
            $workflowManager->flush();
        } catch (\Exception $e) {
            $conn->rollback();
            $workflowManager->clear();
            return true;
        }
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowRepo = $workflowManager->getRepository($workflowResource);
        $workflowToDelete = $workflowRepo->findOneBy(['uuid' => $workflow['uuid']]);
        if ($workflowToDelete == null || empty($workflowToDelete)) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs
        $lPlaybooks = json_decode($this->playbooks, true);
        foreach ($lPlaybooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
