<?php

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use App\Entity\Workflow\Workflow;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20230313051645 extends AbstractMigration implements ContainerAwareInterface
{
    public function getDescription(): string
    {
        return 'Migration for adding schedule playbook for deleting old stats data';
    }

    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private $playbooks = '
    [{
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Purge Stats Data",
        "aliasName": null,
        "tag": null,
        "description": "Scheduled Workflow for Purging Old Stats Data",
        "isActive": true,
        "debug": true,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "lastModifyDate": 1678684021,
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/fe11aa19-2c9d-4afe-93f2-17ae05245253",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Purge Stats",
            "description": null,
            "arguments": {
              "name": "Utilities",
              "params": {
                "iri": "/api/3/delete-with-query/statistics?$showDeleted=true",
                "body": "{\"logic\":\"AND\",\"filters\":[{\"field\":\"createDate\",\"operator\":\"lte\",\"value\":\"{{arrow.utcnow().shift(days=-365).int_timestamp}}\",\"type\":\"datetime\"}]}",
                "method": "DELETE"
              },
              "version": "3.2.3",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "pickFromTenant": false,
              "step_variables": []
            },
            "status": null,
            "top": "200",
            "left": "240",
            "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
            "group": null,
            "uuid": "2a72c8ee-2db8-47a1-a2fb-2ffe31987c46"
          },
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": {
                "input": {
                  "params": []
                }
              }
            },
            "status": null,
            "top": "40",
            "left": "40",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "group": null,
            "uuid": "fe11aa19-2c9d-4afe-93f2-17ae05245253"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "name": "Start -> Purge Stats",
            "targetStep": "/api/3/workflow_steps/2a72c8ee-2db8-47a1-a2fb-2ffe31987c46",
            "sourceStep": "/api/3/workflow_steps/fe11aa19-2c9d-4afe-93f2-17ae05245253",
            "label": null,
            "isExecuted": false,
            "group": null,
            "uuid": "3319d675-112d-4ae8-b50a-9ac75debc481"
          }
        ],
        "groups": [],
        "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
        "uuid": "9a156927-4f24-41bb-982f-0ef3e663661b",
        "id": 449,
        "createDate": 1678682670.968938,
        "modifyDate": 1678684021.158219,
        "owners": [],
        "isPrivate": false,
        "deletedAt": null,
        "importedBy": [],
        "recordTags": [
          "system"
        ]
      }]
    ';

    public function up(Schema $schema): void
    {
        $playbooksToAdd = json_decode($this->playbooks, true);
        foreach ($playbooksToAdd as $playbook) {
            $this->deleteWorkflow($playbook);
        }
        foreach ($playbooksToAdd as $playbook) {
            $this->addWorkflow($playbook);
        }
    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $conn = $workflowManager->getConnection();
        try {
            $context["ignore_field_authorization"] = true;
            $context["ignore_authorization"] = true;
            $context["groups"] = ["fsr_all", "password"];
            $context["fetch_data"] = false;
            $workflowObject = $this->container->get('serializer')->deserialize(
                json_encode($workflow),
                $workflowResource,
                'jsonld',
                $context
            );
            $conn->beginTransaction();
            $workflowManager->persist($workflowObject);
            $conn->commit();
            $workflowManager->flush();
        } catch (\Exception $e) {
            $conn->rollback();
            $workflowManager->clear();
            return true;
        }
    }

    private function deleteWorkflow($workflow)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowRepo = $workflowManager->getRepository($workflowResource);
        $workflowToDelete = $workflowRepo->findOneBy(['uuid' => $workflow['uuid']]);
        if ($workflowToDelete == null || empty($workflowToDelete)) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs
        $lPlaybooks = json_decode($this->playbooks, true);
        foreach ($lPlaybooks as $playbook) {
            $this->deleteWorkflow($playbook);
        }
    }
}
