<?php

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use App\Entity\Workflow\Workflow;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20230830112236 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return 'Migration for playbook Manual Task > Terminate Playbook';
    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private function deleteWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflow['uuid']);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    private $playbook =
    '{
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Manual Task > Terminate Playbook",
        "aliasName": null,
        "tag": "#system",
        "recordTags": ["/api/3/tags/system"],
        "description": "Terminates the playbook executing instance if the task on which it was waiting is deleted.",
        "isActive": true,
        "debug": false,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "collection": "/api/3/workflow_collections/a9816e15-5e42-4785-a0ee-23fe8ddb19b6",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/5f5f0dea-2d10-4d92-9c74-607619d9ba68",
        "steps": [
            {
                "@type": "WorkflowStep",
                "name": "Get Playbook Status",
                "description": null,
                "arguments": {
                    "params": {
                        "iri": "/api/wf/api/workflows/{{vars.input.records[0].workflowid}}/?fields=status",
                        "body": "",
                        "method": "GET"
                    },
                    "version": "3.2.4",
                    "connector": "cyops_utilities",
                    "operation": "make_cyops_request",
                    "operationTitle": "FSR: Make FortiSOAR API Call",
                    "step_variables": {
                        "workflow_status": "{{vars.result.data.status}}"
                    }
                },
                "status": null,
                "top": "30",
                "left": "775",
                "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                "group": null,
                "uuid": "32de9090-66d4-4bca-852b-b2b91100837a"
            },
            {
                "@type": "WorkflowStep",
                "name": "Is Playbook Driven Task",
                "description": null,
                "arguments": {
                    "conditions": [
                        {
                            "option": "Yes",
                            "step_iri": "/api/3/workflow_steps/32de9090-66d4-4bca-852b-b2b91100837a",
                            "condition": "{{ (vars.input.records[0].workflowid != None) and (vars.input.records[0].workflowid| length > 0) }}",
                            "step_name": "Get Playbook Status"
                        },
                        {
                            "option": "No",
                            "default": true,
                            "step_iri": "/api/3/workflow_steps/6554cc5c-b060-4f42-99c9-1cf3a9c96265",
                            "step_name": "No Operation"
                        }
                    ],
                    "step_variables": []
                },
                "status": null,
                "top": "110",
                "left": "450",
                "stepType": "/api/3/workflow_step_types/12254cf5-5db7-4b1a-8cb1-3af081924b28",
                "group": null,
                "uuid": "d7ada216-ca81-4eae-94c2-0c925f8abe13"
            },
            {
                "@type": "WorkflowStep",
                "name": "No Operation",
                "description": null,
                "arguments": {
                    "params": [],
                    "version": "3.2.4",
                    "connector": "cyops_utilities",
                    "operation": "no_op",
                    "operationTitle": "Utils: No Operation",
                    "step_variables": []
                },
                "status": null,
                "top": "190",
                "left": "775",
                "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                "group": null,
                "uuid": "6554cc5c-b060-4f42-99c9-1cf3a9c96265"
            },
            {
                "@type": "WorkflowStep",
                "name": "Start",
                "description": null,
                "arguments": {
                    "resource": "tasks",
                    "resources": [
                        "tasks"
                    ],
                    "__triggerLimit": true,
                    "step_variables": {
                        "input": {
                            "params": [],
                            "records": [
                                "{{vars.input.records[0]}}"
                            ]
                        },
                        "record": "{{vars.input.records[0]}}"
                    },
                    "triggerOnSource": true,
                    "fieldbasedtrigger": {
                        "sort": [],
                        "limit": 30,
                        "logic": "AND",
                        "filters": []
                    },
                    "triggerOnReplicate": false
                },
                "status": null,
                "top": "110",
                "left": "125",
                "stepType": "/api/3/workflow_step_types/ef350fda-1771-477a-8f90-16f68cd7e5cb",
                "group": null,
                "uuid": "5f5f0dea-2d10-4d92-9c74-607619d9ba68"
            },
            {
                "@type": "WorkflowStep",
                "name": "Terminate required Playbooks",
                "description": null,
                "arguments": {
                    "when": "{{vars.record.workflowid and (vars.workflow_status == \"awaiting\")}}",
                    "params": {
                        "iri": "/api/wf/workflow/revoke/tasks/?format=json",
                        "body": "{\"wfLogId\":{{vars.input.records[0].workflowid}},\"user\":\"{{globalVars.Current_User}}\"}",
                        "method": "POST"
                    },
                    "version": "3.2.4",
                    "connector": "cyops_utilities",
                    "operation": "make_cyops_request",
                    "operationTitle": "FSR: Make FortiSOAR API Call",
                    "step_variables": []
                },
                "status": null,
                "top": "30",
                "left": "1100",
                "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
                "group": null,
                "uuid": "42b9735c-27ce-46c5-b144-02e0d97a4848"
            }
        ],
        "routes": [
            {
                "@type": "WorkflowRoute",
                "name": "Get Playbook Status -> Terminate required Playbooks",
                "targetStep": "/api/3/workflow_steps/42b9735c-27ce-46c5-b144-02e0d97a4848",
                "sourceStep": "/api/3/workflow_steps/32de9090-66d4-4bca-852b-b2b91100837a",
                "label": null,
                "isExecuted": false,
                "group": null,
                "uuid": "d01625e6-884e-4b24-b5d8-32d9131dfd74"
            },
            {
                "@type": "WorkflowRoute",
                "name": "Is Playbook Driven Task -> Get Playbook Status",
                "targetStep": "/api/3/workflow_steps/32de9090-66d4-4bca-852b-b2b91100837a",
                "sourceStep": "/api/3/workflow_steps/d7ada216-ca81-4eae-94c2-0c925f8abe13",
                "label": "Yes",
                "isExecuted": false,
                "group": null,
                "uuid": "89369129-2965-444d-a811-829096b98816"
            },
            {
                "@type": "WorkflowRoute",
                "name": "Is Playbook Driven Task -> No Operation",
                "targetStep": "/api/3/workflow_steps/6554cc5c-b060-4f42-99c9-1cf3a9c96265",
                "sourceStep": "/api/3/workflow_steps/d7ada216-ca81-4eae-94c2-0c925f8abe13",
                "label": "No",
                "isExecuted": false,
                "group": null,
                "uuid": "873a5cd1-f1d0-4293-b0c6-191a8fb403c1"
            },
            {
                "@type": "WorkflowRoute",
                "name": "Start -> Is Playbook Driven Task",
                "targetStep": "/api/3/workflow_steps/d7ada216-ca81-4eae-94c2-0c925f8abe13",
                "sourceStep": "/api/3/workflow_steps/5f5f0dea-2d10-4d92-9c74-607619d9ba68",
                "label": null,
                "isExecuted": false,
                "group": null,
                "uuid": "57576e9f-91d5-4a66-ac95-236ff46ea073"
            }
        ],
        "groups": [],
        "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
        "uuid": "2b8a5681-2065-4da7-95db-1e7aa99d2d15",
        "owners": [],
        "isPrivate": false
    }';

    public function up(Schema $schema): void
    {
        $playbook = json_decode($this->playbook, true);
        $this->addWorkflow($playbook);
    }

    public function down(Schema $schema): void
    {
        $playbook = json_decode($this->playbook, true);
        $this->deleteWorkflow($playbook);
    }
}
