<?php

declare(strict_types=1);

namespace App\Migrations;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use App\Entity\Workflow\Workflow;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20230905172941 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return 'Report generation workflow migration.';
    }

    public function up(Schema $schema): void
    {
        $playbook = json_decode($this->workflow, true);
        $this->deleteWorkflow($playbook['uuid']);
        $this->addWorkflow($playbook);
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs

    }

    private function addWorkflow($workflow)
    {
        $workflowResource = Workflow::class;
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $context["groups"] = ["fsr_all", "password"];
        $context["fetch_data"] = false;
        $workflowObject = $this->container->get('serializer')->deserialize(
            json_encode($workflow),
            $workflowResource,
            'jsonld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private $workflow = ' {
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Export Records As CSV",
        "aliasName": null,
        "tag": "#FSR-Utilities",
        "description": null,
        "isActive": true,
        "debug": true,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "lastModifyDate": 1654059911,
        "collection": "/api/3/workflow_collections/3af01eb9-0887-415b-b7fd-4d45a9d2760f",
        "versions": [],
        "triggerStep": "/api/3/workflow_steps/e0b2b02c-946c-4171-97d9-189621f664f6",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "route": "51b565c9-8a40-427a-989d-407f86d88646",
              "title": "Export Records CSV",
              "resources": [],
              "inputVariables": [
                {
                  "name": "query",
                  "type": "object",
                  "label": "Query",
                  "title": "JSON",
                  "usable": true,
                  "tooltip": "",
                  "dataType": "object",
                  "formType": "object",
                  "required": true,
                  "_expanded": true,
                  "mmdUpdate": true,
                  "collection": false,
                  "searchable": false,
                  "templateUrl": "app/components/form/fields/json.html",
                  "defaultValue": "{\n  \"sort\": [],\n  \"logic\": \"AND\",\n  \"filters\": []\n}",
                  "_previousName": "",
                  "lengthConstraint": false,
                  "allowedGridColumn": false,
                  "useRecordFieldDefault": false
                },
                {
                  "name": "moduleName",
                  "type": "string",
                  "label": "Module Name",
                  "tooltip": "Either Provide Module type or Module Name. If both are set preference will be given to the Module Type field",
                  "dataType": "text",
                  "formType": "text",
                  "required": false,
                  "_expanded": false,
                  "defaultValue": null,
                  "_previousName": "",
                  "useRecordFieldDefault": false
                },
                {
                  "name": "moduleType",
                  "type": "string",
                  "label": "Module Type",
                  "tooltip": "Either Provide Module type or Module Name. If both are set preference will be given to the Module Type field",
                  "dataType": "text",
                  "formType": "text",
                  "required": false,
                  "_expanded": false,
                  "defaultValue": "alerts",
                  "_previousName": "",
                  "useRecordFieldDefault": false
                },
                {
                  "name": "userId",
                  "type": "string",
                  "label": "userId",
                  "tooltip": "",
                  "dataType": "text",
                  "formType": "text",
                  "required": false,
                  "_expanded": false,
                  "defaultValue": null,
                  "_previousName": "",
                  "useRecordFieldDefault": false
                },
                {
                  "name": "exportFileName",
                  "type": "string",
                  "label": "exportFileName",
                  "tooltip": "",
                  "dataType": "text",
                  "formType": "text",
                  "required": false,
                  "_expanded": false,
                  "defaultValue": "",
                  "_previousName": "",
                  "useRecordFieldDefault": false
                },
                {
                  "name": "csvNeutralization",
                  "type": "boolean",
                  "label": "CsvNeutralization",
                  "title": "Checkbox",
                  "usable": true,
                  "tooltip": "",
                  "dataType": "checkbox",
                  "formType": "checkbox",
                  "required": false,
                  "_expanded": true,
                  "mmdUpdate": true,
                  "collection": false,
                  "searchable": true,
                  "templateUrl": "app/components/form/fields/checkbox.html",
                  "defaultValue": null,
                  "_previousName": "cNeutralization",
                  "playbookField": true,
                  "lengthConstraint": false,
                  "allowedEncryption": false,
                  "allowedGridColumn": true,
                  "jinjaExpressionView": true,
                  "useRecordFieldDefault": false
                }
              ],
              "step_variables": {
                "input": {
                  "params": {
                    "query": "{{vars.request.data[\"query\"]}}",
                    "userId": "{{vars.request.data[\"userId\"]}}",
                    "moduleName": "{{vars.request.data[\"moduleName\"]}}",
                    "moduleType": "{{vars.request.data[\"moduleType\"]}}",
                    "exportFileName": "{{vars.request.data[\"exportFileName\"]}}",
                    "csvNeutralization": "{{vars.request.data[\"csvNeutralization\"]}}"
                  },
                  "records": "{{vars.input.records}}"
                }
              },
              "_promptexpanded": true,
              "executeButtonText": "Execute",
              "noRecordExecution": true,
              "showToasterMessage": {
                "visible": false,
                "messageVisible": true
              },
              "triggerOnReplicate": false,
              "singleRecordExecution": false
            },
            "status": null,
            "top": "30",
            "left": "300",
            "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
            "uuid": "e0b2b02c-946c-4171-97d9-189621f664f6"
          },
          {
            "@type": "WorkflowStep",
            "name": "Export Records",
            "description": null,
            "arguments": {
              "name": "Report Engine",
              "config": "904ce341-5e48-4583-a1df-80738e017137",
              "params": {
                "query": "{{vars.input.params.query}}",
                "module": "{{vars.input.params.module}}",
                "userId": "{{vars.input.params.userId}}",
                "batchSize": "{{vars.input.params.batchSize}}",
                "moduleName": "{{vars.input.params.moduleName}}",
                "moduleType": "{{vars.input.params.moduleType}}",
                "exportFileName": "{{vars.input.params.exportFileName}}",
                "max_record_size": "{{vars.input.params.max_record_size}}",
                "csvNeutralization": "{{vars.input.params.csvNeutralization}}"
              },
              "version": "1.4.0",
              "connector": "cyops-schedule-report",
              "operation": "generate_csv_export",
              "operationTitle": "Generate CSV Export",
              "pickFromTenant": false,
              "step_variables": []
            },
            "status": null,
            "top": "126",
            "left": "655",
            "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
            "uuid": "76d224ad-ef92-4ec5-876e-366b2bfc5e26"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "name": "Start -> Export Records",
            "targetStep": "/api/3/workflow_steps/76d224ad-ef92-4ec5-876e-366b2bfc5e26",
            "sourceStep": "/api/3/workflow_steps/e0b2b02c-946c-4171-97d9-189621f664f6",
            "label": null,
            "isExecuted": false,
            "uuid": "fba957b3-97a5-4a87-9beb-d8ba8ca8167b"
          }
        ],
        "priority": "/api/3/picklists/2b563c61-ae2c-41c0-a85a-c9709585e3f2",
        "uuid": "0faecebd-d4ce-49b5-b90e-f4134e5414b5",
        "id": 481,
        "owners": [],
        "isPrivate": false,
        "deletedAt": null,
        "importedBy": [],
        "recordTags": []
      }';
    
    private function deleteWorkflow($workflowUuid)
    {
        $workflowResource = Workflow::class;
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource);
        $workflowToDelete = $workflowManager->find($workflowResource, $workflowUuid);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }
}