<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;

use ApiPlatform\Core\Bridge\Doctrine\Orm\ItemDataProvider;
use ApiPlatform\Core\DataProvider\Pagination;
use App\Doctrine\ORM\FilterQueryBuilder;
use App\Providers\AggregateDataProviderInterface;
use Doctrine\ORM\Mapping\ClassMetadataInfo;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;

class AggregateDataProvider extends ItemDataProvider implements AggregateDataProviderInterface
{

    public function __construct(private ManagerRegistry $managerRegistry, private FilterQueryBuilder $filterQueryBuilder, private Pagination $pagination)
    {
    }

    public function getAggregate(string $resource, Request $request, $query)
    {
        $entityClass = $resource; //->getEntityClass();

        /** @var EntityManager $manager */
        $manager = $this->managerRegistry->getManagerForClass($entityClass);
        $repository = $manager->getRepository($entityClass);

        $page = (int) $request->get($this->pagination->getPage(), 1);

        $itemsPerPage = $this->pagination->getLimit();
        if ($requestedItemsPerPage = $request->get('$limit')) {
            $itemsPerPage = (int) $requestedItemsPerPage;
        }

        $queryBuilder = $repository
            ->createQueryBuilder('o')
            ->setFirstResult(($page - 1) * $itemsPerPage)
            ->setMaxResults($itemsPerPage);

        if (!(isset($query['logic']) && isset($query['filters']))) {
            $query['logic'] = "AND";
            $query['filters'] = [];
        }
        $this->filterQueryBuilder->fromQueryObject($query, $queryBuilder, $entityClass);


        $classMetaData = $manager->getClassMetadata($entityClass);
        $identifiers = $classMetaData->getIdentifier();

        foreach ($classMetaData->getAssociationNames() as $i => $association) {
            $mapping = $classMetaData->associationMappings[$association];

            if (ClassMetadataInfo::FETCH_EAGER === $mapping['fetch']) {
                $queryBuilder->leftJoin('o.' . $association, 'a' . $i);
                $queryBuilder->addSelect('a' . $i);
            }
        }

        if (isset($this->order) && 1 === (is_countable($identifiers) ? count($identifiers) : 0)) {
            $identifier = $identifiers[0];
            $queryBuilder->addOrderBy('o.' . $identifier, $this->order);
        }

        try {
            return $queryBuilder->getQuery()->execute();
        } catch (\Throwable $th) {
            $errMsg = $th->getmessage();
            throw new HttpException(Response::HTTP_BAD_REQUEST, $errMsg);
        } catch (\Exception $e) {
            $errMsg = $e->getMessage();
            throw new HttpException(Response::HTTP_BAD_REQUEST, $errMsg);
        }
    }
}
