<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;


use stdClass;
use GuzzleHttp\Client;
use App\Constants\AppConstants;
use App\Encoder\CorrectSimpleJWS;
use App\Providers\SecretKeyProviderInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AuthenticationException;

class BasicFingerPrintProvider
{
    private $container;

    public function __construct(
        ContainerInterface $container
    ) {
        $this->container = $container;
    }

    /**
     * @param $theirFingerprint
     * @param SecretKeyProviderInterface $secretKeyProvider
     * @param Request $request
     * @param int $timeout Seconds
     * @return bool
     */
    public function validateFingerprint($theirFingerprint)
    {
        $decodedToken = base64_decode($theirFingerprint);
        [$username, $password] = explode(':', $decodedToken);
        $body = new stdClass();
        $body->{'credentials'} =  ['loginid' => $username, 'password' => $password, "token" => ""];

        $headers = array(
            "Content-type" => "application/json",
        );
        $method = 'POST';
        $uri = sprintf('%s/auth/authenticate', $this->container->getParameter(AppConstants::CRUDHUB_PARAMETER_KEY));
        $client = new Client();
        try {
            $guzzleRequest = new GuzzleRequest($method, $uri, $headers, json_encode($body));
            $response = $client->send($guzzleRequest, ['verify' => false]);
        } catch (BadRequestHttpException | \Exception $error) {
            throw new AuthenticationException('UserName and Password provided are invalid');
        }
        $jWTtoken = json_decode($response->getBody()->getContents(), true)['token'];
        $jws = CorrectSimpleJWS::load($jWTtoken);
        return $jws->getPayload();
    }
}
