<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

/**
 * Created by PhpStorm.
 * User: Santosh-Diwase
 * Date: 04/04/17
 * Time: 4:04 PM
 */

namespace App\Providers;

use App\Constants\CacheConstants;
use App\Service\UtilityService;
use Symfony\Component\Cache\Adapter\FilesystemTagAwareAdapter;
use Symfony\Component\DependencyInjection\ContainerInterface;

class JwtLogoutKeyProvider
{

    protected $cache;
    protected $container;
    private $utilityService;

    public function __construct(ContainerInterface $container, UtilityService $utilityService)
    {
        $this->container = $container;
        $this->utilityService = $utilityService;
        $cacehDir = sprintf('%s/var/security', $this->container->getParameter('kernel.project_dir'));
        $this->cache = new FilesystemTagAwareAdapter('', 86400, $cacehDir);
    }

    /**
     * @param string $keyIdentifier
     * @return boolean
     * @throws \Exception
     */
    public function isJwtLogoutKey($keyIdentifier)
    {
        $cacheKey = $this->cache->getItem($keyIdentifier);
        if ($cacheKey->isHit()) {
            return true;
        }
        return false;
    }

    /**
     * @param string $keyIdentifier
     * @throws \Exception
     */
    public function setJwtLogoutKey($keyIdentifier)
    {
        $cacheKey = $this->cache->getItem($keyIdentifier);
        if (!$cacheKey->isHit()) {
            $cacheKey->set($keyIdentifier);
            // Don't hardcode
            $cacheKey->expiresAfter(7200);
            $cacheKey->tag(CacheConstants::LOGOUT_TOKEN);
            $this->cache->save($cacheKey);
        }
    }

}
