<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;

use App\Entity\Actors\Actor;
use App\Constants\CacheConstants;
use App\Entity\Authorization\Role;
use App\Providers\PermissionsDictionaryBuilder;
use App\Service\AbstractUserDataProvider;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class PermissionsDictionaryProvider extends AbstractUserDataProvider
{
    public function __construct(
        private PermissionsDictionaryBuilder $permissionsDictionaryBuilder,
        protected TagAwareCacheInterface $cache,
        TokenStorageInterface $tokenStorage,
        private EntityManagerInterface $entityManager
    ) {
        parent::__construct($tokenStorage);
    }
    /**
     * @inheritdoc
     */
    public function getDataForActor(Actor $actor)
    {
        $username = $actor->getUuid();
        $item = $this->cache->getItem($username . "_permissions");
        if (!$item->isHit()) {
            /** @var Role[] $roles */
            $actorRepo = $this->entityManager->getRepository(Actor::class);
            $actor = $actorRepo->findOneBy(["uuid" => $actor->getUuid()]);
            $roles = $actor->getRoles(false, false);
            $permissionsDictionary = $this->permissionsDictionaryBuilder->buildPermissionsDictionary($roles);
            $item->set($permissionsDictionary);
            // Don't hardcode
            $item->expiresAfter(3600);
            $item->tag(CacheConstants::USER_PERMISSIONS);
            $this->cache->save($item);
            return $permissionsDictionary;
        }
        return $item->get();
    }
}
