<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;


//use App\Filter\QueryObjectFactory;
use App\Filter\QueryObjectFactory;
use Symfony\Component\HttpFoundation\RequestStack;

class QueryProvider
{
    
    /** @var array */
    protected $initialQuery;

    /** @var array */
    protected $additionalFilters;

    /** @var array */
    protected $query;

    /**
     * QueryProvider constructor.
     */
    public function __construct(protected RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    /**
     * @param array $initialQuery
     */
    public function setInitialQuery($initialQuery)
    {
        $this->initialQuery = $initialQuery;
    }

    /**
     * @param array $additionalFilters
     */
    public function setAdditionalFilters($additionalFilters)
    {
        $this->additionalFilters = $additionalFilters;
    }

    /**
     * @return array
     */
    public function getQueryObject()
    {
        $request = null;
        $queryString = null;

        if (isset($this->query)) {
            return $this->query;
        }

        if (isset($this->initialQuery)) {
            $this->query = $this->appendAdditionalFilters($this->initialQuery);
            return $this->query;
        }

        $request = $this->requestStack->getCurrentRequest();
        if (array_key_exists("__selectFields", $_GET)) {
            $GLOBALS['__selectFields__'] = $request->query->get("__selectFields");
            $request->query->remove("__selectFields");
            $request->overrideGlobals();
        } elseif (array_key_exists("__ignoreFields", $_GET)) {
            $GLOBALS['__ignoreFields__'] = $request->query->get("__ignoreFields");
            $request->query->remove("__ignoreFields");
            $request->overrideGlobals();
        }

        $queryString = $request->server->get('QUERY_STRING');
        $this->query = $this->appendAdditionalFilters(QueryObjectFactory::fromQueryString($queryString));
        return $this->query;
    }

    /**
     * @param array $baseQuery
     * @return array
     */
    protected function appendAdditionalFilters($baseQuery)
    {
        if (!$this->additionalFilters) {
            return $baseQuery;
        }

        $newBaseQuery = $baseQuery; // clones array
        $newBaseQuery['logic'] = 'AND';
        $newBaseQuery['filters'] = [
            $baseQuery,
            $this->additionalFilters
        ];

        return $newBaseQuery;
    }
}
