<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;


use App\Service\UtilityService;
use App\Encoder\CorrectSimpleJWS;
use App\Providers\JwtLogoutKeyProvider;
use App\Providers\SecretKeyProviderInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Lexik\Bundle\JWTAuthenticationBundle\Services\KeyLoader\KeyLoaderInterface;

class RestrictedFingerprintProvider
{

   
    public function __construct(private KeyLoaderInterface $keyLoader, private JwtLogoutKeyProvider $jwtLogoutKeyProvider, private UtilityService $utilityService)
    {}

    public const ALGORITHM = 'RS256';
    public const TYPE = 'restricted';

    /**
     *
     */
    public function getPublicKey()
    {
        while (!empty($errorStr = openssl_error_string())) {
        }
        $pkey = $this->keyLoader->loadKey("public");
        while (!empty($errorStr = openssl_error_string())) {
        }
        return $pkey;
    }

    private function validateAllowedUri($request, $allowedUrls)
    {
        $currentUri = $request->getPathInfo();
        $currentMethod = $request->getMethod();
        foreach ($allowedUrls as $allowedUri) {
            if ($currentUri == $allowedUri['iri'] && $currentMethod == $allowedUri['method']) {
                return true;
            }
        }
        throw new AuthenticationException('Given URI not part of allowed URI');
    }

    private function checkInvalidateUri($request, $invalidateUrl, $decodedToken)
    {
        $currentUri = $request->getPathInfo();
        $currentMethod = $request->getMethod();
        if ($currentUri === $invalidateUrl["iri"] && $currentMethod == $invalidateUrl["method"]) {
            $token = "Restricted" . $decodedToken['jti'];
            $this->jwtLogoutKeyProvider->setJwtLogoutKey($token);
        }
    }

    /**
     * @param string $jwtKey
     * @return bool
     */
    private function isLogout($jwtKey)
    {
        return $this->jwtLogoutKeyProvider->isJwtLogoutKey("Restricted" . $jwtKey);
    }


    /**
     * @param $theirFingerprint
     * @param SecretKeyProviderInterface $secretKeyProvider
     * @param int $timeout Seconds
     * @return bool
     */
    public function validateFingerprint($theirFingerprint, Request $request = null)
    {
        $jws = CorrectSimpleJWS::load($theirFingerprint);
        $decodedToken = $jws->getPayload();
        if (!$jws->isValid($this->getPublicKey(), self::ALGORITHM, self::TYPE)) {
            throw new AuthenticationException('Invalid Fingerprint');
        }
        $this->validateAllowedUri($request, $decodedToken['allowed_iris']);
        if ($this->isLogout($decodedToken['jti'])) {
            throw new AuthenticationException('Given token is no longer valid');
        }
        $this->checkInvalidateUri($request, $decodedToken['invalidate_iri'], $decodedToken);
        return $decodedToken;
    }
}
