<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Query;


use App\Constants\AttributeLevelOperators;

class FieldExpressionBuilder
{
    /**
     * @param string $fieldName
     */
    public function __construct(protected $fieldName)
    {
    }

    /**
     * @return FieldExpression
     */
    protected function buildFieldExpression(mixed $operator, mixed $value)
    {
        return new FieldExpression($this->fieldName, $operator, $value);
    }

    /**
     * @return FieldExpression
     */
    public function equals(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::EQUALS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function notEquals(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::NOT_EQUALS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function lessThan(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LESS_THAN, $value);
    }

    /**
     * @return FieldExpression
     */
    public function lessThanOrEqualTo(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LESS_THAN_EQUALS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function greaterThan(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::GREATER_THAN, $value);
    }

    /**
     * @return FieldExpression
     */
    public function greaterThanOrEqualTo(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::GREATER_THAN_EQUALS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function exists(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::EXISTS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function like(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LIKE, $value);
    }

    /**
     * @return FieldExpression
     */
    public function contains(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::CONTAINS, $value);
    }

    /**
     * @return FieldExpression
     */
    public function in(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::IN, $value);
    }

    /**
     * @return FieldExpression
     */
    public function notIn(mixed $value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::NIN, $value);
    }

    /**
     * @param boolean $value
     * @return FieldExpression
     */
    public function isNull($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::IS_NULL, $value);
    }

    /**
     * @param string $value
     * @return FieldExpression
     */
    public function search($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::SEARCH, $value);
    }
}
