<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Security;

use App\Entity\Actors\Actor;
use App\Service\UtilityService;
use App\Security\BaseAuthenticator;
use App\Providers\JwtLogoutKeyProvider;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use App\Encoder\Restricted\RestrictedUserToken;
use App\Entity\Custom\Person;
use App\Providers\RestrictedFingerprintProvider;
use Symfony\Component\Security\Http\Authenticator\Passport\Passport;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Http\Authenticator\Passport\Badge\UserBadge;
use Lexik\Bundle\JWTAuthenticationBundle\Services\KeyLoader\KeyLoaderInterface;
use Symfony\Component\Security\Http\Authenticator\Token\PostAuthenticationToken;
use Symfony\Component\Security\Http\Authenticator\Passport\SelfValidatingPassport;
use Symfony\Component\Security\Core\Exception\CustomUserMessageAuthenticationException;
use Lexik\Bundle\JWTAuthenticationBundle\TokenExtractor\AuthorizationHeaderTokenExtractor;

class RestrictedAuthenticator extends BaseAuthenticator
{

    protected $jwtLogoutKeyProvider;
    protected $keyLoader;
    protected $userRepository;
    protected $utilityService;

    public function __construct(
        KeyLoaderInterface $keyLoader,
        JwtLogoutKeyProvider $jwtLogoutKeyProvider,
        UtilityService $utilityService,
        EntityManagerInterface $entityManager
    ) {
        $this->keyLoader = $keyLoader;
        $this->jwtLogoutKeyProvider =  $jwtLogoutKeyProvider;
        $this->utilityService = $utilityService;
        $this->userRepository = $entityManager->getRepository(Actor::class);
    }

    /**
     * Called on every request to decide if this authenticator should be
     * used for the request. Returning `false` will cause this authenticator
     * to be skipped.
     */
    public function supports(Request $request): ?bool
    {
        return $request->headers->has('Authorization') && strpos($request->headers->get('Authorization'), 'Restricted') === 0;
    }

    public function authenticate(Request $request): Passport
    {
        $tokenExtractor = new AuthorizationHeaderTokenExtractor('Restricted', 'Authorization');

        $restrictedToken = $tokenExtractor->extract($request);
        if (null === $restrictedToken) {
            // The token header was empty, authentication fails with HTTP Status
            // Code 401 "Unauthorized"
            throw new CustomUserMessageAuthenticationException('No API token provided');
        }
        $restrictedProvider = new RestrictedFingerprintProvider($this->keyLoader, $this->jwtLogoutKeyProvider, $this->utilityService);
        $decodedToken = $restrictedProvider->validateFingerprint($restrictedToken, $request);
        $email = $decodedToken['email'];
        return new SelfValidatingPassport(
            new UserBadge($email, function ($userIdentifier) {
                $person = new Person();
                $person->setEmail($userIdentifier);
                $person->setFirstname($userIdentifier);
                $person->setLastname($userIdentifier);
                return $person;
            })
        );
    }

    /**
     * Shortcut to create a PostAuthenticationToken for you, if you don't really
     * care about which authenticated token you're using.
     */
    public function createToken(Passport $passport, string $firewallName): TokenInterface
    {
        return new RestrictedUserToken($passport->getUser(), $firewallName, []);
    }
}
