<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Serializer\Normalizer;

use App\Entity\Settings\UserSettings;

final class ActorNormalizer extends EfficientItemNormalizer
{

    public function normalize($object, $format = null, array $context = [])
    {
        $context['ignore_depth'] = false;
        $context['circular_reference_limit'] = 1;
        $data = parent::normalize($object, $format, $context);
        if (is_array($data) && !$object instanceof \App\Entity\Custom\Appliance) {
            $userSettingsResource = UserSettings::class;
            $userSettingsIri = $this->iriConverter->getIriFromResourceClass($userSettingsResource);
            $data['@settings'] = sprintf('%s/%s', $userSettingsIri, $data['uuid']);
        }
        return $data;
    }

    public function supportsNormalization($data, $format = null, array $context = []): bool
    {
        return 'jsonld' === $format && 
            ($data instanceof \App\Entity\Actors\Actor || $data instanceof \App\Entity\Custom\Person || $data instanceof \App\Entity\Custom\Appliance);
    }

    /**
     * {@inheritdoc}
     */
    public function supportsDenormalization($data, $type, $format = null, array $context = [])
    {
        return false;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        return $this->normalizer->denormalize($data, $class, $format, $context);
    }

}
