<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Serializer\Normalizer;

use DateTimeImmutable;
use Symfony\Component\Serializer\Normalizer\CacheableSupportsMethodInterface;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;

class DateTimeNormalizer implements NormalizerInterface, DenormalizerInterface, CacheableSupportsMethodInterface
{
    private $normalizer;
    const FORMAT_JSON_LD = 'jsonld';
    const FORMAT_JSON = 'json';


    public function __construct(ObjectNormalizer $normalizer)
    {
        $this->normalizer = $normalizer;
    }

    public function normalize($object, $format = null, array $context = [])
    {
        if (array_key_exists('dateTimeFormat', $context) && $context['dateTimeFormat']) {
            return $object->format($context['dateTimeFormat']);
        }
        return floatval($object->format('U.u'));
    }

    public function supportsNormalization($data, $format = null): bool
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && $data instanceof \DateTime;
    }

    public function hasCacheableSupportsMethod(): bool
    {
        return true;
    }

    public function supportsDenormalization($data, $type, $format = null)
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && 'DateTime' === $type;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        if (!$data) {
            return null;
        }
        $clone_value = $data;
        $date = DateTimeImmutable::createFromFormat('U.u', $data);
        if (!$date) {
            $date = \DateTime::createFromFormat('U', $clone_value);
        }
        return $date;
    }
}
