<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Serializer\Normalizer;

use Symfony\Component\Serializer\Normalizer\CacheableSupportsMethodInterface;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;

class FileNormalizer implements NormalizerInterface, DenormalizerInterface, CacheableSupportsMethodInterface
{
    const FORMAT_JSON_LD = 'jsonld';
    const FORMAT_JSON = 'json';

    protected $iriConverter;

    public function __construct(ObjectNormalizer $normalizer)
    {
        $this->normalizer = $normalizer;
    }

    public function normalize($object, $format = null, array $context = array()): array
    {
        $data = [];
        if (array_key_exists('include_file_data', $context) && $context['include_file_data'] === true) {
            $file = $this->iriConverter->getItemFromIri($object, true);
            if (is_string($file->getFile())) {
                $data['data'] = $file->getFile();
            } elseif (get_resource_type($file->getFile()) == "stream") {
                $data['data'] = stream_get_contents($file->getFile());
            }
        }
        return $data;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        throw new \Exception('Uploading files are not supported via this method.');
    }

    public function supportsNormalization($data, $format = null): bool
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && is_string($data)  && strpos($data, 'api/3/pgfile');
    }

    public function supportsDenormalization($data, $type, $format = null): bool
    {
        return false;
    }

    public function hasCacheableSupportsMethod(): bool
    {
        return true;
    }
}
