<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use App\Entity\Actors\Actor;
use App\Constants\CacheConstants;
use App\Entity\Authorization\Team;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class AccessibleTeamIdsProvider extends AbstractUserDataProvider
{

    /** @var TagAwareCacheInterface */
    private $cache;

    public function __construct(TokenStorageInterface $tokenStorage, TagAwareCacheInterface $cache)
    {
        $this->cache = $cache;
        parent::__construct($tokenStorage);
    }

    public function appendChildTeams($accessibleTeamIds, $childTeam)
    {
        $accessibleTeamIds[$childTeam->getUuid()] = true;
        return $accessibleTeamIds;
    }

    public function getAllChild($childTeams, $accessibleTeamIds)
    {
        foreach ($childTeams as $childTeam) {
            if (array_key_exists($childTeam->getUuid(), $accessibleTeamIds) == false) {
                $accessibleTeamIds = $this->appendChildTeams($accessibleTeamIds, $childTeam);
                if (count($childTeam->getChildren())) {
                    $accessibleTeamIds = $this->getAllChild($childTeam->getChildren(), $accessibleTeamIds);
                    continue;
                }
            }
        }
        return $accessibleTeamIds;
    }

    public function getAllParent($parentTeams, $accessibleTeamIds)
    {
        foreach ($parentTeams as $parentTeam) {
            if (array_key_exists($parentTeam->getUuid(), $accessibleTeamIds) == false) {
                $accessibleTeamIds = $this->appendChildTeams($accessibleTeamIds, $parentTeam);
                if (count($parentTeam->getParents())) {
                    $accessibleTeamIds = $this->getAllParent($parentTeam->getParents(), $accessibleTeamIds);
                    continue;
                }
            }
        }
        return $accessibleTeamIds;
    }

    /**
     * @return array
     */
    public function getParentDataForTeam($teams)
    {
        $accessibleTeamIds = [];
        foreach ($teams as $team) {
            $item = $this->cache->getItem("_parent_teams_" . $team->getUuid());
            if ($item->isHit()) {
                $accessibleTeamIds[] = $item->get();
            }
            $accessibleTeamIds[$team->getUuid()] = true;

            /** @var Team[] $parentTeams */
            $parentTeams = $team->getParents();
            $accessibleTeamIds = $this->getAllParent($parentTeams, $accessibleTeamIds);

            /** @var Team[] $siblingTeams */
            $siblingTeams = $team->getSiblings();
            foreach ($siblingTeams as $siblingTeam) {
                $accessibleTeamIds[$siblingTeam->getUuid()] = true;
            }
            $item->set(array_keys($accessibleTeamIds));
            $item->expiresAfter(600);
            $item->tag(CacheConstants::ACCESSIBLE_PARENT_TEAMS);
            $this->cache->save($item);
        }
        return array_keys($accessibleTeamIds);
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForActor(Actor $actor)
    {
        $username = $actor->getUuid();
        $item = $this->cache->getItem("_accessible_teams_" . $username);
        if ($item->isHit()) {
            return $item->get();
        }
        $accessibleTeamIds = [];
        /** @var Team[] $teams */
        $teams = $actor->getTeams()->toArray();

        foreach ($teams as $team) {
            $accessibleTeamIds[$team->getUuid()] = true;

            /** @var Team[] $childTeams */
            $childTeams = $team->getChildren();
            $accessibleTeamIds = $this->getAllChild($childTeams, $accessibleTeamIds);

            /** @var Team[] $siblingTeams */
            $siblingTeams = $team->getSiblings();
            foreach ($siblingTeams as $siblingTeam) {
                $accessibleTeamIds[$siblingTeam->getUuid()] = true;
            }
        }
        $accessibleTeamIds = array_keys($accessibleTeamIds);
        $item->set($accessibleTeamIds);
        // Don't hardcode
        $item->expiresAfter(600);
        $item->tag(CacheConstants::ACCESSIBLE_CHILD_TEAMS);
        $this->cache->save($item);
        return $item->get();
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForToken(TokenInterface $token)
    {
        return parent::getDataForToken($token);
    }

    /**
     * @inheritdoc
     */
    public function getDataForCurrentUser()
    {
        return parent::getDataForCurrentUser();
    }
}
