<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Command\ExportJobCommand;
use App\Entity\Core\ExportJob;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Class ExportService
 * @package App\Service
 */
class ExportService extends ConfigPorter
{
    /**
     * 
     *
     * @param array $fullConfig
     * @param ExportJob $exportJob
     * @return mixed
     */

    /** @var InputInterface $input */
    private $input;


    public function exportConfiguration(ExportJob $exportJob, ExportJobCommand $exportJobCommand, SymfonyStyle $io, $tmpDirectory, $tokenType = 'JWT', $useTgz = true)
    {
        $options = $exportJob->getTemplate()->getOptions();
        $exportData = [];
        $currentCount = 0;
        $totalCount = count($this->configTypes) + 2; // 2 for zip compress and file upload tasks
        foreach ($this->configTypes as $key => $service) {
            $config = array_key_exists($key, $options) ? $options[$key] : [];
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $io->writeln(sprintf('Working on %s', $key));
            $progressPercent = (int) (($currentCount / $totalCount) * 100);
            $exportJobCommand->updateExportJob($exportJob, null, $progressPercent, null, null, $key);
            $portingConfig->setIO($io);
            $portingConfig->setExportJob($exportJob);
            if ($key == 'modules' && !empty($config)) {
                // Added for backwards compatibility
                $exportData['includeMmd'] = true;
            }
            if ($useTgz) {
                $exportData[$key] = $portingConfig->exportFile($config, $key, $tmpDirectory, $tokenType);
            } else {
                $exportData[$key] = $portingConfig->export($config, $tokenType);
            }
            $currentCount = $currentCount + 1;
        }

        return $exportData;
    }

    public function exportOptionGenerationFromInputOptions($options, $tokenType = 'JWT')
    {
        $exportData = [];
        foreach ($this->configTypes as $key => $service) {
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            if (!empty($options[$key])) {
                $exportData[$key] = $portingConfig->generateExportOptions($options[$key], $tokenType);
            } else {
                $exportData[$key] = [];
            }
        }
        return $exportData;
    }
}
