<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use App\Traits\Ownable;
use App\Traits\UserOwnable;
use App\Traits\ConditionalOwnable;


class OwnershipChecker
{
    /**
     * @param object $object
     * @return bool
     */
    public function isObjectOwnable($object)
    {
        return $this->isClassOwnable(get_class($object));
    }

    /**
     * @param string $class
     * @return bool
     */
    public function isClassOwnable($class)
    {
        return in_array(Ownable::class, $this->class_uses_deep($class));
    }

    /**
     * @param string $class
     * @return bool
     */
    public function isClassConditionalOwnable($class)
    {
        return in_array(ConditionalOwnable::class, $this->class_uses_deep($class));
    }

    /**
     * @param string $class
     * @return bool
     */
    public function isClassUserOwnable($class)
    {
        return in_array(UserOwnable::class, $this->class_uses_deep($class));
    }

    /**
     * @param $class
     * @param bool $autoload
     *
     * @see http://php.net/manual/en/function.class-uses.php
     *
     * @return array
     */
    function class_uses_deep($class, $autoload = true)
    {
        $traits = [];
        do {
            $traits = array_merge(class_uses($class, $autoload), $traits);
        } while ($class = get_parent_class($class));
        foreach ($traits as $trait) {
            $traits = array_merge(class_uses($trait, $autoload), $traits);
        }
        return array_unique($traits);
    }
}
