<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;


use Ramsey\Uuid\Uuid;
use App\Entity\Actors\Actor;
use Doctrine\ORM\EntityManager;
use App\Entity\Authorization\Role;
use App\Entity\Authorization\Team;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class SamlActorService
{

    protected $container;

    /** @var EntityManager */
    protected $entityManager;

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var ProxyRepository */
    protected $proxyRepository;

    const SAML_DEFAULT_FIRST_NAME = 'Change';
    const SAML_DEFAULT_LAST_NAME = 'Me';

    public function __construct(TokenStorageInterface $tokenStorage, IriConverterInterface $iriConverter, ContainerInterface $container)
    {
        $this->tokenStorage = $tokenStorage;
        $this->iriConverter = $iriConverter;
        $this->container = $container;
        $this->entityManager = $this->container->get('doctrine.orm.entity_manager');
        $this->proxyRepository = $this->entityManager->getRepository(Actor::class);
    }

    /**
     * @return string
     */
    public function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        return $this->iriConverter->getIriFromItem($user);
    }


    /**
     * null
     */
    public function updateRBAC(&$actor, $rolesAttribute, $teamsAttributes)
    {
        $existingTeams = $actor->getTeams();
        foreach ($existingTeams as $value) {
            $actor->removeTeam($value);
        }

        $existingRoles = $actor->getRoles(true);
        foreach ($existingRoles as $value) {
            $actor->removeRole($value);
        }

        foreach ($teamsAttributes as $teamId) {
            $team = $this->getTeam($teamId);
            if ($team !== null) {
                $actor->addTeam($team);
            }
        }

        foreach ($rolesAttribute as $roleId) {
            $role = $this->getRole($roleId);
            if ($role !== null) {
                $actor->addRole($role);
            }
        }
    }



    /**
     *
     * @param $className string
     * @param mixed $actorBody
     * @return boolean
     *
     */
    public function saveSamlUser($className, $actorBody, $userType='SAML')
    {
        $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
        $userAttributes = $actorBody;
        $overrideUserAttribute = $actorBody["overrideUserAttribute"];
        $overrideRbacFlag = $userAttributes['overrideRbac'];
        if (array_key_exists('roles', $userAttributes)) {
            $rolesAttribute = $userAttributes['roles'];
        }
        if (array_key_exists('teams', $userAttributes)) {
            $teamsAttributes = $userAttributes['teams'];
        }
        $userId = null;
        if (array_key_exists('userId', $userAttributes)) {
            $userId = $userAttributes['userId'];
        }
        $actor = $this->getActor($userId);
        //Check if user exist
        if ($actor === null) {
            $actor = new $className();
            $actor->setUuid(Uuid::uuid4()->toString());
            $actor->setCreateDate($timestamp);
            $actor->setUserId($userId);
            $actor->setCsActive(true);

            if (array_key_exists('teams', $userAttributes)) {
                $teamIds = $userAttributes['teams'];
                foreach ($teamIds as $teamId) {
                    $team = $this->getTeam($teamId);
                    if ($team !== null) {
                        $actor->addTeam($team);
                    }
                }
            }

            if (array_key_exists('roles', $userAttributes)) {
                $roleIds = $userAttributes['roles'];
                foreach ($roleIds as $roleId) {
                    $role = $this->getRole($roleId);
                    if ($role !== null) {
                        $actor->addRole($role);
                    }
                }
            }
        }

        // if actor exists and overrideUserAttribute setting is false then return from here
        elseif ($overrideUserAttribute === false ) {
            #this takes care of conversion of non saml user into saml user when overrideUserAttribute is disabled
            if ($actor->getUserType() !== $userType){
                $actor->setUserType($userType);
                $this->entityManager->persist($actor);
                $this->entityManager->flush();
            }
            # If sync SAML roles only instruction is recieved
            if($overrideRbacFlag === true){
                $this->updateRBAC($actor, $rolesAttribute, $teamsAttributes);
                $this->entityManager->persist($actor);
                $this->entityManager->flush();
            }
            return true;
        }

        $actor->setUserType($userType);

        $userAttributes = $userAttributes['attributes'];
        if (array_key_exists('firstName', $userAttributes)) {
            $actor->setFirstname($userAttributes['firstName']);
        } else {
            $actor->setFirstname($userId);
        }
        if (array_key_exists('lastName', $userAttributes)) {
            $actor->setLastname($userAttributes['lastName']);
        }
        if (array_key_exists('email', $userAttributes)) {
            $actor->setEmail($userAttributes['email']);
        }
        if (array_key_exists('title', $userAttributes)) {
            $actor->setTitle($userAttributes['title']);
        }
        if (array_key_exists('department', $userAttributes)) {
            $actor->setDepartment($userAttributes['department']);
        }
        if (array_key_exists('description', $userAttributes)) {
            $actor->setDescription($userAttributes['description']);
        }
        if (array_key_exists('phoneFax', $userAttributes)) {
            $actor->setPhoneFax($userAttributes['phoneFax']);
        }
        if (array_key_exists('phoneHome', $userAttributes)) {
            $actor->setPhoneHome($userAttributes['phoneHome']);
        }
        if (array_key_exists('phoneMobile', $userAttributes)) {
            $actor->setPhoneMobile($userAttributes['phoneMobile']);
        }
        if (array_key_exists('phoneWork', $userAttributes)) {
            $actor->setPhoneWork($userAttributes['phoneWork']);
        }
        if (array_key_exists('accessType', $userAttributes)) {
            $actor->setAccessType($userAttributes['accessType']);
        }

        if ($overrideRbacFlag) {
            $this->updateRBAC($actor, $rolesAttribute, $teamsAttributes);
        }

        $this->entityManager->persist($actor);
        $this->entityManager->flush();
        return true;
    }

    /**
     *
     * @param String $userId
     * @return Actor $actor
     */
    public function getActor($userId)
    {
        $criteria = array('userId' => $userId);
        return $this->proxyRepository->findOneBy($criteria);
    }
    /**
     *
     * @param String $teamId
     * @return Team $team
     */
    private function getTeam($teamId)
    {
        $teamRepository = $this->entityManager->getRepository(Team::class);
        return $teamRepository->find($teamId);
    }

    /**
     *
     * @param String $roleId
     * @return Role $role
     */
    private function getRole($roleId)
    {
        $roleRepository = $this->entityManager->getRepository(Role::class);
        return $roleRepository->find($roleId);
    }
}
