<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use Doctrine\ORM\EntityManager;
use App\Entity\ViewTemplate\UserPreference;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class UserPreferenceService
{

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;

    /** @var ContainerInterface */
    protected $containers;

    public function __construct(TokenStorageInterface $tokenStorage, IriConverterInterface $iriConverter, ContainerInterface $containers)
    {
        $this->tokenStorage = $tokenStorage;
        $this->iriConverter = $iriConverter;
        $this->containers = $containers;
        $this->entityManager = $this->containers->get('doctrine.orm.entity_manager');
    }

    /** @var SerializerInterface */
    protected $serializer;

    /**
     * @param SerializerInterface $serializer
     */
    public function setSerializer($serializer)
    {
        $this->serializer = $serializer;
    }

    /**
     * * @param array $userPrefBody
     * @return UserPreference $userPref
     *
     */
    public function saveUserPref($userPrefBody)
    {
        $userPref = $this->getUserPref();
        if (count($userPref) && !is_null($userPref)) {
            $userPref->setPreferences($userPrefBody);
        } else {
            $userPref = new UserPreference();
            $userPref->setUserId($this->getCurrentUserAsIri());
            $userPref->setPreferences($userPrefBody);
        }
        $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));
        $userPref->setUpdateDate($timestamp);

        $this->entityManager->persist($userPref);
        $this->entityManager->flush();
        return $userPref;
    }

    /**
     * @return UserPreference
     */
    public function getUserPref()
    {
        /** @var UserPreferenceRepository $userPrefRepository */
        $userPrefRepository = $this->entityManager->getRepository(UserPreference::class);
        $userId = $this->getCurrentUserAsIri();
        $template = $userPrefRepository->findBy(["userId" => $userId]);
        if (count($template)) {
            $template = $template[0];
        }
        return $template;
    }

    /**
     * @return string
     */
    private function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        return $this->iriConverter->getIriFromItem($user);
    }
}
