<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use ApiPlatform\Core\Api\IriConverterInterface;
use App\Entity\ViewTemplate\SystemViewTemplate;
use App\Entity\ViewTemplate\UserViewTemplate;
use Doctrine\ORM\EntityManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\SerializerInterface;

class ViewTemplateService
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;

    /** @var SerializerInterface */
    protected $serializer;

    protected $containers;

    public function __construct(TokenStorageInterface $tokenStorage, IriConverterInterface $iriConverter, ContainerInterface $containers)
    {
        $this->tokenStorage = $tokenStorage;
        $this->iriConverter = $iriConverter;
        $this->containers = $containers;
        $this->entityManager = $this->containers->get('doctrine.orm.entity_manager');
    }

    /**
     * @return string
     */
    private function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        return $this->iriConverter->getIriFromItem($user);
    }

    /**
     * @param $name
     *
     * @return SystemViewTemplate|UserViewTemplate
     */
    public function getViewTemplate($name)
    {
        /** @var UserViewTemplateRepository $userViewTemplateRepository */
        $userViewTemplateRepository = $this->entityManager->getRepository(UserViewTemplate::class);

        $viewTemplate = $this->getUserViewTemplate(
            $userViewTemplateRepository,
            $this->getCurrentUserAsIri(),
            $name
        );

        if (!count($viewTemplate) || is_null($viewTemplate)) {
            /** @var SystemViewTemplateRepository $systemViewTemplateRepository */
            $systemViewTemplateRepository = $this->entityManager->getRepository(SystemViewTemplate::class);

            $viewTemplate = $this->getSystemViewTemplate($systemViewTemplateRepository, $name);
        }

        if (count($viewTemplate)) {
            $viewTemplate = $viewTemplate[0];
        } else {
            throw new NotFoundHttpException();
        }

        return $viewTemplate;
    }

    /**
     * @param UserViewTemplate $viewTemplate
     */
    public function saveViewTemplate($name, $viewTemplate)
    {
        $viewTemplate->setName($name);
        $viewTemplate->setOwner($this->getCurrentUserAsIri());

        $this->entityManager->persist($viewTemplate);
        $this->entityManager->flush();
    }

    /**
     * @param SystemViewTemplate $viewTemplate
     */
    public function saveSystemViewTemplate($name, $viewTemplate)
    {
        $viewTemplate->setUuid($name);

        $this->entityManager->persist($viewTemplate);
        $this->entityManager->flush();
    }

    public function getUserViewTemplate($entityRepository, $user, $name)
    {
        return $entityRepository->findBy(["owner" => $user, "name" => $name]);
    }

    public function getSystemViewTemplate($entityRepository, $name)
    {
        return $entityRepository->findBy(["uuid" => $name]);
    }
}
