<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use App\Service\TeamsProvider;
use App\Entity\Base\BaseEntity;
use App\Traits\ConditionalOwnable;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use ApiPlatform\Core\EventListener\EventPriorities;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

final class ConditionalOwnershipListener implements EventSubscriberInterface
{
    /** @var TeamsProvider */
    protected $teamProvider;

    public function __construct(TeamsProvider $teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['add_owners_to_record', EventPriorities::PRE_WRITE],
        ];
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isInstanceOfConditionalOwnableEntity($object)
    {
        return in_array(ConditionalOwnable::class, class_uses(get_class($object)));
    }

    public function add_owners_to_record(ViewEvent $event): void
    {
        $method = $event->getRequest()->getMethod();
        if (Request::METHOD_GET == $method){
            return;
        }
        $object = $event->getControllerResult();
        if (!($object instanceof BaseEntity)) {
            return;
        }
        if ($this->isInstanceOfConditionalOwnableEntity($object)) {
            $owners = $object->getOwners() ? $object->getOwners() : [];
            $is_private = $object->getIsPrivate() ?? false;

            if (($owners == null || count($owners) < 1) && $is_private) {
                $teams = $this->teamProvider->getDataForCurrentUser();
                if (count($teams) === 0) {
                    throw new AccessDeniedException("A user must be on at least one team in order to create records.");
                }
                foreach ($teams as $team) {
                    $object->addOwner($team);
                }
            } elseif (count($owners) > 0 && $is_private == false) {
                foreach ($owners as $team) {
                    $owners->removeOwner($team);
                }
            }
        }
    }
}
