<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;


use App\Entity\Authorization\Team;
use ApiPlatform\Core\Annotation\ApiSubresource;
use Symfony\Component\Serializer\Annotation\Groups;

trait Ownable
{
    /**
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="App\Entity\Authorization\Team", cascade={"persist"})
     * @Symfony\Component\Validator\Constraints\Count(min=1, minMessage="Unable to perform requested operation. Ensure the record has at least one owner assigned.")
     * @ApiSubresource(maxDepth=1)
     * @Groups({"fsr_all", "owners"})
     */
    protected $owners;

    public function addOwner(Team $owner): self
    {
        if (is_array($this->owners) && empty($this->owners)) {
            unset($this->owners);
            $this->owners = [];
        }
        if (!empty($this->owners)) {
            if (!is_array($this->getOwners()) && isset($this->owners) && get_class($this->owners) == "Doctrine\ORM\PersistentCollection") {
                if (!in_array($owner, $this->owners->getValues())) {
                    $this->owners[] = $owner;
                }
            } elseif (!in_array($owner, $this->owners)) {
                $this->owners[] = $owner;
            }
        } else {
            $this->owners[] = $owner;
        }
        return $this;
    }

    public function removeOwner(Team $owner)
    {
        if (is_array($this->owners) && ($key = array_search($owner, $this->owners)) !== false) {
            unset($this->owners[$key]);
            return;
        }
        $this->owners->removeElement($owner);
    }

    public function getOwners()
    {
        return $this->owners;
    }
}
