<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use App\Traits\Ownable;
use App\Entity\Core\File;
use App\Entity\Custom\Agent;
use App\Entity\Custom\Tenant;
use App\Service\TeamsProvider;
use Dunglas\ApiBundle\Event\DataEvent;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class OwnableListener
{
    /** @var TeamsProvider */
    protected $teamProvider;

    private $excludedEntities = [File::class];

    public function __construct(TeamsProvider $teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isInstanceOfOwnableEntity($object)
    {
        return in_array(Ownable::class, class_uses(get_class($object))) && !in_array(get_class($object), $this->excludedEntities);
    }

    /**
     * @param DataEvent $event
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isInstanceOfOwnableEntity($object)) {
            /** @var Ownable $item */
            $owners = $object->getOwners();
            if ($owners == null || count($owners) < 1) {
                $teams = $this->teamProvider->getDataForCurrentUser();
                if (count($teams) === 0) {
                    throw new AccessDeniedException(
                        "A user must be on at least one team in order to create records."
                    );
                }
                foreach ($teams as $team) {
                    $object->addOwner($team);
                }
            }
        }
    }
}
