<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use App\Constants\AppConstants;
use App\Entity\Core\Tag;
use App\Repository\TagRepository;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;

class TagSubscriber
{

    protected $entityManager;

    /** @var TagRepository */
    protected $tagRepository;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->tagRepository = $this->entityManager->getRepository(Tag::class);
    }

    public function validateAndGetTags($object)
    {
        $tags = [];
        foreach ($object->getRecordTags() as $tag) {
            if (preg_match("/[#,?'\"]/", $tag->getUuid())) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, AppConstants::INVALID_CHAR_IN_TAG);
            }
            $tags[] = $tag->getUuid();
        }
        return $tags;
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if (method_exists($object, 'getRecordTags') && $object->getRecordTags()) {
            $tags = $this->validateAndGetTags($object);
            $this->tagRepository->upsertTag($tags);
        }
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function preUpdate(LifecycleEventArgs $args)
    {
        $data = $args->getObject();
        // $previous = $args->getPrevious();
        if (method_exists($data, 'getRecordTags') && $data->getRecordTags()) {
            $previous_tags = [];
            $tags = $this->validateAndGetTags($data);
            $previousTags = $args->getObject()->getRecordTags()->getSnapshot();
            if (count($previousTags)) {
                foreach ($previousTags as $tag) {
                    $previous_tags[] = $tag->getUuid();
                }
            }
            $diff_tags = array_diff($tags, $previous_tags);
            if (count($diff_tags)) {
                $this->tagRepository->upsertTag($diff_tags);
            }
        }
    }
}
