<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use App\Entity\Core\Tag;
use Symfony\Component\Serializer\Annotation\Groups;
use Doctrine\ORM\Mapping as ORM;

trait Taggable
{
    /**
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="App\Entity\Core\Tag", cascade={"persist"})
     * @ORM\OrderBy({"uuid" = "ASC"})
     * @Groups({"fsr_primary", "fsr_all", "recordTags"})
     */
    protected $recordTags = [];

    public function addRecordTag($recordTag): self
    {
        if (is_array($this->recordTags) && empty($this->recordTags)) {
            unset($this->recordTags);
            $this->recordTags = [];
        }
        if (!empty($this->recordTags)) {
            if (!is_array($this->getRecordTags()) && isset($this->recordTags) && get_class($this->recordTags) == "Doctrine\ORM\PersistentCollection") {
                if (!in_array($recordTag, $this->recordTags->getValues())) {
                    $this->recordTags[] = $recordTag;
                }
            } elseif (!in_array($recordTag, $this->recordTags)) {
                $this->recordTags[] = $recordTag;
            }
        } else {
            $this->recordTags[] = $recordTag;
        }
        return $this;
    }

    public function removeRecordTag($recordTag)
    {
        if (is_array($this->recordTags) && ($key = array_search($recordTag, $this->recordTags)) !== false) {
            unset($this->recordTags[$key]);
            return;
        }
        $this->recordTags->removeElement($recordTag);
    }

    public function getRecordTags()
    {
        if (is_null($this->recordTags)) {
            return [];
        }
        return $this->recordTags;
    }
}
