<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use App\Traits\Trackable;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class TrackableListener
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    protected $container;

    protected $requestStack;

    public function __construct(TokenStorageInterface $tokenStorage, ContainerInterface $container, RequestStack $requestStack)
    {
        $this->tokenStorage = $tokenStorage;
        $this->container = $container;
        $this->requestStack = $requestStack;
    }

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isTrackable($object)) {
            list($user, $now) = $this->getTrackableInfo();
            /** @var Trackable $object */
            $object->setCreateDate($now);
            $object->setModifyDate($now);
            if (!is_null($user)) {
                $object->setCreateUser($user);
                $object->setModifyUser($user);
            }
        }
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isTrackable($object)
    {
        if ($this->requestStack->getCurrentRequest() && $this->requestStack->getCurrentRequest()->attributes->get('fsr_subrequest_type') === 'import') {
            return false;
        }
        return in_array(Trackable::class, class_uses(get_class($object)));
    }

    /**
     * @return array
     */
    private function getTrackableInfo()
    {
        $userToken = $this->tokenStorage->getToken();
        $user = null;
        if (!is_null($userToken)) {
            $user = $userToken->getUser();
        }
        $now = new \DateTime('NOW', new \DateTimeZone('UTC'));
        return [$user, $now];
    }
}