<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Traits;

use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Annotation\ApiSubresource;

trait UserOwnable
{
    /**
     * @Doctrine\ORM\Mapping\Column(name="`assignee`", type="string", nullable=true, unique=false)
     * @Groups({"fsr_primary", "fsr_all", "assignee"})
     */
    protected $assignee='';

    public function getAssignee()
    {
        return $this->assignee;
    }

    public function setAssignee($assignee)
    {
        $this->assignee = $assignee;
    }

    /**
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="App\Entity\Actors\Actor", cascade={"detach"})
     * @ApiSubresource(maxDepth=1)
     * @Groups({"fsr_all", "userOwners"})
     */
    protected $userOwners;

    /**
     * Add userOwner
     *
     * @param Actor $userOwner
     * @return self
     */
    public function addUserOwner($userOwner)
    {
        if (is_array($this->userOwners) && empty($this->userOwners)) {
            unset($this->userOwners);
            $this->userOwners = [];
        }
        if (!empty($this->userOwners)) {
            if (!is_array($this->getUserOwners()) && isset($this->userOwners) && get_class($this->userOwners) == "Doctrine\ORM\PersistentCollection") {
                if (!in_array($userOwner, $this->userOwners->getValues())) {
                    $this->userOwners[] = $userOwner;
                }
            } elseif (!in_array($userOwner, $this->userOwners)) {
                $this->userOwners[] = $userOwner;
            }
        } else {
            $this->userOwners[] = $userOwner;
        }
        return $this;
    }

    /**
     * Remove userOwner
     *
     * @param Actor $userOwner
     */
    public function removeUserOwner($userOwner)
    {
        if (is_array($this->userOwners) && ($key = array_search($userOwner, $this->userOwners)) !== false) {
            unset($this->userOwners[$key]);
            return;
        }
        $this->userOwners->removeElement($userOwner);
    }

    /**
     * Get userOwners
     *
     * @return Actor[]
     */
    public function getUserOwners()
    {
        return $this->userOwners;
    }
}
