<?php
/* Copyright start
  Copyright (C) 2008 - 2023 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Validator;

use Doctrine\ORM\EntityManagerInterface;
use App\Entity\ModelMetadata\ModelMetadata;
use Symfony\Component\Validator\Constraint;
use App\Validator\DescriptionNameConstraint;
use App\Entity\ModelMetadata\StagingModelMetadata;
use Symfony\Component\Validator\ConstraintValidator;

class DescriptionNameConstraintValidator extends ConstraintValidator
{
    protected $entityManager;

    private const RESERVE_MODULE = ['people', 'appliances', 'agents', 'approvals', 'tenants', 'routers', 'comments', 'saved_reports'];
    private const RESERVE_NON_PEER_REPLICABLE_MODULE = ['people', 'appliances', 'agents', 'tenants', 'routers', 'threat_intel_feeds'];
    private const RESERVE_NON_ARCHIVABLE_MODULE = ['people', 'appliances', 'agents', 'approvals', 'tenants', 'routers', 'saved_reports'];

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function validate($value, Constraint $constraint)
    {
        /* @var $constraint \App\Validator\SingularName */
        if (null === $value || '' === $value || !$constraint instanceof DescriptionNameConstraint) {
            return;
        }

        if ($value->getArchivable() && !$value->getTrackable()) {
            $this->context->buildViolation($constraint::NON_ARCHIVABLE_MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getType()))
                    ->addViolation();
        } else if ($value->getSoftDeleteable() && in_array($value->getType(), self::RESERVE_MODULE)) {
            $this->context->buildViolation($constraint::RESERVE_NON_RECYCLEABLE_MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getType()))
                    ->addViolation();
        } else if ($value->getArchivable() && in_array($value->getType(), self::RESERVE_NON_ARCHIVABLE_MODULE)) {
            $this->context->buildViolation($constraint::RESERVE_NON_ARCHIVABLE_MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getType()))
                    ->addViolation();
        } else if ($value->getPeerReplicable() && in_array($value->getType(), self::RESERVE_NON_PEER_REPLICABLE_MODULE)) {
            $this->context->buildViolation($constraint::RESERVE_NON_PEER_REPLICABLE_MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getType()))
                    ->addViolation();
        }
        $modelMetadataRepository = $this->entityManager->getRepository(StagingModelMetadata::class);
        $stagingData = $modelMetadataRepository->findAll();
        foreach ($stagingData as $modelMetadata) {
            $singularName = $modelMetadata->getDescriptions()['singular'];
            $type = $modelMetadata->getType();
            if ($value->getDescriptions()['singular'] == $singularName && $value->getType() != $type) {
                $this->context->buildViolation($constraint::MESSAGE)
                    ->setParameter('{{ name }}', $this->formatValue($value->getDescriptions()['singular']))
                    ->addViolation();
                break;
            }
        }
    }
}
